/*
 * wxcitems.cc
 * wxWindows command processing: panel items
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#include "wx_hash.h"
#include "wx_types.h"
#include "wx_ipc.h"

#if !defined(USE_IOSTREAMH) || USE_IOSTREAMH
#include <iostream.h>
#else
#include <iostream>
#endif
#include <ctype.h>
#include <stdlib.h>

#include "wx_cmds.h"
#include "wxcmdsp.h"

#ifdef USE_CLIPS
#ifndef __WATCOMC__
#include "clips.h"
#endif
#endif

/************************************************************************
 * GENERIC PANEL ITEM FUNCTIONS
 *
 ************************************************************************/

#ifdef USE_CLIPS
long clipsPanelItemSetDefault()
{
  if (ArgCountCheck("clipsPanelItemSetDefault", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);

  return wxcPanelItemSetDefault(id);
}

long clipsPanelItemSetLabel()
{
  if (ArgCountCheck("clipsPanelItemSetLabel", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  char *label = RtnLexeme(2);
  return wxcPanelItemSetLabel(id, label);
}

char *clipsPanelItemGetLabel()
{
  if (ArgCountCheck("clipsPanelItemSetLabel", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");

  long id = RtnLong(1);
  char *ans = wxcPanelItemGetLabel(id);
  if (ans)
    return (char *)AddSymbol(ans);
  else
    return (char *)AddSymbol("");
}
#endif

/************************************************************************
 * PANEL ITEM CALLBACK
 * Only one generic callback function required.
 *
 ************************************************************************/

static long wxClipsCommandEventId = 0;
void wxClipsItemCallback(Bool *handled, WXID id, WXID eventId)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnCommand");
  if (func)
  {
    wxClipsCommandEventId = eventId;
    *handled = TRUE;
#ifdef USE_CLIPS
    char buf[100];
    sprintf(buf, "(%s %ld)", func, id);
    wxRouteNoEcho(buf);
#else
    (* (void (*)(long)) func) (id);
#endif
    wxClipsCommandEventId = 0;
  }
}

// Function to retrieve the current command event id since it's
// not passed to the callback.

#ifdef USE_CLIPS
long clipsPanelItemGetCommandEvent()
{
  if (ArgCountCheck("clipsPanelItemGetCommandEvent", EXACTLY, 0) == -1)
    return 0;

  return wxClipsCommandEventId;
}
#endif

/************************************************************************
 * PANEL ITEM FUNCTIONS
 *
 ************************************************************************/

// Create a button.
// Note that we have to overload creation functions such as this
// to be compatible with what wxCLIPS expects: i.e. a language-specific
// callback parameter (wxClipsFunction func). We implement
// this in terms of the more generic creation function, and
// register the callback as an OnCommand event.

WXID wxcButtonCreate(WXID parentId, wxClipsFunction func, char *label,
  int x, int y, int width, int height, char *styleString, char *name)
{
#ifdef USE_CLIPS
  if (func && (strlen(func) == 0))
    func = NULL;
#endif
  // Call the generic function in wxextend library
  WXID button = wxcButtonCreate(parentId, label, x, y, width, height, styleString, name);
  if (func)
    wxcObjectAddCallback(button, "OnCommand", func);
  return button;
}

#ifdef USE_CLIPS
long clipsButtonCreate()
{
  if (ArgCountCheck("clipsButtonCreate", AT_LEAST, 3) == -1)
    return 0;

  int no_args = RtnArgCount();

  WXID parentId = RtnLong(1);
  char *func = RtnLexeme(2);

  if (func && (strlen(func) == 0))
    func = NULL;

  char *label = RtnLexeme(3);
  if (!label)
    return -1;

  // Set up defaults
  int x = -1;
  int y = -1;
  int width = -1;
  int height = -1;
  char *style = "";
  char *name = "button";

  if (no_args > 3)
    x = (int) RtnLong(4);

  if (no_args > 4)
    y = (int) RtnLong(5);

  if (no_args > 5)
    width = (int) RtnLong(6);

  if (no_args > 6)
    height = (int) RtnLong(7);

  if (no_args > 7)
    style = RtnLexeme(8);

  if (no_args > 8)
    name = RtnLexeme(9);

  return wxcButtonCreate(parentId, func, label, x, y, width, height, style, name);
}
#endif

// Create a button from a bitmap
WXID wxcButtonCreateFromBitmap(WXID parentId, wxClipsFunction func, WXID bitmapId,
  int x, int y, int width, int height, char *styleString, char *name)
{
#ifdef USE_CLIPS
  if (func && (strlen(func) == 0))
    func = NULL;
#endif
  // Call the generic function in wxextend library
  WXID button = wxcButtonCreateFromBitmap(parentId, bitmapId, x, y, width, height, styleString, name);
  if (func)
    wxcObjectAddCallback(button, "OnCommand", func);
  return button;
}

#ifdef USE_CLIPS
long clipsButtonCreateFromBitmap()
{
  if (ArgCountCheck("clipsButtonCreateFromBitmap", AT_LEAST, 3) == -1)
    return 0;

  int no_args = RtnArgCount();

  WXID parentId = RtnLong(1);
  char *func = RtnLexeme(2);

  if (func && (strlen(func) == 0))
    func = NULL;

  long bitmapId = RtnLong(3);

  // Set up defaults
  int x = -1;
  int y = -1;
  int width = -1;
  int height = -1;
  char *style = "";
  char *name = "button";

  if (no_args > 3)
    x = (int) RtnLong(4);

  if (no_args > 4)
    y = (int) RtnLong(5);

  if (no_args > 5)
    width = (int) RtnLong(6);

  if (no_args > 6)
    height = (int) RtnLong(7);

  if (no_args > 7)
    style = RtnLexeme(8);

  if (no_args > 8)
    name = RtnLexeme(9);

  return wxcButtonCreateFromBitmap(parentId, func, bitmapId, x, y, width, height, style, name);
}
#endif

// Create a text widget
WXID wxcTextCreate(WXID parentId, wxClipsFunction func, char *label, char *value,
  int x, int y, int width, int height, char *styleString, char *name)
{
#ifdef USE_CLIPS
  if (func && (strlen(func) == 0))
    func = NULL;
#endif
  // Call the generic function in wxextend library
  WXID id = wxcTextCreate(parentId, label, value, x, y, width, height, styleString, name);
  if (func)
    wxcObjectAddCallback(id, "OnCommand", func);
  return id;
}

#ifdef USE_CLIPS
long clipsTextCreate()
{
  if (ArgCountCheck("clipsTextCreate", AT_LEAST, 3) == -1)
    return 0;

  int no_args = RtnArgCount();

  WXID parentId = RtnLong(1);
  char *func = RtnLexeme(2);

  if (func && (strlen(func) == 0))
    func = NULL;

  char *label = RtnLexeme(3);
  if (!label)
    return -1;

  // Set up defaults
  int x = -1;
  int y = -1;
  int width = -1;
  int height = -1;
  char *value = "";
  char *style = "";
  char *name = "text";

  if (no_args > 3)
    value = RtnLexeme(4);
  if (!value)
    return -1;

  if (no_args > 4)
    x = (int) RtnLong(5);

  if (no_args > 5)
    y = (int) RtnLong(6);

  if (no_args > 6)
    width = (int) RtnLong(7);

  if (no_args > 7)
    height = (int) RtnLong(8);

  if (no_args > 8)
    style = RtnLexeme(9);
    
  if (no_args > 9)
    name = RtnLexeme(10);
    
  return wxcTextCreate(parentId, func, label, value, x, y, width, height, style, name);
}

long clipsTextSetValue()
{
  if (ArgCountCheck("clipsTextSetValue", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  char *val = RtnLexeme(2);
  return wxcTextSetValue(id, val);
}

char *clipsTextGetValue()
{
  if (ArgCountCheck("clipsTextGetValue", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");

  long id = RtnLong(1);
  char *ans = wxcTextGetValue(id);
  if (ans)
    return (char *)AddSymbol(ans);
  else
    return (char *)AddSymbol("");
}
#endif

// Create a multi-line text widget
WXID wxcMultiTextCreate(WXID parentId, wxClipsFunction func, char *label, char *value,
  int x, int y, int width, int height, char *styleString, char *name)
{
#ifdef USE_CLIPS
  if (func && (strlen(func) == 0))
    func = NULL;
#endif
  // Call the generic function in wxextend library
  WXID id = wxcMultiTextCreate(parentId, label, value, x, y, width, height, styleString, name);
  if (func)
    wxcObjectAddCallback(id, "OnCommand", func);
  return id;
}

#ifdef USE_CLIPS
long clipsMultiTextCreate()
{
  if (ArgCountCheck("clipsMultiTextCreate", AT_LEAST, 3) == -1)
    return 0;

  int no_args = RtnArgCount();

  WXID parentId = RtnLong(1);
  char *func = RtnLexeme(2);
  char *label = RtnLexeme(3);
  // Set up defaults
  int x = -1;
  int y = -1;
  int width = -1;
  int height = -1;
  char *value = "";
  char *style = "";
  char *name = "multiText";

  if (no_args > 3)
    value = RtnLexeme(4);
  if (!value)
    return -1;

  if (no_args > 4)
    x = (int) RtnLong(5);

  if (no_args > 5)
    y = (int) RtnLong(6);

  if (no_args > 6)
    width = (int) RtnLong(7);

  if (no_args > 7)
    height = (int) RtnLong(8);

  if (no_args > 8)
    style = RtnLexeme(9);

  if (no_args > 9)
    name = RtnLexeme(10);

  return wxcMultiTextCreate(parentId, func, label, value, x, y, width, height, style, name);
}

long clipsMultiTextSetValue()
{
  if (ArgCountCheck("clipsMultiTextSetValue", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  char *val = RtnLexeme(2);
  return wxcMultiTextSetValue(id, val);
}

char *clipsMultiTextGetValue()
{
  if (ArgCountCheck("clipsMultiTextGetValue", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");

  long id = RtnLong(1);
  char *ans = wxcMultiTextGetValue(id);
  if (ans)
    return (char *)AddSymbol(ans);
  else
    return (char *)AddSymbol("");
}

/* Windows only, so far.
 */
 
#ifdef wx_msw
VOID clipsMultiTextWrite()
{
  if (ArgCountCheck("clipsMultiTextWrite", AT_LEAST, 1) == -1)
    return;

  long id = RtnLong(1);
  char *s = RtnLexeme(2);
  if (!s)
    return;
    
  wxcMultiTextWrite(id, s);
}

long clipsMultiTextCopy()
{
  if (ArgCountCheck("clipsMultiTextCopy", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return (long)wxcMultiTextCopy(id);
}

long clipsMultiTextPaste()
{
  if (ArgCountCheck("clipsMultiTextPaste", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return (long)wxcMultiTextPaste(id);
}

long clipsMultiTextCut()
{
  if (ArgCountCheck("clipsMultiTextCut", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return (long)wxcMultiTextCut(id);
}

long clipsMultiTextReplace()
{
  if (ArgCountCheck("clipsMultiTextReplace", EXACTLY, 4) == -1)
    return 0;

  long id = RtnLong(1);
  long from = RtnLong(2);
  long to = RtnLong(3);
  char *text = RtnLexeme(4);
  return (long)wxcMultiTextReplace(id, from, to, text);
}

long clipsMultiTextRemove()
{
  if (ArgCountCheck("clipsMultiTextRemove", EXACTLY, 3) == -1)
    return 0;

  long id = RtnLong(1);
  long from = RtnLong(2);
  long to = RtnLong(3);
  return (long)wxcMultiTextRemove(id, from, to);
}

long clipsMultiTextSetSelection()
{
  if (ArgCountCheck("clipsMultiTextSetSelection", EXACTLY, 3) == -1)
    return 0;

  long id = RtnLong(1);
  long from = RtnLong(2);
  long to = RtnLong(3);
  return (long)wxcMultiTextSetSelection(id, from, to);
}

long clipsMultiTextGetLastPosition()
{
  if (ArgCountCheck("clipsMultiTextGetLastPosition", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return (long)wxcMultiTextGetLastPosition(id);
}

long clipsMultiTextShowPosition()
{
  if (ArgCountCheck("clipsMultiTextShowPosition", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long pos = RtnLong(2);
  return (long)wxcMultiTextShowPosition(id, pos);
}

long clipsMultiTextPositionToLine()
{
  if (ArgCountCheck("clipsMultiTextPositionToLine", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long pos = RtnLong(2);
  return (long)wxcMultiTextPositionToLine(id, pos);
}

long clipsMultiTextPositionToChar()
{
  if (ArgCountCheck("clipsMultiTextPositionToChar", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long pos = RtnLong(2);
  return (long)wxcMultiTextPositionToChar(id, pos);
}

long clipsMultiTextXYToPosition()
{
  if (ArgCountCheck("clipsMultiTextXYToPosition", EXACTLY, 3) == -1)
    return 0;

  long id = RtnLong(1);
  long x = RtnLong(2);
  long y = RtnLong(3);
  return (long)wxcMultiTextXYToPosition(id, x, y);
}

long clipsMultiTextSetInsertionPoint()
{
  if (ArgCountCheck("clipsMultiTextSetInsertionPoint", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long pos = RtnLong(2);
  wxcMultiTextSetInsertionPoint(id, pos);
  return 1;
}

long clipsMultiTextGetInsertionPoint()
{
  if (ArgCountCheck("clipsMultiTextGetInsertionPoint", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcMultiTextGetInsertionPoint(id);
}

long clipsMultiTextGetLineLength()
{
  if (ArgCountCheck("clipsMultiTextGetLineLength", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long line = RtnLong(2);
  return wxcMultiTextGetLineLength(id, line);
}

char *clipsMultiTextGetLineText()
{
  if (ArgCountCheck("clipsMultiTextGetLineText", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long line = RtnLong(2);
  char *s =  wxcMultiTextGetLineText(id, line);
  if (!s)
    return (char *)AddSymbol("");
  else
    return (char *)AddSymbol(s);
}

long clipsMultiTextGetNumberOfLines()
{
  if (ArgCountCheck("clipsMultiTextGetNumberOfLines", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcMultiTextGetNumberOfLines(id);
}
#endif
  // End wx_msw
  
#endif

// Create a check box widget
WXID wxcCheckBoxCreate(WXID parentId, wxClipsFunction func, char *label,
  int x, int y, int width, int height, char *styleString, char *name)
{
#ifdef USE_CLIPS
  if (func && (strlen(func) == 0))
    func = NULL;
#endif
  // Call the generic function in wxextend library
  WXID id = wxcCheckBoxCreate(parentId, label, x, y, width, height, styleString, name);
  if (func)
    wxcObjectAddCallback(id, "OnCommand", func);
  return id;
}

#ifdef USE_CLIPS
long clipsCheckBoxCreate()
{
  if (ArgCountCheck("clipsCheckBoxCreate", AT_LEAST, 2) == -1)
    return 0;

  int no_args = RtnArgCount();

  WXID parentId = RtnLong(1);
  char *func = RtnLexeme(2);
  char *label = RtnLexeme(3);
  // Set up defaults
  int x = -1;
  int y = -1;
  int width = -1;
  int height = -1;
  char *style = "";
  char *name = "checkBox";

  if (no_args > 3)
    x = (int) RtnLong(4);

  if (no_args > 4)
    y = (int) RtnLong(5);

  if (no_args > 5)
    width = (int) RtnLong(6);

  if (no_args > 6)
    height = (int) RtnLong(7);

  if (no_args > 7)
    style = RtnLexeme(8);

  if (no_args > 8)
    name = RtnLexeme(9);

  return wxcCheckBoxCreate(parentId, func, label, x, y, width, height, style, name);
}

long clipsCheckBoxSetValue()
{
  if (ArgCountCheck("clipsCheckBoxSetValue", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  Bool val = (Bool)RtnLong(2);
  return wxcCheckBoxSetValue(id, val);
}

long clipsCheckBoxGetValue()
{
  if (ArgCountCheck("clipsCheckBoxGetValue", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return (long)wxcCheckBoxGetValue(id);
}
#endif

#ifdef USE_CLIPS
long clipsGroupBoxCreate()
{
  if (ArgCountCheck("clipsGroupBoxCreate", AT_LEAST, 6) == -1)
    return 0;

  int no_args = RtnArgCount();

  WXID parentId = RtnLong(1);

  char *label = RtnLexeme(2);
  if (!label)
    return -1;

  // Set up defaults
  int x = -1;
  int y = -1;
  int width = -1;
  int height = -1;
  char *style = "";
  char *name = "name";

  x = (int) RtnLong(3);
  y = (int) RtnLong(4);
  width = (int) RtnLong(5);
  height = (int) RtnLong(6);

  if (no_args > 6)
    style = RtnLexeme(7);
    
  if (no_args > 7)
    name = RtnLexeme(8);
    
  return wxcGroupBoxCreate(parentId, label, x, y, width, height, style, name);
}
#endif

// Create a slider widget
WXID wxcSliderCreate(WXID parentId, wxClipsFunction func, char *label,
  int value, int min_value, int max_value, int width, int x, int y, char *styleString, char *name)
{
#ifdef USE_CLIPS
  if (func && (strlen(func) == 0))
    func = NULL;
#endif
  // Call the generic function in wxextend library
  WXID id = wxcSliderCreate(parentId, label, value, min_value, max_value,
    width, x, y, styleString, name);
  if (func)
    wxcObjectAddCallback(id, "OnCommand", func);
  return id;
}

#ifdef USE_CLIPS
long clipsSliderCreate()
{
  if (ArgCountCheck("clipsSliderCreate", AT_LEAST, 7) == -1)
    return 0;

  int no_args = RtnArgCount();

  WXID parentId = RtnLong(1);
  char *func = RtnLexeme(2);
  char *label = RtnLexeme(3);
  int value = (int)RtnLong(4);
  int min_value = (int)RtnLong(5);
  int max_value = (int)RtnLong(6);
  int width = (int)RtnLong(7);
  char *style = "";
  char *name = "slider";

  // Set up defaults
  int x = -1;
  int y = -1;

  if (no_args > 7)
    x = (int) RtnLong(8);

  if (no_args > 8)
    y = (int) RtnLong(9);

  if (no_args > 9)
    style = RtnLexeme(10);

  if (no_args > 10)
    name = RtnLexeme(11);

  return wxcSliderCreate(parentId, func, label, value, min_value, max_value,
    width, x, y, style, name);
}

long clipsSliderSetValue()
{
  if (ArgCountCheck("clipsSliderSetValue", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long val = RtnLong(2);
  return (long)wxcSliderSetValue(id, (int)val);
}

long clipsSliderGetValue()
{
  if (ArgCountCheck("clipsSliderGetValue", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return (long)wxcSliderGetValue(id);
}

long clipsMessageCreate()
{
  if (ArgCountCheck("clipsMessageCreate", AT_LEAST, 2) == -1)
    return 0;

  int no_args = RtnArgCount();

  WXID parentId = RtnLong(1);
  char *label = RtnLexeme(2);

  // Set up defaults
  int x = -1;
  int y = -1;
  char *style = "";
  char *name = "message";

  if (no_args > 2)
    x = (int) RtnLong(3);

  if (no_args > 3)
    y = (int) RtnLong(4);

  if (no_args > 4)
    style = RtnLexeme(5);

  if (no_args > 5)
    name = RtnLexeme(6);

  return wxcMessageCreate(parentId, label, x, y, style, name);
}

long clipsMessageCreateFromBitmap()
{
  if (ArgCountCheck("clipsMessageCreateFromBitmap", AT_LEAST, 2) == -1)
    return 0;

  int no_args = RtnArgCount();

  WXID parentId = RtnLong(1);
  long bitmap_id = RtnLong(2);

  // Set up defaults
  int x = -1;
  int y = -1;
  char *style = "";
  char *name = "message";

  if (no_args > 2)
    x = (int) RtnLong(3);

  if (no_args > 3)
    y = (int) RtnLong(4);

  if (no_args > 4)
    style = RtnLexeme(5);

  if (no_args > 5)
    name = RtnLexeme(6);

  return wxcMessageCreateFromBitmap(parentId, bitmap_id, x, y, style, name);
}
#endif

// Create a listbox widget
WXID wxcListBoxCreate(WXID parentId, wxClipsFunction func, char *label, Bool multiple,
  int x, int y, int width, int height, char *styleString, char *name)
{
#ifdef USE_CLIPS
  if (func && (strlen(func) == 0))
    func = NULL;
#endif
  // Call the generic function in wxextend library
  WXID id = wxcListBoxCreate(parentId, label, multiple, x, y, width, height, styleString, name);
  if (func)
    wxcObjectAddCallback(id, "OnCommand", func);
  return id;
}

#ifdef USE_CLIPS
long clipsListBoxCreate()
{
  if (ArgCountCheck("clipsListBoxCreate", AT_LEAST, 3) == -1)
    return 0;

  int no_args = RtnArgCount();

  WXID parentId = RtnLong(1);
  char *func = RtnLexeme(2);
  char *label = RtnLexeme(3);
  // Set up defaults
  Bool multiple = FALSE;
  int x = -1;
  int y = -1;
  int width = -1;
  int height = -1;
  char *style = "";
  char *name = "listBox";

  if (no_args > 3)
    multiple = (Bool)RtnLong(4);

  if (no_args > 4)
    x = (int) RtnLong(5);

  if (no_args > 5)
    y = (int) RtnLong(6);

  if (no_args > 6)
    width = (int) RtnLong(7);

  if (no_args > 7)
    height = (int) RtnLong(8);

  if (no_args > 8)
    style = RtnLexeme(9);

  if (no_args > 9)
    name = RtnLexeme(10);

  return wxcListBoxCreate(parentId, func, label, multiple, x, y, width, height, style, name);
}

long clipsListBoxAppend()
{
  if (ArgCountCheck("clipsListBoxAppend", AT_LEAST, 2) == -1)
    return 0;

  int no_args = RtnArgCount();

  long id = RtnLong(1);
  char *item_string = RtnLexeme(2);
  char *client_data = NULL;
  if (no_args > 2)
    client_data = RtnLexeme(3);

  return wxcListBoxAppend(id, item_string, client_data);
}

long clipsListBoxFindString()
{
  if (ArgCountCheck("clipsListBoxFindString", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  char *item_string = RtnLexeme(2);

  return wxcListBoxFindString(id, item_string);
}

long clipsListBoxClear()
{
  if (ArgCountCheck("clipsListBoxClear", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcListBoxClear(id);
}

long clipsListBoxSetSelection()
{
  if (ArgCountCheck("clipsListBoxSetSelection", AT_LEAST, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long sel = RtnLong(2);
  Bool flag = TRUE;
  if (RtnArgCount() > 2)
    flag = (Bool)RtnLong(3);
  return wxcListBoxSetSelection(id, (int)sel, flag);
}

long clipsListBoxDeselect()
{
  if (ArgCountCheck("clipsListBoxDeselect", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long sel = RtnLong(2);
  return wxcListBoxDeselect(id, (int)sel);
}

long clipsListBoxGetSelection()
{
  if (ArgCountCheck("clipsListBoxGetSelection", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return (long)wxcListBoxGetSelection(id);
}

char *clipsListBoxGetStringSelection()
{
  if (ArgCountCheck("clipsListBoxGetStringSelection", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");

  long id = RtnLong(1);
  char *ans = wxcListBoxGetStringSelection(id);
  if (ans)
    return (char *)AddSymbol(ans);
  else
    return (char *)AddSymbol("");
}

long clipsListBoxSetStringSelection()
{
  if (ArgCountCheck("clipsListBoxSetStringSelection", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  char *sel = RtnLexeme(2);
  return wxcListBoxSetStringSelection(id, sel);
}

long clipsListBoxNumber()
{
  if (ArgCountCheck("clipsListBoxSetNumber", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return (long)wxcListBoxNumber(id);
}

long clipsListBoxDelete()
{
  if (ArgCountCheck("clipsListBoxSetNumber", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long n = RtnLong(2);
  return (long)wxcListBoxDelete(id, (int)n);
}

char *clipsListBoxGetClientData()
{
  if (ArgCountCheck("clipsListBoxGetClientData", EXACTLY, 2) == -1)
    return (char *)AddSymbol("");

  long id = RtnLong(1);
  long sel = RtnLong(2);
  char *ans = wxcListBoxGetClientData(id, (int)sel);
  if (ans)
    return (char *)AddSymbol(ans);
  else
    return (char *)AddSymbol("");
}

char *clipsListBoxGetString()
{
  if (ArgCountCheck("clipsListBoxGetString", EXACTLY, 2) == -1)
    return (char *)AddSymbol("");

  long id = RtnLong(1);
  long sel = RtnLong(2);
  char *ans = wxcListBoxGetString(id, (int)sel);
  if (ans)
    return (char *)AddSymbol(ans);
  else
    return (char *)AddSymbol("");
}

long clipsListBoxGetFirstSelection()
{
  if (ArgCountCheck("clipsListBoxGetFirstSelection", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcListBoxGetFirstSelection(id);
}

long clipsListBoxGetNextSelection()
{
  if (ArgCountCheck("clipsListBoxGetNextSelection", EXACTLY, 0) == -1)
    return 0;
  return wxcListBoxGetNextSelection();
}

long clipsListBoxIsSelected()
{
  if (ArgCountCheck("clipsListBoxIsSelected", EXACTLY, 2) == -1)
    return 0;
  return wxcListBoxIsSelected(RtnLong(1), RtnLong(2));
}
#endif

// Generic choice widget creator, used only indirectly by C-only or CLIPS-only
// functions. This is so we can have a generic function for all languages.
// Create a choice widget
WXID wxcChoiceCreate(WXID parentId, wxClipsFunction func, char *label, int x, int y,
  int width, int height, int n, char **choices, char *styleString, char *name)
{
#ifdef USE_CLIPS
  if (func && (strlen(func) == 0))
    func = NULL;
#endif
  // Call the generic function in wxextend library
  WXID id = wxcChoiceCreate(parentId, label, x, y, width, height, n, choices, styleString, name);
  if (func)
    wxcObjectAddCallback(id, "OnCommand", func);
  return id;
}

#ifndef USE_CLIPS
// Create a choice widget
WXID wxcChoiceCreate(WXID parentId, wxClipsFunction func, char *label, int x, int y,
  int width, int height, char *name)
{
  return wxcChoiceCreate(parentId, func, label, x, y, width, height, 0, NULL, name);
}

// Create a choice widget
WXID wxcChoiceCreate(WXID parentId, wxClipsFunction func, char *label, int x, int y,
  int width, int height, ClipsExpr& choices, char *styleString, char *name)
{
  if (choices.Type() != ClipsList)
    return NULL;

  int n = 0;
  char **choiceStrings = new char *[choices.Number()];
  ClipsExpr *expr = choices.GetFirst();
  while (expr)
  {
    if (expr->Type() == ClipsString || expr->Type() == ClipsWord)
    {
      choiceStrings[n] = expr->StringValue();
      n ++;
    }
    else if (expr->Type() == ClipsStringClass)
    {
      choiceStrings[n] = (char *)expr->StringClassValue().GetData();
      n ++;
    }
    expr = expr->GetNext();
  }

  if (!label)
    return -1;

  // Use generic, char * array version above
  long retValue = wxcChoiceCreate(parentId, func, label, x, y, width, height, n, choiceStrings, styleString, name);
  delete[] choiceStrings;
  return retValue;
}
#endif

#ifdef USE_CLIPS
long clipsChoiceCreate()
{
  if (ArgCountCheck("clipsChoiceCreate", AT_LEAST, 3) == -1)
    return 0;

  int no_args = RtnArgCount();

  WXID parentId = RtnLong(1);
  char *func = RtnLexeme(2);
  char *label = RtnLexeme(3);
  // Set up defaults
  int x = -1;
  int y = -1;
  int width = -1;
  int height = -1;
  char *style = "";
  char *name = "choice";
  DATA_OBJECT arg;

  if (no_args > 3)
    x = (int) RtnLong(4);

  if (no_args > 4)
    y = (int) RtnLong(5);

  if (no_args > 5)
    width = (int) RtnLong(6);

  if (no_args > 6)
    height = (int) RtnLong(7);

  int n = 0;
  char **choices = NULL;

  if (no_args > 7)
  {
    // Extract a list of strings
    RtnUnknown(8, &arg);

    if (ClipsGetType(arg) != MULTIFIELD)
    {
      ExtensionError << "Error in choice-create: 8th argument must be a multifield of strings.\n";
      return -1;
    }
    int start = GetpDOBegin(&arg);
    int end = GetpDOEnd(&arg);
    n = end-start + 1;

    VOID *multifieldPtr = ClipsGetValue(arg);

    choices = new char *[n];

    for (int i = start; i <= end; i++)
    {
      if ((GetMFType(multifieldPtr, i) == STRING) || (GetMFType(multifieldPtr, i) == SYMBOL))
      {
        choices[i-start] = ValueToString(GetMFValue(multifieldPtr, i));
      }
      else
      {
        ExtensionError << "Error in choice-create: 8th argument, multifield element " << i-start+1 << " must be a string or symbol.\n";
        delete[] choices;
        return -1;
      }
    }
  }

  if (no_args > 8)
    style = RtnLexeme(9);

  if (no_args > 9)
    name = RtnLexeme(10);

  long retValue = wxcChoiceCreate(parentId, func, label, x, y, width, height, n, choices, style, name);
  if (choices)
    delete[] choices;
  return retValue;
}

long clipsChoiceAppend()
{
  if (ArgCountCheck("clipsChoiceAppend", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  char *item_string = RtnLexeme(2);
  return wxcChoiceAppend(id, item_string);
}

long clipsChoiceNumber()
{
  if (ArgCountCheck("clipsChoiceNumber", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return (long)wxcChoiceNumber(id);
}

long clipsChoiceFindString()
{
  if (ArgCountCheck("clipsChoiceFindString", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  char *item_string = RtnLexeme(2);
  return wxcChoiceFindString(id, item_string);
}

long clipsChoiceClear()
{
  if (ArgCountCheck("clipsChoiceClear", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcChoiceClear(id);
}

long clipsChoiceSetSelection()
{
  if (ArgCountCheck("clipsChoiceSetSelection", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long sel = RtnLong(2);

  return wxcChoiceSetSelection(id, (int)sel);
}

long clipsChoiceGetSelection()
{
  if (ArgCountCheck("clipsChoiceGetSelection", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcChoiceGetSelection(id);
}

char *clipsChoiceGetStringSelection()
{
  if (ArgCountCheck("clipsChoiceGetStringSelection", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");

  long id = RtnLong(1);
  char *ans = wxcChoiceGetStringSelection(id);
  if (ans)
    return (char *)AddSymbol(ans);
  else
    return (char *)AddSymbol("");
}

long clipsChoiceSetStringSelection()
{
  if (ArgCountCheck("clipsChoiceSetStringSelection", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  char *sel = RtnLexeme(2);
  return wxcChoiceSetStringSelection(id, sel);
}

char *clipsChoiceGetString()
{
  if (ArgCountCheck("clipsChoiceGetString", EXACTLY, 2) == -1)
    return (char *)AddSymbol("");

  long id = RtnLong(1);
  long sel = RtnLong(2);
  char *ans = wxcChoiceGetString(id, (int)sel);
  if (ans)
    return (char *)AddSymbol(ans);
  else
    return (char *)AddSymbol("");
}
#endif

// Generic radiobox widget creator, used only indirectly by C-only or CLIPS-only
// functions. This is so we can have a generic function for all languages.
WXID wxcRadioBoxCreate(WXID parentId, wxClipsFunction func, char *label, int x, int y,
  int width, int height, int n, char **choices, int majorDim, char *styleString, char *name)
{
#ifdef USE_CLIPS
  if (func && (strlen(func) == 0))
    func = NULL;
#endif
  // Call the generic function in wxextend library
  WXID id = wxcRadioBoxCreate(parentId, label, x, y, width, height, n, choices, majorDim, styleString, name);
  if (func)
    wxcObjectAddCallback(id, "OnCommand", func);
  return id;
}

#ifndef USE_CLIPS
// Create a radiobox widget
WXID wxcRadioBoxCreate(WXID parentId, wxClipsFunction func, char *label, int x, int y,
  int width, int height, ClipsExpr& choices, int majorDim, char *styleString, char *name)
{
  if (choices.Type() != ClipsList)
    return NULL;

  int n = 0;
  char **choiceStrings = new char *[choices.Number()];
  ClipsExpr *expr = choices.GetFirst();
  while (expr)
  {
    if (expr->Type() == ClipsString || expr->Type() == ClipsWord)
    {
      choiceStrings[n] = expr->StringValue();
      n ++;
    }
    else if (expr->Type() == ClipsStringClass)
    {
      choiceStrings[n] = (char *)expr->StringClassValue().GetData();
      n ++;
    }
    expr = expr->GetNext();
  }

  if (!label)
    return -1;

  // Use generic, char * array version above
  long retValue = wxcRadioBoxCreate(parentId, func, label, x, y, width, height, n, choiceStrings,
      majorDim, styleString, name);
  delete[] choiceStrings;
  return retValue;
}
#endif

#ifdef USE_CLIPS
long clipsRadioBoxCreate()
{
  if (ArgCountCheck("clipsRadioBoxCreate", AT_LEAST, 3) == -1)
    return 0;

  int no_args = RtnArgCount();

  WXID parentId = RtnLong(1);
  char *func = RtnLexeme(2);
  char *label = RtnLexeme(3);
  // Set up defaults
  int x = -1;
  int y = -1;
  int width = -1;
  int height = -1;
  int majorDim = 0;
  char *style = "";
  char *name = "radioBox";
  DATA_OBJECT arg;

  if (no_args > 3)
    x = (int) RtnLong(4);

  if (no_args > 4)
    y = (int) RtnLong(5);

  if (no_args > 5)
    width = (int) RtnLong(6);

  if (no_args > 6)
    height = (int) RtnLong(7);

  int n = 0;
  char **choices = NULL;

  if (no_args > 7)
  {
    // Extract a list of strings
    RtnUnknown(8, &arg);

    if (ClipsGetType(arg) != MULTIFIELD)
    {
      ExtensionError << "Error in radio-box-create: 8th argument must be a multifield of strings.\n";
      return -1;
    }
    int start = GetpDOBegin(&arg);
    int end = GetpDOEnd(&arg);
    n = end-start + 1;

    VOID *multifieldPtr = ClipsGetValue(arg);

    choices = new char *[n];

    for (int i = start; i <= end; i++)
    {
      if ((GetMFType(multifieldPtr, i) == STRING) || (GetMFType(multifieldPtr, i) == SYMBOL))
      {
        choices[i-start] = ValueToString(GetMFValue(multifieldPtr, i));
      }
      else
      {
        ExtensionError << "Error in radio-box-create: 8th argument, multifield element " << i-start+1 << " must be a string or symbol.\n";
        delete[] choices;
        return -1;
      }
    }
  }

  if (no_args > 8)
    majorDim = (int)RtnLong(9);

  if (no_args > 9)
    style = RtnLexeme(10);

  if (no_args > 10)
    name = RtnLexeme(11);

  long retValue = wxcRadioBoxCreate(parentId, func, label, x, y, width, height, n, choices, majorDim, style, name);
  if (choices)
    delete[] choices;
  return retValue;
}

long clipsRadioBoxSetSelection()
{
  if (ArgCountCheck("clipsRadioBoxSetSelection", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long sel = RtnLong(2);

  return wxcRadioBoxSetSelection(id, (int)sel);
}

long clipsRadioBoxGetSelection()
{
  if (ArgCountCheck("clipsRadioBoxGetSelection", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcRadioBoxGetSelection(id);
}

#if USE_GAUGE
long clipsGaugeCreate()
{
  if (ArgCountCheck("clipsButtonCreate", AT_LEAST, 3) == -1)
    return 0;

  int no_args = RtnArgCount();

  WXID parentId = RtnLong(1);
  char *label = RtnLexeme(2);
  int range = (int)RtnLong(3);

  // Set up defaults
  int x = -1;
  int y = -1;
  int width = -1;
  int height = -1;
  char *style = "";
  char *name = "gauge";

  if (no_args > 3)
    x = (int) RtnLong(4);

  if (no_args > 4)
    y = (int) RtnLong(5);

  if (no_args > 5)
    width = (int) RtnLong(6);

  if (no_args > 6)
    height = (int) RtnLong(7);

  if (no_args > 7)
    style = RtnLexeme(8);

  if (no_args > 8)
    name = RtnLexeme(9);

  return wxcGaugeCreate(parentId, label, range, x, y, width, height, style, name);
}

long clipsGaugeSetValue()
{
  if (ArgCountCheck("clipsGaugeSetValue", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long value = RtnLong(2);
  return wxcGaugeSetValue(id, (int)value);
}

long clipsGaugeSetShadowWidth()
{
  if (ArgCountCheck("clipsGaugeSetValue", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long value = RtnLong(2);
  return wxcGaugeSetShadowWidth(id, (int)value);
}

long clipsGaugeSetBezelFace()
{
  if (ArgCountCheck("clipsGaugeSetBezelFace", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long value = RtnLong(2);
  return wxcGaugeSetBezelFace(id, (int)value);
}
#endif // USE_GAUGE

void wxExtensionMenuCallback(wxMenu& item, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback(CLASSINFO(wxMenu), "OnCommand");
  if (func)
  {
    wxStoreThingInTable((long)&event, &event, CLASSINFO(wxCommandEvent));
    
    Bool handled = FALSE;
    
    (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item, (long)&event);

    wxDeleteThingInTable((long)&event);
  }
}

long clipsMenuCreate()
{
  if (ArgCountCheck("clipsMenuCreate", AT_LEAST, 0) == -1)
    return 0;

  int no_args = RtnArgCount();

  // Set up defaults
  char *title = NULL;
  wxClipsFunction func = 0;
  if (no_args > 0)
    title = RtnLexeme(1);
  if (no_args > 1)
    func = RtnLexeme(2);

  if (title && strlen(title) == 0)
    title = NULL;

  long id = wxcMenuCreate(title);
  if (id > 0)
  {
    if (func)
    {
      wxMenu *menu = (wxMenu *)id;
      menu->Callback((wxFunction)wxExtensionMenuCallback);
      wxcObjectAddCallback(id, "OnCommand", func);
    }
  }
  return id;
}

long clipsMenuAppend()
{
  if (ArgCountCheck("clipsMenuAppend", AT_LEAST, 3) == -1)
    return 0;

  int no_args = RtnArgCount();

  long id = RtnLong(1);
  long item_id = RtnLong(2);
  char *menu_string = RtnLexeme(3);
  char *help_string = NULL;
  long submenu_id = -1;
  Bool checkable = FALSE;
  if (no_args > 3)
    submenu_id = RtnLong(4);
  if (no_args > 4)
    help_string = RtnLexeme(5);
  if (no_args > 5)
    checkable = (Bool)RtnLong(6);

  if (help_string && strlen(help_string) == 0)
    help_string = NULL;
    
  return wxcMenuAppend(id, (int)item_id, menu_string, submenu_id, help_string, checkable);
}

long clipsMenuAppendSeparator()
{
  if (ArgCountCheck("clipsMenuAppendSeparator", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcMenuAppendSeparator(id);
}

long clipsMenuBreak()
{
  if (ArgCountCheck("clipsMenuBreak", EXACTLY, 1) == -1)
    return 0;

  long id = RtnLong(1);
  return wxcMenuBreak(id);
}

long clipsMenuEnable()
{
  if (ArgCountCheck("clipsMenuEnable", EXACTLY, 3) == -1)
    return 0;

  long id = RtnLong(1);
  long item_id = RtnLong(2);
  long enabled = RtnLong(3);
  return wxcMenuEnable(id, (int)item_id, (Bool)enabled);
}

long clipsMenuCheck()
{
  if (ArgCountCheck("clipsMenuCheck", EXACTLY, 3) == -1)
    return 0;

  long id = RtnLong(1);
  long item_id = RtnLong(2);
  long checked = RtnLong(3);

  return wxcMenuCheck(id, (int)item_id, (Bool)checked);
}

long clipsMenuSetLabel()
{
  if (ArgCountCheck("clipsMenuSetLabel", EXACTLY, 3) == -1)
    return 0;

  long id = RtnLong(1);
  long item_id = RtnLong(2);
  char *label = RtnLexeme(3);

  return wxcMenuSetLabel(id, (int)item_id, label);
}

long clipsMenuBarCreate()
{
  if (ArgCountCheck("clipsMenuBarCreate", EXACTLY, 0) == -1)
    return 0;
  return wxcMenuBarCreate();
}

long clipsMenuBarCreateFromResource()
{
  if (ArgCountCheck("clipsMenuBarCreateFromResource", EXACTLY, 1) == -1)
    return 0;
  return wxcMenuBarCreateFromResource(RtnLexeme(1));
}

long clipsMenuBarAppend()
{
  if (ArgCountCheck("clipsMenuBarAppend", EXACTLY, 3) == -1)
    return 0;

  long id = RtnLong(1);
  long menu_id = RtnLong(2);
  char *menu_string = RtnLexeme(3);
  return wxcMenuBarAppend(id, menu_id, menu_string);
}

long clipsMenuBarEnable()
{
  if (ArgCountCheck("clipsMenuBarEnable", EXACTLY, 3) == -1)
    return 0;

  long id = RtnLong(1);
  long item_id = RtnLong(2);
  long enabled = RtnLong(3);
  return wxcMenuBarEnable(id, (int)item_id, (Bool)enabled);
}

long clipsMenuBarCheck()
{
  if (ArgCountCheck("clipsMenuBarCheck", EXACTLY, 3) == -1)
    return 0;

  long id = RtnLong(1);
  long item_id = RtnLong(2);
  long checked = RtnLong(3);
  return wxcMenuBarCheck(id, (int)item_id, (Bool)checked);
}

long clipsMenuBarChecked()
{
  if (ArgCountCheck("clipsMenuBarChecked", EXACTLY, 2) == -1)
    return 0;

  long id = RtnLong(1);
  long item_id = RtnLong(2);
  return wxcMenuBarChecked(id, (int)item_id);
}
#endif

/*************************************************************/
/* UserFunctions:  The function which informs CLIPS of any   */
/*   user defined functions.  In the default case, there are */
/*   no user defined functions.  To define functions, either */
/*   this function must be replaced by a function with the   */
/*   same name within this file, or this function can be     */
/*   deleted from this file and included in another file.    */
/*   User defined functions may be included in this file or  */
/*   other files.                                            */
/*   Example of redefined UserFunctions:                     */
/*     UserFunctions()                                       */
/*       {                                                   */
/*        DefineFunction("fun1",'i',fun1,"fun1");            */
/*        DefineFunction("other",'f',other,"other");         */
/*       }                                                   */
/*************************************************************/

#ifdef USE_CLIPS

void wxItemUserFunctions()
{  
  // Item functions
  DefineFunction("button-create", 'l', PTIF clipsButtonCreate, "clipsButtonCreate");
  DefineFunction("button-create-from-bitmap", 'l', PTIF clipsButtonCreateFromBitmap, "clipsButtonCreateFromBitmap");

  DefineFunction("check-box-create", 'l', PTIF clipsCheckBoxCreate, "clipsCheckBoxCreate");
  DefineFunction("check-box-set-value", 'l', PTIF clipsCheckBoxSetValue, "clipsCheckBoxSetValue");
  DefineFunction("check-box-get-value", 'l', PTIF clipsCheckBoxGetValue, "clipsCheckBoxGetValue");

  DefineFunction("choice-create", 'l', PTIF clipsChoiceCreate, "clipsChoiceCreate");
  DefineFunction("choice-append", 'l', PTIF clipsChoiceAppend, "clipsChoiceAppend");
  DefineFunction("choice-number", 'l', PTIF clipsChoiceNumber, "clipsChoiceNumber");
  DefineFunction("choice-find-string", 'l', PTIF clipsChoiceFindString, "clipsChoiceFindString");
  DefineFunction("choice-clear", 'l', PTIF clipsChoiceClear, "clipsChoiceClear");
  DefineFunction("choice-set-selection", 'l', PTIF clipsChoiceSetSelection, "clipsChoiceSetSelection");
  DefineFunction("choice-get-string-selection", 's', PTIF clipsChoiceGetStringSelection, "clipsChoiceGetStringSelection");
  DefineFunction("choice-get-selection", 'l', PTIF clipsChoiceGetSelection, "clipsChoiceGetSelection");
  DefineFunction("choice-set-string-selection", 'l', PTIF clipsChoiceSetStringSelection, "clipsChoiceSetStringSelection");
  DefineFunction("choice-get-string", 's', PTIF clipsChoiceGetString, "clipsChoiceGetString");

  DefineFunction("list-box-create", 'l', PTIF clipsListBoxCreate, "clipsListBoxCreate");
  DefineFunction("list-box-append", 'l', PTIF clipsListBoxAppend, "clipsListBoxAppend");
  DefineFunction("list-box-find-string", 'l', PTIF clipsListBoxFindString, "clipsListBoxFindString");
  DefineFunction("list-box-clear", 'l', PTIF clipsListBoxClear, "clipsListBoxClear");
  DefineFunction("list-box-set-selection", 'l', PTIF clipsListBoxSetSelection, "clipsListBoxSetSelection");
  DefineFunction("list-box-get-string-selection", 's', PTIF clipsListBoxGetStringSelection, "clipsListBoxGetStringSelection");
  DefineFunction("list-box-get-selection", 'l', PTIF clipsListBoxGetSelection, "clipsListBoxGetSelection");
  DefineFunction("list-box-set-string-selection", 'l', PTIF clipsListBoxSetStringSelection, "clipsListBoxSetStringSelection");
  DefineFunction("list-box-number", 'l', PTIF clipsListBoxNumber, "clipsListBoxNumber");
  DefineFunction("list-box-delete", 'l', PTIF clipsListBoxDelete, "clipsListBoxDelete");
  DefineFunction("list-box-get-string", 's', PTIF clipsListBoxGetString, "clipsListBoxGetString");
  DefineFunction("list-box-get-first-selection", 'l', PTIF clipsListBoxGetFirstSelection, "clipsListBoxGetFirstSelection");
  DefineFunction("list-box-get-next-selection", 'l', PTIF clipsListBoxGetNextSelection, "clipsListBoxGetNextSelection");
  DefineFunction("list-box-is-selected", 'l', PTIF clipsListBoxIsSelected, "clipsListBoxIsSelected");

  DefineFunction("menu-create", 'l', PTIF clipsMenuCreate, "clipsMenuCreate");
  DefineFunction("menu-append", 'l', PTIF clipsMenuAppend, "clipsMenuAppend");
  DefineFunction("menu-append-separator", 'l', PTIF clipsMenuAppendSeparator, "clipsMenuAppendSeparator");
  DefineFunction("menu-check", 'l', PTIF clipsMenuCheck, "clipsMenuCheck");
  DefineFunction("menu-enable", 'l', PTIF clipsMenuEnable, "clipsMenuEnable");
  DefineFunction("menu-break", 'l', PTIF clipsMenuBreak, "clipsMenuBreak");
  DefineFunction("menu-set-label", 'l', PTIF clipsMenuSetLabel, "clipsMenuSetLabel");

  DefineFunction("menu-bar-create", 'l', PTIF clipsMenuBarCreate, "clipsMenuBarCreate");
  DefineFunction("menu-bar-create-from-resource", 'l', PTIF clipsMenuBarCreateFromResource, "clipsMenuBarCreateFromResource");
  DefineFunction("menu-bar-append", 'l', PTIF clipsMenuBarAppend, "clipsMenuBarAppend");
  DefineFunction("menu-bar-check", 'l', PTIF clipsMenuBarCheck, "clipsMenuBarCheck");
  DefineFunction("menu-bar-checked", 'l', PTIF clipsMenuBarChecked, "clipsMenuBarChecked");
  DefineFunction("menu-bar-enable", 'l', PTIF clipsMenuBarEnable, "clipsMenuBarEnable");

  DefineFunction("message-create", 'l', PTIF clipsMessageCreate, "clipsMessageCreate");
  DefineFunction("message-create-from-bitmap", 'l', PTIF clipsMessageCreateFromBitmap, "clipsMessageCreateFromBitmap");

  DefineFunction("multi-text-create", 'l', PTIF clipsMultiTextCreate, "clipsMultiTextCreate");
  DefineFunction("multi-text-set-value", 'l', PTIF clipsMultiTextSetValue, "clipsMultiTextSetValue");
  DefineFunction("multi-text-get-value", 's', PTIF clipsMultiTextGetValue, "clipsMultiTextGetValue");

#ifdef wx_msw
  DefineFunction("multi-text-write", 'v', PTIF clipsMultiTextWrite, "clipsMultiTextWrite");
  DefineFunction("multi-text-copy", 'l', PTIF clipsMultiTextCopy, "clipsMultiTextCopy");
  DefineFunction("multi-text-cut", 'l', PTIF clipsMultiTextCut, "clipsMultiTextCut");
  DefineFunction("multi-text-paste", 'l', PTIF clipsMultiTextPaste, "clipsMultiTextPaste");
  DefineFunction("multi-text-replace", 'l', PTIF clipsMultiTextReplace, "clipsMultiTextReplace");
  DefineFunction("multi-text-remove", 'l', PTIF clipsMultiTextRemove, "clipsMultiTextRemove");
  DefineFunction("multi-text-set-selection", 'l', PTIF clipsMultiTextSetSelection, "clipsMultiTextSetSelection");
  DefineFunction("multi-text-get-last-position", 'l', PTIF clipsMultiTextGetLastPosition, "clipsMultiTextGetLastPosition");
  DefineFunction("multi-text-show-position", 'l', PTIF clipsMultiTextShowPosition, "clipsMultiTextShowPosition");
  DefineFunction("multi-text-position-to-line", 'l', PTIF clipsMultiTextPositionToLine, "clipsMultiTextPositionToLine");
  DefineFunction("multi-text-position-to-char", 'l', PTIF clipsMultiTextPositionToChar, "clipsMultiTextPositionToChar");
  DefineFunction("multi-text-set-insertion-point", 'l', PTIF clipsMultiTextSetInsertionPoint, "clipsMultiTextSetInsertionPoint");
  DefineFunction("multi-text-get-insertion-point", 'l', PTIF clipsMultiTextGetInsertionPoint, "clipsMultiTextGetInsertionPoint");
  DefineFunction("multi-text-get-line-length", 'l', PTIF clipsMultiTextGetLineLength, "clipsMultiTextGetLineLength");
  DefineFunction("multi-text-get-line-length", 'l', PTIF clipsMultiTextGetLineLength, "clipsMultiTextGetLineLength");
  DefineFunction("multi-text-get-line-text", 's', PTIF clipsMultiTextGetLineText, "clipsMultiTextGetLineText");
  DefineFunction("multi-text-get-number-of-lines", 'l', PTIF clipsMultiTextGetNumberOfLines, "clipsMultiTextGetNumberOfLines");
  DefineFunction("multi-text-xy-to-position", 'l', PTIF clipsMultiTextXYToPosition, "clipsMultiTextXYToPosition");
#endif

  DefineFunction("panel-item-set-default", 'l', PTIF clipsPanelItemSetDefault, "clipsPanelItemSetDefault");
  DefineFunction("panel-item-set-label", 'l', PTIF clipsPanelItemSetLabel, "clipsPanelItemSetLabel");
  DefineFunction("panel-item-get-label", 's', PTIF clipsPanelItemGetLabel, "clipsPanelItemGetLabel");
  DefineFunction("panel-item-get-command-event", 'l', PTIF clipsPanelItemGetCommandEvent, "clipsPanelItemGetCommandEvent");

  DefineFunction("radio-box-create", 'l', PTIF clipsRadioBoxCreate, "clipsRadioBoxCreate");
  DefineFunction("radio-box-set-selection", 'l', PTIF clipsRadioBoxSetSelection, "clipsRadioBoxSetSelection");
  DefineFunction("radio-box-get-selection", 'l', PTIF clipsRadioBoxGetSelection, "clipsRadioBoxGetSelection");

  DefineFunction("slider-create", 'l', PTIF clipsSliderCreate, "clipsSliderCreate");
  DefineFunction("slider-set-value", 'l', PTIF clipsSliderSetValue, "clipsSliderSetValue");
  DefineFunction("slider-get-value", 'l', PTIF clipsSliderGetValue, "clipsSliderGetValue");

  DefineFunction("text-create", 'l', PTIF clipsTextCreate, "clipsTextCreate");
  DefineFunction("text-set-value", 'l', PTIF clipsTextSetValue, "clipsTextSetValue");
  DefineFunction("text-get-value", 's', PTIF clipsTextGetValue, "clipsTextGetValue");

  DefineFunction("group-box-create", 'l', PTIF clipsGroupBoxCreate, "clipsGroupBoxCreate");

#if USE_GAUGE
  DefineFunction("gauge-create", 'l', PTIF clipsGaugeCreate, "clipsGaugeCreate");
  DefineFunction("gauge-set-value", 'l', PTIF clipsGaugeSetValue, "clipsGaugeSetValue");
  DefineFunction("gauge-set-shadow-width", 'l', PTIF clipsGaugeSetShadowWidth, "clipsGaugeSetShadowWidth");
  DefineFunction("gauge-set-bezel-face", 'l', PTIF clipsGaugeSetBezelFace, "clipsGaugeSetBezelFace");
#endif
}

#endif

