/*
 * wxckif.cc
 * KIF parser functionality (local AIAI project only)
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#include "agentkit.h"
#include "clipscpp.h"

#define EXACTLY 0
#define AT_LEAST 1
#define PTIF (int (*) ())

#define USE_CLIPS 1

/************************************************************************
 * KQML functions
 *
 ************************************************************************/

long clipsKQMLIsPerformative()
{
  if (ArgCountCheck("clipsKQMLIsPerformative", EXACTLY, 1) == -1)
    return 0;
    
  expression *expr = (expression *)RtnLong(1);
  return (long)KQMLIsPerformative(expr);
}

char *clipsKQMLGetPerformative()
{
  if (ArgCountCheck("clipsKQMLGetPerformative", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");
    
  expression *expr = (expression *)RtnLong(1);
  wxString str(KQMLGetPerformative(expr));
  strcpy(wxBuffer, str.GetData());
  return (char *)AddSymbol(wxBuffer);
}

void clipsKQMLDeleteExpression()
{
  if (ArgCountCheck("clipsKQMLDeleteExpression", EXACTLY, 1) == -1)
    return;
    
  expression *expr = (expression *)RtnLong(1);
  delete expr;
}

char *clipsKQMLGetFunctor()
{
  if (ArgCountCheck("clipsKQMLGetFunctor", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");
    
  expression *expr = (expression *)RtnLong(1);
  wxString str(KQMLGetFunctor(expr));
  strcpy(wxBuffer, str.GetData());
  return (char *)AddSymbol(wxBuffer);
}

char *clipsKQMLGetKeywordValueString()
{
  if (ArgCountCheck("clipsKQMLGetKeywordValueString", EXACTLY, 2) == -1)
    return (char *)AddSymbol("");

  char *s = RtnLexeme(1);
  wxString str(s);
  expression *expr = (expression *)RtnLong(2);
  wxString str2(KQMLGetKeywordValueString(str, expr));
  strcpy(wxBuffer, str2.GetData());
  return (char *)AddSymbol(wxBuffer);
}

long clipsKQMLGetKeywordValue()
{
  if (ArgCountCheck("clipsKQMLGetKeywordValue", EXACTLY, 2) == -1)
    return 0;
    
  char *s = RtnLexeme(1);
  wxString str(s);
  expression *expr = (expression *)RtnLong(1);
  expression *expr2 = KQMLGetKeywordValue(str, expr);
  return (long)expr2;
}

long clipsKQMLGetTaskId()
{
  if (ArgCountCheck("clipsKQMLGetTaskId", EXACTLY, 1) == -1)
    return 0;
 
  expression *expr = (expression *)RtnLong(1);
  return KQMLGetTaskId(expr);
}

long clipsKQMLGetContent()
{
  if (ArgCountCheck("clipsKQMLGetContent", EXACTLY, 1) == -1)
    return 0;
 
  expression *expr = (expression *)RtnLong(1);
  return (long)KQMLGetContent(expr);
}

char *clipsKQMLGetCapability()
{
  if (ArgCountCheck("clipsKQMLGetCapability", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");
 
  expression *expr = (expression *)RtnLong(1);
  wxString str(KQMLGetCapability(expr));
  
  strcpy(wxBuffer, str.GetData());
  return (char *)AddSymbol(wxBuffer);
}

char *clipsKQMLGetFunction()
{
  if (ArgCountCheck("clipsKQMLGetFunction", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");
 
  expression *expr = (expression *)RtnLong(1);
  wxString str(KQMLGetFunction(expr));
  
  strcpy(wxBuffer, str.GetData());
  return (char *)AddSymbol(wxBuffer);
}

char *clipsKQMLGetInputDataString()
{
  if (ArgCountCheck("clipsKQMLGetInputDataString", EXACTLY, 2) == -1)
    return (char *)AddSymbol("");
 
  int i = (int)RtnLong(1);
  expression *expr = (expression *)RtnLong(2);
  wxString str(KQMLGetInputDataString(i, expr));
  
  strcpy(wxBuffer, str.GetData());
  return (char *)AddSymbol(wxBuffer);
}

long clipsKQMLGetInputData()
{
  if (ArgCountCheck("clipsKQMLGetInputData", EXACTLY, 2) == -1)
    return 0;
 
  int i = (int)RtnLong(1);
  expression *expr = (expression *)RtnLong(2);
  expression *expr2 = KQMLGetInputData(i, expr);
  
  return (long)expr2;
}

long clipsKQMLGetInputArgCount()
{
  if (ArgCountCheck("clipsKQMLGetInputArgCount", EXACTLY, 1) == -1)
    return 0;
 
  expression *expr = (expression *)RtnLong(1);
  return KQMLGetInputArgCount(expr);
}

long clipsKQMLGetOutputData()
{
  if (ArgCountCheck("clipsKQMLGetOutputData", EXACTLY, 1) == -1)
    return 0;
 
  expression *expr = (expression *)RtnLong(1);
  return (long)KQMLGetOutputData(expr);
}

char *clipsKQMLGetOutputDataString()
{
  if (ArgCountCheck("clipsKQMLGetOutputDataString", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");
 
  expression *expr = (expression *)RtnLong(1);
  wxString str(KQMLGetOutputDataString(expr));
  strcpy(wxBuffer, str.GetData());
  return (char *)AddSymbol(wxBuffer);
}

char *clipsKQMLGetLanguage()
{
  if (ArgCountCheck("clipsKQMLGetLanguage", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");
 
  expression *expr = (expression *)RtnLong(1);
  wxString str(KQMLGetLanguage(expr));
  strcpy(wxBuffer, str.GetData());
  return (char *)AddSymbol(wxBuffer);
}

char *clipsKQMLMakeString()
{
  if (ArgCountCheck("clipsKQMLMakeString", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");
 
  expression *expr = (expression *)RtnLong(1);
  wxString str(KQMLMakeString(expr));
  strcpy(wxBuffer, str.GetData());
  return (char *)AddSymbol(wxBuffer);
}

long clipsKQMLMakeExpression()
{
  if (ArgCountCheck("clipsKQMLMakeExpression", EXACTLY, 1) == -1)
    return 0;
 
  wxString str(RtnLexeme(1));
  expression *expr = KQMLMakeExpression(str);
  return (long)expr;
}

long clipsKQMLGetNth()
{
  if (ArgCountCheck("clipsKQMLGetNth", EXACTLY, 2) == -1)
    return 0;
 
  expression *expr = (expression *)RtnLong(2);
  int i = (int)RtnLong(1);

  return (long)KQMLGetNth(i, expr);
}

char *clipsKQMLGetNthString()
{
  if (ArgCountCheck("clipsKQMLGetNthString", EXACTLY, 2) == -1)
    return (char *)AddSymbol("");
 
  expression *expr = (expression *)RtnLong(2);
  int i = (int)RtnLong(1);

  wxString str(KQMLGetNthString(i, expr));
  strcpy(wxBuffer, str.GetData());
  return (char *)AddSymbol(wxBuffer);
}

long clipsKQMLGetLength()
{
  if (ArgCountCheck("clipsKQMLGetLength", EXACTLY, 1) == -1)
    return 0;
 
  expression *expr = (expression *)RtnLong(1);
  return (long)KQMLGetLength(expr);
}

char *clipsKQMLGetType()
{
  if (ArgCountCheck("clipsKQMLGetType", EXACTLY, 1) == -1)
    return (char *)AddSymbol("");
 
  expression *expr = (expression *)RtnLong(1);
  wxString str(KQMLGetType(expr));
  strcpy(wxBuffer, str.GetData());
  return (char *)AddSymbol(wxBuffer);
}

long clipsKQMLNotifyTaskManagerStarted()
{
  if (ArgCountCheck("clipsKQMLNotifyTaskManagerStarted", AT_LEAST, 1) == -1)
    return 0;
 
  char *agentName = RtnLexeme(1);
  char *taskManName = "TASK MANAGER";
  if (RtnArgCount() > 1)
    taskManName = RtnLexeme(2);
    
  return (long)KQMLNotifyTaskManagerStarted(wxString(agentName), wxString(taskManName));
}

long clipsKQMLNotifyTaskManagerFinished()
{
  if (ArgCountCheck("clipsKQMLNotifyTaskManagerFinished", AT_LEAST, 1) == -1)
    return 0;
 
  char *agentName = RtnLexeme(1);
  char *taskManName = "TASK MANAGER";
  if (RtnArgCount() > 1)
    taskManName = RtnLexeme(2);
    
  return (long)KQMLNotifyTaskManagerFinished(wxString(agentName), wxString(taskManName));
}

/*************************************************************/
/* UserFunctions:  The function which informs CLIPS of any   */
/*   user defined functions.  In the default case, there are */
/*   no user defined functions.  To define functions, either */
/*   this function must be replaced by a function with the   */
/*   same name within this file, or this function can be     */
/*   deleted from this file and included in another file.    */
/*   User defined functions may be included in this file or  */
/*   other files.                                            */
/*   Example of redefined UserFunctions:                     */
/*     UserFunctions()                                       */
/*       {                                                   */
/*        DefineFunction("fun1",'i',fun1,"fun1");            */
/*        DefineFunction("other",'f',other,"other");         */
/*       }                                                   */
/*************************************************************/

#ifdef USE_CLIPS
void wxKifUserFunctions()
{  
  // Miscellaneous functions
  DefineFunction("kqml-delete-expression", 'v', PTIF clipsKQMLDeleteExpression, "clipsKQMLDeleteExpression");
  DefineFunction("kqml-is-performative", 'l', PTIF clipsKQMLIsPerformative, "clipsKQMLIsPerformative");
  DefineFunction("kqml-get-performative", 's', PTIF clipsKQMLGetPerformative, "clipsKQMLGetPerformative");
  DefineFunction("kqml-get-functor", 's', PTIF clipsKQMLGetFunctor, "clipsKQMLGetFunctor");
  DefineFunction("kqml-get-keyword-value-string", 's', PTIF clipsKQMLGetKeywordValueString, "clipsKQMLGetKeywordValueString");
  DefineFunction("kqml-get-keyword-value", 'l', PTIF clipsKQMLGetKeywordValue, "clipsKQMLGetKeywordValue");
  DefineFunction("kqml-get-task-id", 'l', PTIF clipsKQMLGetTaskId, "clipsKQMLGetTaskId");
  DefineFunction("kqml-get-content", 'l', PTIF clipsKQMLGetContent, "clipsKQMLGetContent");
  DefineFunction("kqml-get-capability", 's', PTIF clipsKQMLGetCapability, "clipsKQMLGetCapability");
  DefineFunction("kqml-get-function", 's', PTIF clipsKQMLGetFunction, "clipsKQMLGetFunction");
  DefineFunction("kqml-get-input-data-string", 's', PTIF clipsKQMLGetInputDataString, "clipsKQMLGetInputDataString");
  DefineFunction("kqml-get-input-data", 'l', PTIF clipsKQMLGetInputData, "clipsKQMLGetInputData");
  DefineFunction("kqml-get-input-arg-count", 'l', PTIF clipsKQMLGetInputArgCount, "clipsKQMLGetInputArgCount");
  DefineFunction("kqml-get-output-data", 'l', PTIF clipsKQMLGetOutputData, "clipsKQMLGetOutputData");
  DefineFunction("kqml-get-output-data-string", 's', PTIF clipsKQMLGetOutputDataString, "clipsKQMLGetOutputDataString");
  DefineFunction("kqml-get-language", 's', PTIF clipsKQMLGetLanguage, "clipsKQMLGetLanguage");
  DefineFunction("kqml-make-string", 's', PTIF clipsKQMLMakeString, "clipsKQMLMakeString");
  DefineFunction("kqml-make-expression", 'l', PTIF clipsKQMLMakeExpression, "clipsKQMLMakeExpression");
  DefineFunction("kqml-get-nth", 'l', PTIF clipsKQMLGetNth, "clipsKQMLGetNth");
  DefineFunction("kqml-get-nth-string", 's', PTIF clipsKQMLGetNthString, "clipsKQMLGetNthString");
  DefineFunction("kqml-get-length", 'l', PTIF clipsKQMLGetLength, "clipsKQMLGetLength");
  DefineFunction("kqml-get-type", 's', PTIF clipsKQMLGetType, "clipsKQMLGetType");
  DefineFunction("kqml-notify-task-manager-started", 'l', PTIF clipsKQMLNotifyTaskManagerStarted, "clipsKQMLNotifyTaskManagerStarted");
  DefineFunction("kqml-notify-task-manager-finished", 'l', PTIF clipsKQMLNotifyTaskManagerFinished, "clipsKQMLNotifyTaskManagerFinished");
}

#endif
