/*
 * wxclips.cc
 *
 * wxWindows front end to CLIPS
 *
 */
#ifndef WXCLIPS
#define WXCLIPS
#endif

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#if !defined(USE_IOSTREAMH) || USE_IOSTREAMH
#include <iostream.h>
#else
#include <iostream>
#endif
#include <stdlib.h>
#include <ctype.h>

#ifdef wx_msw
#include <direct.h>
#endif

#include <wx_help.h>
#include "clipscpp.h"
#ifdef wx_msw
#include "wx_bbar.h"
#endif
#include "wx_cmds.h"
#include "wxclips.h"

/* following include added at NRCC -- if using nonANSI C compiler for clips and 
   ANSI CC compiler for this file must do something like this!! */
#include "setup.h"
#ifndef __WATCOMC__
#include "clips.h"
#endif

#ifdef GetValue
#undef GetValue
#undef SetValue
#endif

#ifdef wx_x
#include "aiai.xbm"
#endif

#ifdef CLIPS6
typedef void (*ClipsFunction) ();
#else
typedef int (*ClipsFunction) ();
#endif

#ifdef wx_msw
// Bitmaps for toolbar
static wxBitmap *ToolbarLoadBitmap = NULL;
static wxBitmap *ToolbarHelpBitmap = NULL;
static wxBitmap *ToolbarBatchBitmap = NULL;
static wxBitmap *ToolbarClearBitmap = NULL;
static wxBitmap *ToolbarLaunchBitmap = NULL;
static wxBitmap *ToolbarResetBitmap = NULL;
static wxBitmap *ToolbarRunBitmap = NULL;
static wxBitmap *ToolbarCopyBitmap = NULL;
#endif

#ifdef wx_x
#define COMMAND_PANEL_HEIGHT 130
#define COMMAND_TEXT_WIDTH   350
#endif
#ifdef wx_msw
#define COMMAND_PANEL_HEIGHT 100
#define COMMAND_TEXT_WIDTH   400
#endif

#ifdef FUZZYCLIPS
char *wxClipsTitle = "Fuzzy wxCLIPS";
#else
char *wxClipsTitle = "wxCLIPS";
#endif

extern wxObject *wxGetTypedObject(long, WXTYPE);

int all_watch = 0;
int compilations_watch = 1;
int facts_watch = 0;
int rules_watch = 0;
int activations_watch = 0;
int statistics_watch = 0;
int globals_watch = 0;
int deffunctions_watch = 0;
int limit = -1;

float wxClipsVersion = 1.64;

wxHelpInstance *HelpInstance = NULL;
wxClipsEnvFrame *wxTheClipsFrame = NULL;
#ifdef wx_msw
IDEServer *TheIDEServer = NULL;
IDEConnection *TheIDEConnection = NULL;
#endif

// Buffering of output, since output can be very slow
// otherwise
static char *OutputWindowBuffer = NULL;
static int OutputWindowBufferSize = 200;
static Bool OutputWindowBuffering = TRUE;
static int OutputWindowBufferIndex = 0;
// Buffered output
void BufferedPrintOutput(char *str);
// Toggle buffering
void SetOutputWindowBuffering(Bool on);
// Flush buffer
void FlushOutputWindowBuffer(void);

// A macro needed for some compilers (AIX) that need 'main' to be defined
// in the application itself.
IMPLEMENT_WXWIN_MAIN

// This statement initialises the whole application
MyApp     myApp;

wxFrame *wxStartApp(void)
{
  wxFrame *main_frame = NULL;
  clipsReturnType = clipsUNKNOWN;

  wxRouteNoEcho("(return-result (app-on-init))");

  if (clipsReturnType == clipsLONG)
  {
    if (clipsReturnLong == 0)
      return NULL;
      
    main_frame = (wxFrame *)wxGetTypedObject(clipsReturnLong, CLASSINFO(wxFrame));
    return main_frame;
  }
  else return NULL;
/*
  if (!main_frame)
  {
    ExtensionError << "Error in wxStartApp: app-on-init should return a main frame ID!\n";
#ifdef wx_x
    if (!wxTheClipsFrame)
      cerr << "Error in wxStartApp: app-on-init should return a main frame ID!\n";
#endif
    return NULL;
  }
  else return main_frame;
 */
}

void wxInitializeClips(void)
{
  // Make an output buffer 
  OutputWindowBuffer = new char[OutputWindowBufferSize+1];

  // Initialize CLIPS
  wxInitClips();

  AddRunFunction("wxyield", (ClipsFunction)ClipsYield, 100);

#ifdef CLIPS6
  AddRouter("wxrouter", 10,
            FindOutput,
            PrintOutput,
            DummyGetc,
            DummyUngetc,
            ExitOutput);
#else
  AddRouter("wxrouter", 10,
            (ClipsFunction)FindOutput,
            (ClipsFunction)PrintOutput,
            (ClipsFunction)DummyGetc,
            (ClipsFunction)DummyUngetc,
            (ClipsFunction)ExitOutput);
#endif

#ifdef CLIPS6
  Watch("compilations");
  Watch("facts");
  Watch("rules");
  Watch("activations");
  Watch("statistics");
  Watch("globals");
  Watch("all");

  Watch("compilations");
#else
  AddWatchItem("compilations", &compilations_watch, 100);
  AddWatchItem("facts", &facts_watch, 100);
  AddWatchItem("rules", &rules_watch, 100);
  AddWatchItem("activations", &activations_watch, 100);
  AddWatchItem("statistics", &statistics_watch, 100);
  AddWatchItem("globals", &globals_watch, 100);
  AddWatchItem("all", &all_watch, 100);

  SetWatchItem("compilations", compilations_watch);
#endif
}

void Usage(void)
{
#ifdef wx_x
  cout << "WXCLIPS command line options:\n";
  cout << "wxclips [-clips    <filename>]  ; batch file\n";
  cout << "        [-load     <filename>]  ; load file (constructs only)\n";
  cout << "        [-start]                ; Call (app-on-init), no debugging window\n";
  cout << "        [-dir]     <directory>] ; Change to this directory on startup\n";
  cout << "        [-h]                    ; this help screen\n";
  cout << "        [-help]                 ; list of generic X window arguments\n";
  exit(0);
#endif
}

// The `main program' equivalent, creating the windows and returning the
// main frame
wxFrame *MyApp::OnInit(void)
{
//  wxDebugContext::SetFile("debug.log");
  work_proc = wxClipsWorkProc;

  // Load toolbar bitmaps
#ifdef wx_msw
  ToolbarLoadBitmap = new wxBitmap("LOADTOOL");
  ToolbarHelpBitmap = new wxBitmap("HELPTOOL");
  ToolbarBatchBitmap = new wxBitmap("BATCHTOOL");
  ToolbarClearBitmap = new wxBitmap("CLEARTOOL");
  ToolbarLaunchBitmap = new wxBitmap("LAUNCHTOOL");
  ToolbarResetBitmap = new wxBitmap("RESETTOOL");
  ToolbarRunBitmap = new wxBitmap("RUNTOOL");
  ToolbarCopyBitmap = new wxBitmap("COPYTOOL");
#endif

  wxInitializeClips();
  wxIPCInitialize();

#ifdef wx_msw
  char *server_name = "WXCLIPS";

  // Create a new server
  TheIDEServer = new IDEServer;
  TheIDEServer->Create(server_name);
#endif

  HelpInstance = new wxHelpInstance(TRUE);
  HelpInstance->Initialize("wxclips");

  char *batchFile = NULL;
  char *loadFile = NULL;
  Bool start = FALSE;

  // Initial directory to set
  char *initDir = NULL;

  int i = 1;
  while (i < argc)
  {
    if (strcmp(argv[i], "-clips") == 0)
    {
      i ++;
      if (i < argc)
        batchFile = copystring(argv[i]);
      i ++;
    }
    else if (strcmp(argv[i], "-load") == 0)
    {
      i ++;
      if (i < argc)
        loadFile = copystring(argv[i]);
      i ++;
    }
    else if (strcmp(argv[i], "-start") == 0)
    {
      i ++;
      start = TRUE;
    }
    // Specify a starting directory
    else if (strcmp(argv[i], "-dir") == 0)
    {
      i ++;
      if (argc == i)
        wxError("-dir must take a directory name!");
      else
      {
        initDir = argv[i];
        i ++;
      }
    }
#ifdef wx_x
    else if (strcmp(argv[i], "-h") == 0)
    {
      i ++;
      Usage();
    }
#endif
    else
    {
      // Assume an unrecognised, first argument is a file to batch.
      // Also, change into that directory, and start the CLIPS program.
      if ( i == 1 )
      {
        batchFile = copystring(argv[i]);
        initDir = copystring(wxPathOnly(batchFile));
        start = TRUE;
      }
      else
      {
#ifdef wx_x
        cerr << "Invalid command-line option.\n";
#else
        wxError("Invalid command-line option.", "wxCLIPS error");
#endif
        Usage();
      }
      i ++;
    }
  }

  if (initDir)
    wxSetWorkingDirectory(initDir);

  wxFrame *clipsMainFrame = NULL;
  if (batchFile)
  {
    wxExecuteClipsFile(batchFile);
    if (start)
    {
      clipsMainFrame = wxStartApp();
    }
  }
  else if (loadFile)
  {
    LoadConstructs(loadFile);
    if (start)
    {
      clipsMainFrame = wxStartApp();
    }
  }

  if (start && clipsMainFrame)
  {
    return clipsMainFrame;
  }
  else
  {
    wxShowClipsWindow();
    return wxTheClipsFrame;
  }
}

void wxShowClipsWindow(void)
{
  if (wxTheClipsFrame)
  {
    wxTheClipsFrame->Iconize(FALSE);
    wxTheClipsFrame->Show(TRUE);
  }
  else
    wxTheClipsFrame = new wxClipsEnvFrame(NULL, wxClipsTitle, 10, 10, 480, 450);
}

void wxDeleteClipsWindow(void)
{
  if (wxTheClipsFrame && wxTheClipsFrame->OnClose())
  {
    delete wxTheClipsFrame;
    wxTheClipsFrame = NULL;
  }
}

// Send a command to CLIPS
void EnterCommand(wxButton& but, wxCommandEvent& event)
{
  char *text = wxTheClipsFrame->command_line->GetValue();
  if (text)
  {
    char *comm = copystring(text);

    long pos = wxTheClipsFrame->text_window->GetLastPosition();
    wxTheClipsFrame->text_window->SetInsertionPoint(pos);

    wxTheClipsFrame->text_window->WriteText("CLIPS> ");
    wxTheClipsFrame->text_window->WriteText(comm);
    wxTheClipsFrame->text_window->WriteText("\n");
    wxRouteCommand(comm);
    SetHaltExecution(CLIPS_FALSE);
    SetEvaluationError(CLIPS_FALSE);
    delete[] comm;
    PeriodicCleanup(CLIPS_TRUE,CLIPS_FALSE);
    if (wxTheClipsFrame)
    {
      wxTheClipsFrame->command_line->SetValue("");
      wxTheClipsFrame->command_line->SetFocus();
    }
    FlushOutputWindowBuffer();
  }
}

void CommandTextItemProc(wxText& text, wxCommandEvent& event)
{
  if (event.eventType == wxEVENT_TYPE_SET_FOCUS)
  {
    wxPanel *panel = (wxPanel *)text.GetParent();
    wxButton *but = (wxButton *)panel->GetDefaultItem();
    if (but)
      but->SetDefault();
  }
  else if (event.GetEventType() == wxEVENT_TYPE_TEXT_ENTER_COMMAND)
  {
    char *txt = wxTheClipsFrame->command_line->GetValue();
    if (txt)
    {
      char *comm = copystring(txt);
      long pos = wxTheClipsFrame->text_window->GetLastPosition();
      wxTheClipsFrame->text_window->SetInsertionPoint(pos);

      wxTheClipsFrame->text_window->WriteText("CLIPS> ");
      wxTheClipsFrame->text_window->WriteText(comm);
      wxTheClipsFrame->text_window->WriteText("\n");
      wxRouteCommand(comm);
      SetHaltExecution(CLIPS_FALSE);
      SetEvaluationError(CLIPS_FALSE);
      delete[] comm;
      PeriodicCleanup(CLIPS_TRUE,CLIPS_FALSE);
      if (wxTheClipsFrame)
      {
        wxTheClipsFrame->command_line->SetValue("");
        wxTheClipsFrame->command_line->SetFocus();
      }
      FlushOutputWindowBuffer();
    }
  }
//    EnterCommand((wxButton&)text, event);
}

// Define my frame constructor
wxClipsEnvFrame::wxClipsEnvFrame(wxFrame *frame, char *title, int x, int y, int w, int h):
  wxFrame(frame, title, x, y, w, h)
{
  panel = NULL;
  text_window = NULL;
#ifdef wx_msw
  toolbar = NULL;
#endif

  wxTheClipsFrame = this;
  // Give it a status line
  CreateStatusLine();
  char buf[200];
  sprintf(buf, "Welcome to wxCLIPS %.2f, a GUI environment for CLIPS", wxClipsVersion);
  SetStatusText(buf);

  // Give it an icon
#ifdef wx_msw
  wxIcon *icon = new wxIcon("wxClipsIcon");
  SetIcon(icon);
#endif
#ifdef wx_x
  wxIcon *icon = new wxIcon(aiai_bits, 64, 64);
  SetIcon(icon);
#endif

  // Make a menubar

  wxMenu *file_menu = new wxMenu;

  file_menu->Append(WXCLIPS_BATCH, "&Batch...", "Execute constructs and commands in batch file");
  file_menu->Append(WXCLIPS_LOAD, "&Load definitions...", "Load CLIPS constructs only");
  file_menu->Append(WXCLIPS_SAVE, "&Save definitions...", "Save CLIPS definitions");
  file_menu->AppendSeparator();
  file_menu->Append(WXCLIPS_BLOAD, "BL&oad...", "Load binary CLIPS definitions");
  file_menu->Append(WXCLIPS_BSAVE, "BS&ave...", "Save binary CLIPS definitions");
  file_menu->AppendSeparator();
  file_menu->Append(WXCLIPS_CLEAR_SCREEN, "&Clear screen", "Clear screen");
  file_menu->AppendSeparator();
  file_menu->Append(WXCLIPS_QUIT, "&Exit", "Exit from wxCLIPS");

  wxMenu *run_menu = new wxMenu;

  run_menu->Append(WXCLIPS_RESETRUN, "R&eset and Run", "Reset CLIPS and run");
  run_menu->Append(WXCLIPS_RUN, "&Run", "Run CLIPS");
  run_menu->Append(WXCLIPS_RUN_LIMIT, "Run with &limit...", "Run with limit");
  run_menu->Append(WXCLIPS_RESET, "Re&set", "Reset CLIPS");
  run_menu->AppendSeparator();
  run_menu->Append(WXCLIPS_CLEAR, "&Clear", "Clear CLIPS");
  run_menu->AppendSeparator();
  run_menu->Append(WXCLIPS_WATCHES, "&Watch...", "Select watches");
/*
  run_menu->Append(WXCLIPS_ADD_BREAKPOINT, "&Add breakpoint", "Add a breakpoint on selected rule");
  run_menu->Append(WXCLIPS_REMOVE_BREAKPOINT, "Re&move breakpoint", "Remove a breakpoint from selected rule");
*/
  run_menu->AppendSeparator();
//  run_menu->Append(WXCLIPS_SET_STRATEGY, "&Set strategy");
  run_menu->Append(WXCLIPS_ASSERT_FACT, "Assert &fact...", "Assert a fact");

  wxMenu *app_menu = new wxMenu;
  app_menu->Append(WXCLIPS_CLEAN_WINDOWS, "&Clean up windows", "Clean up user-created windows");
  app_menu->Append(WXCLIPS_RUN_APP, "&Start Application", "Call app-on-init");

  wxMenu *list_menu = new wxMenu;
#if !RUN_TIME
  list_menu->Append(WXCLIPS_LIST_RULES, "&Rules", "List rules");
  list_menu->Append(WXCLIPS_LIST_FACTS, "&Facts", "List facts");
  list_menu->Append(WXCLIPS_LIST_MATCHES, "&Matches", "List matches");
  list_menu->Append(WXCLIPS_LIST_AGENDA, "&Agenda", "List agenda");
  list_menu->Append(WXCLIPS_LIST_BREAKPOINTS, "&Breakpoints", "List breakpoints");
  list_menu->Append(WXCLIPS_LIST_TEMPLATES, "&Templates", "List templates");
  list_menu->Append(WXCLIPS_LIST_GLOBALS, "&Globals", "List globals");
//  list_menu->Append(WXCLIPS_LIST_STRATEGY, "&Current strategy");
#endif

  wxMenu *help_menu = new wxMenu;
  help_menu->Append(WXCLIPS_REFERENCE, "wxCLIPS &Reference", "Invoke wxCLIPS manual");
  help_menu->Append(WXCLIPS_CLIPS_HELP, "&CLIPS 6.0 Reference", "Invoke CLIPS manual");
  help_menu->AppendSeparator();
  help_menu->Append(WXCLIPS_ABOUT, "&About wxCLIPS", "About wxCLIPS");

  wxMenuBar *menu_bar = new wxMenuBar;

  menu_bar->Append(file_menu, "&File");
  menu_bar->Append(run_menu, "&Rules");
  menu_bar->Append(app_menu, "&Application");
  menu_bar->Append(list_menu, "&List");
  menu_bar->Append(help_menu, "&Help");

  // Associate the menu bar with the frame
  SetMenuBar(menu_bar);

#ifdef wx_x
  int toolbarHeight = 0;
#endif
#ifdef wx_msw
  int toolbarHeight = 28;
#endif
  int panelY = toolbarHeight;
  int textWindowY = panelY + COMMAND_PANEL_HEIGHT;
  
#ifdef wx_msw  
  // Create the toolbar
  toolbar = new wxClipsToolBar(this, 0, 0, -1, -1, 0,
                                        wxVERTICAL, 1);
  toolbar->SetMargins(2, 2);
  toolbar->GetDC()->SetBackground(wxLIGHT_GREY_BRUSH);

  int width = 24;
  int dx = 2;
  int gap = 10;
/* If we do a toolbar for X...
#else
  int width = ToolbarLoadBitmap->GetWidth();
  int dx = 2;
  int gap = 6;
#endif
*/
  int currentX = gap;
  toolbar->AddSeparator();
  toolbar->AddTool(DEVELOPMENT_TOOLBAR_LOAD_FILE, ToolbarLoadBitmap, NULL,
                   FALSE, (float)currentX, -1, NULL, "Load");
  currentX += width + dx;
  toolbar->AddTool(DEVELOPMENT_TOOLBAR_BATCH, ToolbarBatchBitmap, NULL,
                   FALSE, (float)currentX, -1, NULL, "Batch");
  currentX += width + dx + gap;
  toolbar->AddSeparator();
  toolbar->AddTool(DEVELOPMENT_TOOLBAR_RUN, ToolbarRunBitmap, NULL,
                   FALSE, (float)currentX, -1, NULL, "Run rules");
  currentX += width + dx;
  toolbar->AddTool(DEVELOPMENT_TOOLBAR_LAUNCH, ToolbarLaunchBitmap, NULL,
                   FALSE, (float)currentX, -1, NULL, "Launch app");
  currentX += width + dx;
  toolbar->AddTool(DEVELOPMENT_TOOLBAR_RESET, ToolbarResetBitmap, NULL,
                   FALSE, (float)currentX, -1, NULL, "Reset");
  currentX += width + dx + gap;
  toolbar->AddSeparator();
  toolbar->AddTool(DEVELOPMENT_TOOLBAR_COPY, ToolbarCopyBitmap, NULL,
                   FALSE, (float)currentX, -1, NULL, "Copy");
  currentX += width + dx;
  toolbar->AddTool(DEVELOPMENT_TOOLBAR_CLEAR, ToolbarClearBitmap, NULL,
                   FALSE, (float)currentX, -1, NULL, "Clear screen");
  currentX += width + dx + gap;
  toolbar->AddSeparator();
  toolbar->AddTool(DEVELOPMENT_TOOLBAR_HELP, ToolbarHelpBitmap, NULL,
                   FALSE, (float)currentX, -1, NULL, "Help");
  toolbar->CreateTools();
#endif

  // Make a text window
  panel = new wxPanel(this, 0, panelY, 500, COMMAND_PANEL_HEIGHT);
#if 0
  wxButton *button = new wxButton(panel, (wxFunction)EnterCommand,
     "Do");
  button->SetDefault();
#endif
//  command_line = new wxText(panel, (wxFunction)CommandTextItemProc, "CLIPS> ", "", -1, -1,
  int clX = -1;
#ifdef wx_msw
  clX = 2;
#endif
  command_line = new wxText(panel, (wxFunction)CommandTextItemProc, NULL, "", clX, -1,
                                  COMMAND_TEXT_WIDTH, -1, wxPROCESS_ENTER);

  text_window = new wxTextWindow(this, 0, textWindowY, 500, 400, wxBORDER|wxNATIVE_IMPL|wxREADONLY);

  SetSize(x, y, w, h);
  Centre(wxBOTH);
  Show(TRUE);
}

wxClipsEnvFrame::~wxClipsEnvFrame(void)
{
  wxTheClipsFrame = NULL;
}

// Intercept menu commands
char *wxBatchFile = NULL;
void wxClipsEnvFrame::OnMenuCommand(int id)
{
  switch (id)
  {
    case WXCLIPS_BATCH:
    {
      char *path = NULL;
      char *name = NULL;
      if (wxBatchFile)
      {
        path = PathOnly(wxBatchFile);
        name = FileNameFromPath(wxBatchFile);
      }

      char *file = wxFileSelector("CLIPS batch file to execute",
                     path, name,
                     "clp", "*.clp", 0, this);
      if (file)
      {
        file = copystring(file);
        wxExecuteClipsFile(file);
        if (wxBatchFile) delete[] wxBatchFile;
        wxBatchFile = file;
        FlushOutputWindowBuffer();
      }
      break;
    }
    case WXCLIPS_LOAD:
    {
      char *file = wxFileSelector("CLIPS file to load",
                     PathOnly(wxBatchFile), NULL,
                     "clp", "*.clp", 0, this);
      if (file)
      {
        file = copystring(file);
        SetPrintWhileLoading(CLIPS_TRUE);
        LoadConstructs(file);
        SetPrintWhileLoading(CLIPS_FALSE);
        if (wxBatchFile) delete[] wxBatchFile;
        wxBatchFile = file;
        FlushOutputWindowBuffer();
      }
      break;
    }
    case WXCLIPS_SAVE:
    {
      char *file = wxFileSelector("CLIPS file to save to",
                     PathOnly(wxBatchFile), NULL,
                     "clp", "*.clp", wxSAVE|wxOVERWRITE_PROMPT, this);
      if (file)
      {
        file = copystring(file);
	SaveConstructs(file);
        if (wxBatchFile) delete[] wxBatchFile;
        wxBatchFile = file;
        FlushOutputWindowBuffer();
      }
      break;
    }
    case WXCLIPS_BLOAD:
    {
      char *file = wxFileSelector("CLIPS file to binary load",
                     PathOnly(wxBatchFile), NULL,
                     "clp", "*.clp", 0, this);
      if (file)
      {
        file = copystring(file);
        SetPrintWhileLoading(CLIPS_TRUE);
	Bload(file);
        SetPrintWhileLoading(CLIPS_FALSE);
        if (wxBatchFile) delete[] wxBatchFile;
        wxBatchFile = file;
        FlushOutputWindowBuffer();
      }
      break;
    }
    case WXCLIPS_BSAVE:
    {
#if (!RUN_TIME && BLOAD_AND_BSAVE)
      char *file = wxFileSelector("CLIPS file to binary save to",
                     NULL, NULL,
                     "clp", "*.clp", wxSAVE|wxOVERWRITE_PROMPT, this);
      if (file)
      {
        file = copystring(file);
	Bsave(file);
        if (wxBatchFile) delete[] wxBatchFile;
        wxBatchFile = file;
        FlushOutputWindowBuffer();
      }
#endif
      break;
    }
    case WXCLIPS_RUN:
    {
      RunCLIPS(-1);
      FlushOutputWindowBuffer();
      break;
    }
    case WXCLIPS_RUN_LIMIT:
    {
      char *s = IntToString(limit);
      char *new_s = wxGetTextFromUser("Enter limit value: ",
                            "Run with limit",s, this);
      if (new_s)
      {
	StringToInt(new_s, &limit);
	RunCLIPS(limit);
        FlushOutputWindowBuffer();
      }
      if (s)
	delete s;
      break;
    }
    case WXCLIPS_RESETRUN:
    {
#ifdef CLIPS6
      Reset();
#else
      ResetCLIPS();
#endif
      RunCLIPS(-1);
      FlushOutputWindowBuffer();
      break;
    }
    case WXCLIPS_CLEAR:
    {
      ClearCLIPS();
      break;
    }
    case WXCLIPS_RESET:
    {
#ifdef CLIPS6
      Reset();
#else
      ResetCLIPS();
#endif
      break;
    }
    case WXCLIPS_WATCHES:
    {
      wxDialogBox dialog(this, "Watches", TRUE, 100, 100, 200, 200);
      (void)new wxMessage(&dialog, "Edit watches");
      dialog.NewLine();

      wxCheckBox check0(&dialog, NULL, "All");
      check0.SetValue(all_watch);

      dialog.NewLine();

      wxCheckBox check1(&dialog, NULL, "Facts");
      check1.SetValue(facts_watch);

      dialog.NewLine();

      wxCheckBox check2(&dialog, NULL, "Rules");
      check2.SetValue(rules_watch);

      dialog.NewLine();

      wxCheckBox check3(&dialog, NULL, "Activations");
      check3.SetValue(activations_watch);

      dialog.NewLine();

      wxCheckBox check4(&dialog, NULL, "Compilations");
      check4.SetValue(compilations_watch);

      dialog.NewLine();

      wxCheckBox check5(&dialog, NULL, "Statistics");
      check5.SetValue(statistics_watch);

      dialog.NewLine();

      wxCheckBox check6(&dialog, NULL, "Globals");
      check6.SetValue(globals_watch);

      dialog.NewLine();
      (void) new wxButton(&dialog, (wxFunction)GenericOk, "Ok");

      dialog.Fit();
      dialog.Show(TRUE);

      all_watch = check0.GetValue();
      facts_watch = check1.GetValue();
      rules_watch = check2.GetValue();
      activations_watch = check3.GetValue();
      compilations_watch = check4.GetValue();
      statistics_watch = check5.GetValue();
      globals_watch = check6.GetValue();
#ifdef CLIPS6
      if (all_watch)
        Watch("all");
      else Unwatch("all");

      if (facts_watch)
        Watch("facts");
      else Unwatch("facts");

      if (rules_watch)
        Watch("rules");
      else Unwatch("rules");

      if (activations_watch)
        Watch("activations");
      else Unwatch("activations");

      if (compilations_watch)
        Watch("compilations");
      else Unwatch("compilations");

      if (statistics_watch)
        Watch("statistics");
      else Unwatch("statistics");

      if (globals_watch)
        Watch("globals");
      else Unwatch("globals");
#else
      SetWatchItem("all", all_watch);
      SetWatchItem("facts", facts_watch);
      SetWatchItem("rules", rules_watch);
      SetWatchItem("activations", activations_watch);
      SetWatchItem("compilations", compilations_watch);
      SetWatchItem("statistics", statistics_watch);
      SetWatchItem("globals", globals_watch);
#endif
      break;
    }
    case WXCLIPS_ADD_BREAKPOINT:
    {
      char *name = PromptForRuleName();
      if (name)
      {
	void *ptr = FindDefrule(name);
	if (ptr)
	{
	  AddBreakpoint(ptr);
	  char buf[200];
	  sprintf(buf, "Put breakpoint on %s.\n", name);
	  PrintCLIPS("wdialog", buf);
	}
      }
      break;
    }
    case WXCLIPS_REMOVE_BREAKPOINT:
    {
      char *name = PromptForRuleName();
      if (name)
      {
	void *ptr = FindDefrule(name);
	if (ptr)
	{
          RemoveBreakpoint(ptr);
	  char buf[200];
          sprintf(buf, "Removed breakpoint on %s.\n", name);
	  PrintCLIPS("wdialog", buf);
	}
      }
      break;
    }
    case WXCLIPS_ASSERT_FACT:
    {
      char *s = wxGetTextFromUser(
        "Enter fact (no outer brackets): ", "Assert fact","", this);
      if (s)
      {
        char buf[300];
        sprintf(buf, "Asserting fact %s\n", s);
        PrintCLIPS("wdialog", buf);
        AssertString(s);
      }
      break;
    }
    case WXCLIPS_LIST_AGENDA:
    {
#if (!RUN_TIME && DEBUGGING_FUNCTIONS)
      ListAgenda();
#endif
      break;
    }
    case WXCLIPS_LIST_RULES:
    {
#if (!RUN_TIME && DEFRULE_CONSTRUCT && DEBUGGING_FUNCTIONS)
#ifdef CLIPS6
      ListDefrules("stdout", NULL);
#else
      ListDefrules();
#endif
#endif
      break;
    }
    case WXCLIPS_LIST_FACTS:
    {
#if (!RUN_TIME && DEFRULE_CONSTRUCT && DEBUGGING_FUNCTIONS)
#ifdef CLIPS6
      ListDeffacts("stdout", NULL);
#else
      ListDeffacts();
#endif
#endif
      break;
    }
    case WXCLIPS_LIST_TEMPLATES:
    {
#if (!RUN_TIME && DEFTEMPLATE_CONSTRUCT && DEBUGGING_FUNCTIONS)
#ifdef CLIPS6
      ListDeftemplates("stdout", NULL);
#else
      ListDeftemplates();
#endif
#endif
      break;
    }
    case WXCLIPS_LIST_GLOBALS:
    {
#if (!RUN_TIME && DEFGLOBAL_CONSTRUCT && DEBUGGING_FUNCTIONS)
#ifdef CLIPS6
      ListDefglobals("stdout", NULL);
#else
      ListDefglobals();
#endif
#endif
      break;
    }
    case WXCLIPS_LIST_BREAKPOINTS:
    {
#if (!RUN_TIME && DEBUGGING_FUNCTIONS)
      ListBreakpoints();
#endif
      break;
    }
    case WXCLIPS_CLEAR_SCREEN:
    {
      text_window->Clear();
      break;
    }
    case WXCLIPS_CLEAN_WINDOWS:
    {
      wxcCleanWindows();
      break;
    }
    case WXCLIPS_RUN_APP:
    {
      wxcCleanWindows();
      (void)wxStartApp();
      break;
    }
    case WXCLIPS_REFERENCE:
    {
      HelpInstance->LoadFile("wxclips");
      HelpInstance->DisplayContents();
      break;
    }
    case WXCLIPS_CLIPS_HELP:
    {
      HelpInstance->LoadFile("clips6");
      HelpInstance->DisplayContents();
      break;
    }

    case WXCLIPS_ABOUT:
    {
      char aboutBuffer[200];
#ifdef WIN32
      char *platform = "(WIN32) ";
#else
      char *platform = "";
#endif
#ifdef CLIPS6
#ifdef FUZZYCLIPS
      sprintf(aboutBuffer, "wxCLIPS %swith Fuzzy Extensions\n(c) NASA, NRCC and Julian Smart, AIAI 1996\nCLIPS Version " VERSION_STRING ", wxCLIPS Version %.2f",
              platform, wxClipsVersion);
#else
      sprintf(aboutBuffer, "wxCLIPS %s\n(c) NASA and Julian Smart, AIAI 1996\nCLIPS Version " VERSION_STRING ", wxCLIPS Version %.2f",
              platform, wxClipsVersion);
#endif
#else
      sprintf(aboutBuffer, "wxCLIPS %s\n(c) NASA and Julian Smart, AIAI 1996\nCLIPS Version 5.1, wxCLIPS Version %.2f",
              platform, wxClipsVersion);
#endif
      wxMessageBox(aboutBuffer, "About wxCLIPS");
      break;
    }
    case WXCLIPS_QUIT:
    {
      OnClose();
      delete this;
      break;
    }
  }
}

// Size the subwindows when the frame is resized
void wxClipsEnvFrame::OnSize(int w, int h)
{
#ifdef wx_x
  if (panel && text_window)
  {
    int width, height;
    GetClientSize(&width, &height);

    int textWidth, textHeight, textX, textY;
    command_line->GetSize(&textWidth, &textHeight);
    command_line->GetPosition(&textX, &textY);

    int newTextWidth = width - textX - 15;
    int panelHeight = wxMax((textHeight + textY + 5), 20);
    
    panel->SetSize(0, 0, width, panelHeight);
    text_window->SetSize(0, panelHeight, width, height-panelHeight);

    command_line->SetSize(-1, -1, newTextWidth, -1);
  }
#else
  if (panel && text_window && toolbar)
  {
    int width, height;
    GetClientSize(&width, &height);

#if WIN95
    int toolBarHeight = 26;
#else
    int toolBarHeight = 28;
#endif
    toolbar->SetSize(0, 0, width, toolBarHeight);

    int textWidth, textHeight, textX, textY;
    command_line->GetSize(&textWidth, &textHeight);
    command_line->GetPosition(&textX, &textY);

    int newTextWidth = width - textX - 5;
    int panelHeight = wxMax((textHeight + textY + 5), 20);
    
    panel->SetSize(0, toolBarHeight, width, panelHeight);
    text_window->SetSize(0, panelHeight + toolBarHeight, width, height-panelHeight-toolBarHeight);

    command_line->SetSize(-1, -1, newTextWidth, -1);
  }
#endif
}

// Define the behaviour for the frame closing
// - must delete all frames except for the main one.
Bool wxClipsEnvFrame::OnClose(void)
{
#ifdef wx_msw
  if (TheIDEServer)
    delete TheIDEServer;
#endif
  wxcCleanWindows();
  wxCleanupClips();
  wxIPCCleanUp();
  return TRUE;
}

void wxClipsEnvFrame::OnActivate(Bool active)
{
  if (active && command_line)
    command_line->SetFocus();
}

char *wxClipsEnvFrame::PromptForRuleName(void)
{
#ifdef CLIPS6
  return NULL;
#else
  wxStringList strings;
  void *ptr = GetNextDefrule(NULL);
  while (ptr)
  {
    char *name = GetDefruleName((struct constructHeader *)ptr);
    strings.Add(name);
    ptr = GetNextDefrule(ptr);
  }
  char **sarray = strings.ListToArray();
  char *the_name = wxGetSingleChoice("Select a rule", "Selection",
       strings.Number(), sarray, this);

  delete sarray;

  return the_name;
#endif
}


void GenericOk(wxButton& but, wxCommandEvent& event)
{
  wxDialogBox *dialog = (wxDialogBox *)but.GetParent();

  dialog->Show(FALSE);
}


// Compatibility
#ifdef wx_msw
int system(char *command)
{
  wxExecute(command);
  return 1;
}
#endif

// I/O Routers
int FindOutput(char *name)
{
  if ((strcmp(name, "wtrace") == 0) ||
      (strcmp(name, "stdout") == 0) ||
      (strcmp(name, "werror") == 0) ||
      (strcmp(name, "wclips") == 0) ||
      (strcmp(name, "wdisplay") == 0) ||
      (strcmp(name, "wagenda") == 0) ||
      (strcmp(name, "wdialog") == 0))
    return (CLIPS_TRUE);
  else
    return (CLIPS_FALSE);
}

int PrintOutput(char *name, char *str)
{
  if (wxTheClipsFrame)
    BufferedPrintOutput(str);
  return 1;
}

void BufferedPrintOutput(char *str)
{
  if (OutputWindowBuffering)
  {
    int len = strlen(str);
    int i = 0;
    while (i < len)
    {
      OutputWindowBuffer[OutputWindowBufferIndex] = str[i];
      if (str[i] == '\n')
      {
        OutputWindowBuffer[OutputWindowBufferIndex+1] = 0;

        long pos = wxTheClipsFrame->text_window->GetLastPosition();
        wxTheClipsFrame->text_window->SetInsertionPoint(pos);

        wxTheClipsFrame->text_window->WriteText(OutputWindowBuffer);
        OutputWindowBufferIndex = 0;
        i ++;
      }
      else
      {
        i ++;
        OutputWindowBufferIndex ++;
      }
      if (OutputWindowBufferIndex == OutputWindowBufferSize)
      {
        OutputWindowBuffer[OutputWindowBufferIndex] = 0;

        long pos = wxTheClipsFrame->text_window->GetLastPosition();
        wxTheClipsFrame->text_window->SetInsertionPoint(pos);

        wxTheClipsFrame->text_window->WriteText(OutputWindowBuffer);
        OutputWindowBufferIndex = 0;
      }
    }
  }
  else
  {
    long pos = wxTheClipsFrame->text_window->GetLastPosition();
    wxTheClipsFrame->text_window->SetInsertionPoint(pos);
    wxTheClipsFrame->text_window->WriteText(str);
  }
}

void FlushOutputWindowBuffer(void)
{
  if (OutputWindowBufferIndex > 0)
  {
    long pos = wxTheClipsFrame->text_window->GetLastPosition();
    wxTheClipsFrame->text_window->SetInsertionPoint(pos);

    OutputWindowBuffer[OutputWindowBufferIndex] = 0;
    wxTheClipsFrame->text_window->WriteText(OutputWindowBuffer);
    OutputWindowBufferIndex = 0;
  }
}

void SetOutputWindowBuffering(Bool on)
{
  OutputWindowBuffering = on;
}

int ExitOutput(int exitcode)
{
  return 1;
}

// I don't know why J.S.'s method of specifying routers blows up in 6...
// but this seems to work. RobW

int DummyGetc(char * c)
{
  return 1;
}

int DummyUngetc(int i, char * c)
{
  return 1;
}

int ClipsYield(void)
{
  wxYield();
  return 1;
}

/*
 * Functions specific to this front end
 *
 */
long clipsQuit()
{
  wxTheClipsFrame->OnClose();
  delete wxTheClipsFrame;
  return 1;
}

long clipsGetIDEWindow()
{
  if (wxTheClipsFrame)
  {
    if (!wxGetThingFromTable((long)wxTheClipsFrame))
      wxStoreThingInTable((long)wxTheClipsFrame, wxTheClipsFrame, CLASSINFO(wxFrame));
    return (long)wxTheClipsFrame;
  }
  else
    return 0;
}

void clipsShowIDEWindow()
{
  wxShowClipsWindow();
}

void clipsClearIDEWindow()
{
  if (wxTheClipsFrame)
    wxTheClipsFrame->text_window->Clear();
}

/*
 * Required by wx_cmds or CLIPS itself
 *
 */

void ExtensionErrorFunction(char *s)
{
  if (wxTheClipsFrame)
    *(wxTheClipsFrame->text_window) << s;
#ifdef wx_x
  else
    cerr << s;
#endif
}

#if FUZZY_DEFTEMPLATES
  
// extern "C" double my_sqr(void);

#endif  /* end of #if FUZZY_DEFTEMPLATES */

static char *ClipsWorkProc = NULL;
extern "C" void clipsSetWorkProc(void)
{
  if (ClipsWorkProc)
    delete[] ClipsWorkProc;
  ClipsWorkProc = NULL;
  char *s = RtnLexeme(1);
  if (s)
  {
    if (strcmp(s, "") == 0)
    {
    }
    else
    {
      ClipsWorkProc = new char[strlen(s) + 5];
      sprintf(ClipsWorkProc, "(%s)", s);
    }
  }
}


void UserFunctions()
{
  wxUserFunctions();

  DefineFunction("quit", 'l', PTIF clipsQuit, "clipsQuit");
  DefineFunction("get-ide-window", 'l', PTIF clipsGetIDEWindow, "clipsGetIDEWindow");
  DefineFunction("show-ide-window", 'v', PTIF clipsShowIDEWindow, "clipsShowIDEWindow");
  DefineFunction("clear-ide-window", 'v', PTIF clipsClearIDEWindow, "clipsClearIDEWindow");
  DefineFunction("set-work-proc", 'v', PTIF clipsSetWorkProc, "clipsSetWorkProc");

#if FUZZY_DEFTEMPLATES
  
/* functions for use with fuzzy modifiers - defined in FUZZYUTL.C\ */

//  DefineFunction2("sqr",'d', PTIF my_sqr, "my_sqr", "11n");

#endif  /* end of #if FUZZY_DEFTEMPLATES */

}


void wxClipsWorkProc(wxApp *app)
{
  if (ClipsWorkProc)
  {
    wxRouteCommand(ClipsWorkProc);
    SetHaltExecution(CLIPS_FALSE);
    SetEvaluationError(CLIPS_FALSE);
  }
}

/*
 * Toolbar (Windows only so far)
 *
 */

#ifdef wx_msw
wxClipsToolBar::wxClipsToolBar(wxFrame *frame, int x, int y, int w, int h,
            long style, int direction, int RowsOrColumns):
  wxButtonBar(frame, x, y, w, h, style, direction, RowsOrColumns)
{
}

Bool wxClipsToolBar::OnLeftClick(int toolIndex, Bool toggled)
{
  switch (toolIndex)
  {
    case DEVELOPMENT_TOOLBAR_LOAD_FILE:
    {
      wxTheClipsFrame->OnMenuCommand(WXCLIPS_LOAD);
      break;
    }
    case DEVELOPMENT_TOOLBAR_BATCH:
    {
      wxTheClipsFrame->OnMenuCommand(WXCLIPS_BATCH);
      break;
    }
    case DEVELOPMENT_TOOLBAR_RUN:
    {
      wxTheClipsFrame->OnMenuCommand(WXCLIPS_RUN);
      break;
    }
    case DEVELOPMENT_TOOLBAR_LAUNCH:
    {
      wxTheClipsFrame->OnMenuCommand(WXCLIPS_RUN_APP);
      break;
    }
    case DEVELOPMENT_TOOLBAR_RESET:
    {
      wxTheClipsFrame->OnMenuCommand(WXCLIPS_RESET);
      break;
    }
    case DEVELOPMENT_TOOLBAR_CLEAR:
    {
      wxTheClipsFrame->OnMenuCommand(WXCLIPS_CLEAR_SCREEN);
      break;
    }
    case DEVELOPMENT_TOOLBAR_COPY:
    {
      wxTheClipsFrame->text_window->Copy();
      break;
    }
    case DEVELOPMENT_TOOLBAR_HELP:
    {
      wxTheClipsFrame->OnMenuCommand(WXCLIPS_REFERENCE);
      break;
    }
    default:
      break;
  }
  return TRUE;
}

void wxClipsToolBar::OnMouseEnter(int toolIndex)
{
  if (!wxTheClipsFrame) return;
  
  if (toolIndex > -1)
  {
      switch (toolIndex)
      {
        case DEVELOPMENT_TOOLBAR_LOAD_FILE:
          wxTheClipsFrame->SetStatusText("Load CLIPS constructs");
          break;
        case DEVELOPMENT_TOOLBAR_BATCH:
          wxTheClipsFrame->SetStatusText("Batch CLIPS file");
          break;
        case DEVELOPMENT_TOOLBAR_RUN:
          wxTheClipsFrame->SetStatusText("Run rules");
          break;
        case DEVELOPMENT_TOOLBAR_LAUNCH:
          wxTheClipsFrame->SetStatusText("Launch CLIPS application (call app-on-init)");
          break;
        case DEVELOPMENT_TOOLBAR_RESET:
          wxTheClipsFrame->SetStatusText("Reset CLIPS");
          break;
        case DEVELOPMENT_TOOLBAR_CLEAR:
          wxTheClipsFrame->SetStatusText("Clear window");
          break;
        case DEVELOPMENT_TOOLBAR_COPY:
          wxTheClipsFrame->SetStatusText("Copy selected text to clipboard");
          break;
        case DEVELOPMENT_TOOLBAR_HELP:
          wxTheClipsFrame->SetStatusText("Display help contents");
          break;
        default:
          break;
      }
  }
  else wxTheClipsFrame->SetStatusText("");
}

void wxClipsToolBar::OnPaint(void)
{
  wxButtonBar::OnPaint();

  int w, h;
  GetSize(&w, &h);
  wxDC *dc = GetDC();
  dc->SetPen(wxBLACK_PEN);
  dc->SetBrush(wxTRANSPARENT_BRUSH);
  DrawLine(0, h-1, w, h-1);
}

#endif

#ifdef wx_msw
wxConnection *IDEServer::OnAcceptConnection(char *topic)
{
  if ((strcmp(topic, "STDIO") != 0) && (strcmp(topic, "WXCLIPS") == 0) && !TheIDEConnection)
    return new IDEConnection;
  else
    return NULL;
}

IDEConnection::IDEConnection(void)
{
  TheIDEConnection = this;
}

IDEConnection::~IDEConnection(void)
{
  TheIDEConnection = NULL;
}

Bool IDEConnection::OnExecute(char *topic, char *data, int size, int format)
{
  char command = data[0];

  switch (command)
  {
    // Load a file
    case 'L':
    {
      char *actualData = data + 2;
      SetPrintWhileLoading(CLIPS_TRUE);
      LoadConstructs(actualData);
      SetPrintWhileLoading(CLIPS_FALSE);
      if (wxBatchFile) delete wxBatchFile;
      wxBatchFile = copystring(actualData);
      FlushOutputWindowBuffer();
      break;
    }
    // Batch a file
    case 'B':
    {
      char *actualData = data + 2;
      wxExecuteClipsFile(actualData);
      if (wxBatchFile) delete wxBatchFile;
      wxBatchFile = copystring(actualData);
      FlushOutputWindowBuffer();
      break;
    }
    // Execute a CLIPS command
    case 'E':
    {
      char *actualData = data + 2;
      wxRouteCommand(actualData);
      SetHaltExecution(CLIPS_FALSE);
      SetEvaluationError(CLIPS_FALSE);
      FlushOutputWindowBuffer();
      break;
    }
    // Clear window
    case 'C':
    {
      if (wxTheClipsFrame)
        wxTheClipsFrame->text_window->Clear();
      break;
    }
    // Quit
    case 'Q':
    {
      if (wxTheClipsFrame)
      {
        wxFrame *frame = wxTheClipsFrame;
        wxTheClipsFrame->OnClose();
        delete frame;
      }
      break;
    }
    default:
    {
      wxMessageBox("Received unknown DDE command.", "Error", wxOK);
      break;
    }
  }
  return TRUE;
}

Bool IDEConnection::OnPoke(char *topic, char *item, char *data, int size, int format)
{
  return TRUE;
}

char *IDEConnection::OnRequest(char *topic, char *item, int *size, int format)
{
  return "";
}

#endif

// This has to be handled by the wxCLIPS IDE (not the wxCLIPS library)
// since it works with the wxApp defined by each application.
#if defined(wx_msw)
Bool MyApp::OnCharHook (wxKeyEvent& event)
{
  // First check for an active window, and try sending the message to that.
  // This is much easier than separately overriding OnCharHook for each window class.
  wxWindow *win = wxGetActiveWindow();
  if (win && wxGetThingFromTable((long)win))
  {
    wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxWindow), "OnCharHook");
    if (func)
    {
      wxStoreThingInTable ((long) &event, &event, CLASSINFO(wxKeyEvent));
      Bool handled = FALSE;
      Bool ret = (*(Bool (*)(Bool *, long, long)) func) (&handled, (long) win, (long)&event);
      wxDeleteThingInTable ((long) &event);

      if (handled)
        return ret;
    }
  }

  // Now check for handling characters for the whole app
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxApp), "OnCharHook");
  if (func)
    {
      wxStoreThingInTable ((long) &event, &event, CLASSINFO(wxKeyEvent));
      Bool handled = FALSE;
      Bool ret = (*(Bool (*)(Bool *, long, long)) func) (&handled, (long) this, (long)&event);
      wxDeleteThingInTable ((long) &event);

      if (!handled)
        return FALSE;
      return ret;
    }
  // Default processing
  return FALSE;
}
#endif
