/*
 * wxctbar.cc
 * wxWindows command processing: toolbar
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#include "wx_hash.h"
#include "wx_types.h"

#include "wx_tbar.h"

#ifdef wx_msw
#include "wx_bbar.h"
#endif

#if !defined(USE_IOSTREAMH) || USE_IOSTREAMH
#include <iostream.h>
#else
#include <iostream>
#endif
#include <ctype.h>
#include <stdlib.h>

#include "wx_cmds.h"
#include "wxcmdsp.h"

#ifdef USE_CLIPS
#ifndef __WATCOMC__
#include "clips.h"
#endif
#endif

// #include "wxctbarp.h"

extern "C" int clipsEchoOn;
extern ExtensionErrorClass ExtensionError;

Bool wxClipsToolBarClassOnLeftClick(Bool *handled, WXID id, int toolIndex, Bool toggleDown)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnLeftClick");
  if (func)
  {
    *handled = TRUE;
#ifdef USE_CLIPS
    char buf[100];
    clipsReturnType = clipsUNKNOWN;
    sprintf(buf, "(return-result (%s %ld %ld %ld))", func, id, (long)toolIndex, (long)toggleDown);
    wxRouteNoEcho(buf);
#else
    clipsReturnLong = (* (long (*)(long, int, Bool)) func) (id, (long)toolIndex, toggleDown);
    clipsReturnType = clipsLONG;
#endif
    if (clipsReturnType != clipsLONG)
    {
      ExtensionError << "OnLeftClick callback for toolbar should return an integer (1 or 0)!\n";
      return FALSE;
    }
    else return (Bool)clipsReturnLong;
  }
  return TRUE;
}

void wxClipsToolBarClassOnRightClick(Bool *handled, WXID id, int toolIndex, float x, float y)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnRightClick");
  if (func)
  {
    *handled = TRUE;
#ifdef USE_CLIPS
    char buf[100];
    sprintf(buf, "(return-result (%s %ld %ld %.2f %.2f))", func, id, (long)toolIndex, x, y);
    wxRouteNoEcho(buf);
#else
    (* (void (*)(long, int, double, double)) func) (id, toolIndex, (double)x, (double)y);
#endif
  }
}

void wxClipsToolBarClassOnMouseEnter(Bool *handled, WXID id, int toolIndex)
{
  wxClipsFunction func = wxClipsFindCallback(id, "OnMouseEnter");
  if (func)
  {
    *handled = TRUE;
#ifdef USE_CLIPS
    char buf[100];
    sprintf(buf, "(return-result (%s %ld %ld))", func, id, (long)toolIndex);
    wxRouteNoEcho(buf);
#else
    (* (void (*)(long, long)) func) (id, (long)toolIndex);
#endif
  }
}

#ifdef USE_CLIPS
long clipsToolbarCreate()
{
  if (ArgCountCheck("clipsToolbarCreate", AT_LEAST, 1) == -1)
    return 0;

  int no_args = RtnArgCount();

  long parent_id = RtnLong(1);

  // Set up defaults
  int x = -1;
  int y = -1;
  int width = -1;
  int height = -1;
  char *style = "";
  char *name = "toolBar";
  char *orientation = "wxVERTICAL";
  int rowsOrColumns = 2;
  Bool createButtons = 1;

  if (no_args > 1)
    x = (int) RtnLong(2);

  if (no_args > 2)
    y = (int) RtnLong(3);

  if (no_args > 3)
    width = (int) RtnLong(4);

  if (no_args > 4)
    height = (int) RtnLong(5);

  if (no_args > 5)
    style = RtnLexeme(6);

  if (no_args > 6)
    orientation = RtnLexeme(7);

  if (no_args > 7)
    rowsOrColumns = (int)RtnLong(8);

  if (no_args > 8)
    createButtons = (Bool)RtnLong(9);

  if (no_args > 9)
    name = RtnLexeme(10);

  return wxcToolbarCreate(parent_id, x, y, width, height, style, orientation, rowsOrColumns, createButtons, name);
}

long clipsToolbarAddTool()
{
  if (ArgCountCheck("clipsToolbarAddTool", AT_LEAST, 3) == -1)
    return 0;

  long id = RtnLong(1);
  int index = (int)RtnLong(2);
  long bitmapId1 = RtnLong(3);
  long bitmapId2 = 0;
  Bool toggle = FALSE;
  double x = -1.0;
  double y = -1.0;
  long clientData = 0;
  char *helpString1 = NULL;
  char *helpString2 = NULL;
  
  int no_args = RtnArgCount();

  if (no_args > 3)
    bitmapId2 = RtnLong(4);

  if (no_args > 4)
    toggle = (Bool) RtnLong(5);

  if (no_args > 5)
    x = RtnDouble(6);

  if (no_args > 6)
    y = RtnDouble(7);

  if (no_args > 7)
    clientData = RtnLong(8);

  if (no_args > 8)
    helpString1 = RtnLexeme(9);

  if (no_args > 9)
    helpString2 = RtnLexeme(10);

  if (helpString1 && strlen(helpString1) == 0)
    helpString1 = NULL;

  if (helpString2 && strlen(helpString2) == 0)
    helpString2 = NULL;

  return (long)wxcToolbarAddTool(id, index, bitmapId1, bitmapId2, toggle, x, y, clientData, helpString1, helpString2);
}

long clipsToolbarClearTools()
{
  if (ArgCountCheck("clipsToolbarClearTools", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcToolbarClearTools(RtnLong(1));
}

long clipsToolbarCreateTools()
{
  if (ArgCountCheck("clipsToolbarCreateTools", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcToolbarCreateTools(RtnLong(1));
}

long clipsToolbarAddSeparator()
{
  if (ArgCountCheck("clipsToolbarAddSeparator", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcToolbarAddSeparator(RtnLong(1));
}

long clipsToolbarEnableTool()
{
  if (ArgCountCheck("clipsToolbarEnableTool", EXACTLY, 3) == -1)
    return 0;

  return (long)wxcToolbarEnableTool(RtnLong(1), (int)RtnLong(2), (Bool)RtnLong(3));
}

long clipsToolbarToggleTool()
{
  if (ArgCountCheck("clipsToolbarToggleTool", EXACTLY, 3) == -1)
    return 0;

  return (long)wxcToolbarToggleTool(RtnLong(1), (int)RtnLong(2), (Bool)RtnLong(3));
}

char *clipsToolbarGetToolShortHelp()
{
  if (ArgCountCheck("clipsToolbarGetToolShortHelp", EXACTLY, 2) == -1)
    return (char *)AddSymbol("");

  char *s = wxcToolbarGetToolShortHelp(RtnLong(1), (int)RtnLong(2));
  if (s)
    return (char *)AddSymbol(s);
  else
    return (char *)AddSymbol("");
}

char *clipsToolbarGetToolLongHelp()
{
  if (ArgCountCheck("clipsToolbarGetToolLongHelp", EXACTLY, 2) == -1)
    return (char *)AddSymbol("");

  char *s = wxcToolbarGetToolLongHelp(RtnLong(1), (int)RtnLong(2));
  if (s)
    return (char *)AddSymbol(s);
  else
    return (char *)AddSymbol("");
}

long clipsToolbarSetToolShortHelp()
{
  if (ArgCountCheck("clipsToolbarSetToolShortHelp", EXACTLY, 3) == -1)
    return 0;

  return (long)wxcToolbarSetToolShortHelp(RtnLong(1), (int)RtnLong(2), RtnLexeme(3));
}

long clipsToolbarSetToolLongHelp()
{
  if (ArgCountCheck("clipsToolbarSetToolLongHelp", EXACTLY, 3) == -1)
    return 0;

  return (long)wxcToolbarSetToolLongHelp(RtnLong(1), (int)RtnLong(2), RtnLexeme(3));
}

long clipsToolbarLayout()
{
  if (ArgCountCheck("clipsToolbarLayout", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcToolbarLayout(RtnLong(1));
}

double clipsToolbarGetMaxWidth()
{
  if (ArgCountCheck("clipsToolbarGetMaxWidth", EXACTLY, 1) == -1)
    return 0;

  return wxcToolbarGetMaxWidth(RtnLong(1));
}

double clipsToolbarGetMaxHeight()
{
  if (ArgCountCheck("clipsToolbarGetMaxHeight", EXACTLY, 1) == -1)
    return 0;

  return wxcToolbarGetMaxHeight(RtnLong(1));
}

long clipsToolbarOnPaint()
{
  if (ArgCountCheck("clipsToolbarOnPaint", EXACTLY, 1) == -1)
    return 0;

  return (long)wxcToolbarOnPaint(RtnLong(1));
}

long clipsToolbarGetToolClientData()
{
  if (ArgCountCheck("clipsToolbarGetToolClientData", EXACTLY, 2) == -1)
    return 0;

  return wxcToolbarGetToolClientData(RtnLong(1), (int)RtnLong(2));
}

long clipsToolbarGetToolEnabled()
{
  if (ArgCountCheck("clipsToolbarGetToolEnabled", EXACTLY, 2) == -1)
    return 0;

  return (long)wxcToolbarGetToolEnabled(RtnLong(1), (int)RtnLong(2));
}

long clipsToolbarGetToolState()
{
  if (ArgCountCheck("clipsToolbarGetToolState", EXACTLY, 2) == -1)
    return 0;

  return (long)wxcToolbarGetToolState(RtnLong(1), (int)RtnLong(2));
}

long clipsToolbarSetMargins()
{
  if (ArgCountCheck("clipsToolbarSetMargins", EXACTLY, 3) == -1)
    return 0;

  return (long)wxcToolbarSetMargins(RtnLong(1), RtnDouble(2), RtnDouble(3));
}

long clipsToolbarSetDefaultSize()
{
  if (ArgCountCheck("clipsToolbarSetDefaultSize", EXACTLY, 3) == -1)
    return 0;

  return (long)wxcToolbarSetDefaultSize(RtnLong(1), (int)RtnLong(2), (int)RtnLong(3));
}
#endif

#ifdef USE_CLIPS
void wxToolbarUserFunctions()
{
  DefineFunction("toolbar-add-tool", 'l', PTIF clipsToolbarAddTool, "clipsToolbarAddTool");
  DefineFunction("toolbar-add-separator", 'l', PTIF clipsToolbarAddSeparator, "clipsToolbarAddSeparator");
  DefineFunction("toolbar-clear-tools", 'l', PTIF clipsToolbarClearTools, "clipsToolbarClearTools");
  DefineFunction("toolbar-create-tools", 'l', PTIF clipsToolbarCreateTools, "clipsToolbarCreateTools");
  DefineFunction("toolbar-create", 'l', PTIF clipsToolbarCreate, "clipsToolbarCreate");
  DefineFunction("toolbar-enable-tool", 'l', PTIF clipsToolbarEnableTool, "clipsToolbarEnableTool");
  DefineFunction("toolbar-toggle-tool", 'l', PTIF clipsToolbarToggleTool, "clipsToolbarToggleTool");
  DefineFunction("toolbar-get-tool-state", 'l', PTIF clipsToolbarGetToolState, "clipsToolbarGetToolState");
  DefineFunction("toolbar-get-tool-enabled", 'l', PTIF clipsToolbarGetToolEnabled, "clipsToolbarGetToolEnabled");
  DefineFunction("toolbar-get-max-width", 'd', PTIF clipsToolbarGetMaxWidth, "clipsToolbarGetMaxWidth");
  DefineFunction("toolbar-get-max-height", 'd', PTIF clipsToolbarGetMaxHeight, "clipsToolbarGetMaxHeight");
  DefineFunction("toolbar-get-tool-client-data", 'l', PTIF clipsToolbarGetToolClientData,
     "clipsToolbarGetToolClientData");
  DefineFunction("toolbar-get-tool-short-help", 's', PTIF clipsToolbarGetToolShortHelp,
     "clipsToolbarGetToolShortHelp");
  DefineFunction("toolbar-get-tool-long-help", 's', PTIF clipsToolbarGetToolLongHelp,
     "clipsToolbarGetToolLongHelp");
  DefineFunction("toolbar-set-tool-short-help", 'l', PTIF clipsToolbarSetToolShortHelp,
     "clipsToolbarGetToolShortHelp");
  DefineFunction("toolbar-set-tool-long-help", 'l', PTIF clipsToolbarSetToolLongHelp,
     "clipsToolbarGetToolLongHelp");
  DefineFunction("toolbar-layout", 'l', PTIF clipsToolbarLayout, "clipsToolbarLayout");
  DefineFunction("toolbar-on-paint", 'l', PTIF clipsToolbarOnPaint, "clipsToolbarOnPaint");
  DefineFunction("toolbar-set-margins", 'l', PTIF clipsToolbarSetMargins, "clipsToolbarSetMargins");
  DefineFunction("toolbar-set-default-size", 'l', PTIF clipsToolbarSetDefaultSize, "clipsToolbarSetDefaultSize");
}
#endif
