/*
 * wxcdc.cc
 * Command processing: device context
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#include "wx_hash.h"
#include "wx_types.h"
#ifdef wx_msw
#include "wx_mf.h"
#endif

#if !defined(USE_IOSTREAMH) || USE_IOSTREAMH
#include <iostream.h>
#else
#include <iostream>
#endif
#include <ctype.h>
#include <stdlib.h>

#include "wxextend.h"

/*
 * Device context functions
 */


long wxcDCGetSize(WXID id, float *w, float *h)
{
    wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
    if (dc) {
        dc->GetSize(w, h);
        return 1;
    }
    else {
        ExtensionError << "Error in dc-get-size: could not find dc "
                 << id << ".\n";
        return 0;
    }
}

Bool wxcDCSetMapMode(WXID id, int mode)
{
    wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
    if (dc) {
        dc->SetMapMode(mode);
        return TRUE;
    }
    ExtensionError << "Error in dc-get-size: could not find dc " << id << ".\n";
    return FALSE;
}

// String Version
Bool wxcDCSetMapMode(WXID id, char *mode_string)
{
    wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
    if (dc) {
        long mode = wxExtensionParseBitList(mode_string);
        dc->SetMapMode(mode);
        return TRUE;
    }
    ExtensionError << "Error in dc-get-size: could not find dc " << id << ".\n";
    return FALSE;
}


long wxcDCDrawArc(WXID id, float x1, float y1, float x2, float y2,
             float xc, float yc)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->DrawArc(x1, y1, x2, y2, xc, yc);
    return 1;
  }
  else
  {
    ExtensionError << "Error in dc-draw-ellipse: could not find dc " << id << ".\n";
    return 0;
  }
}

Bool wxcDCDrawIcon(WXID id, WXID icon_id, float x, float y)
{
    wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
    if (dc) {
	wxIcon *icon = (wxIcon *)wxGetTypedObject(icon_id, CLASSINFO(wxIcon));
	if (icon) {
          dc->DrawIcon(icon, x, y);
          return TRUE;
	}
        ExtensionError << "Error in dc-draw-icon: could not find icon " << id << ".\n";
    }
    else
      ExtensionError << "Error in dc-draw-icon: could not find dc " << id << ".\n";
    return FALSE;
}

long wxcDCGetTextExtent(WXID id, char* text, float* width, float* height)
{
    wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
    if (dc) {
        dc->GetTextExtent(text, width, height);
        return 1;
    }
    else {
        ExtensionError << "Error in dc-get-text-extent: could not find dc " << id << ".\n";
        return 0;
    }
}


long wxcDCBeginDrawing(WXID id)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->BeginDrawing();
    return 1;
  }
  else
  {
    ExtensionError << "Error in dc-begin-drawing: could not find dc " << id << ".\n";
    return 0;
  }
}

long wxcDCEndDrawing(WXID id)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->EndDrawing();
    return 1;
  }
  else
  {
    ExtensionError << "Error in dc-end-drawing: could not find dc " << id << ".\n";
    return 0;
  }
}

double wxcDCGetWidth(WXID id)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    float x, y;
    dc->GetSize(&x, &y);
    return x;
  }
  else
  {
    ExtensionError << "Error in dc-get-width: could not find dc " << id << ".\n";
    return 0.0;
  }
}

double wxcDCGetHeight(WXID id)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    float x, y;
    dc->GetSize(&x, &y);
    return y;
  }
  else
  {
    ExtensionError << "Error in dc-get-height: could not find dc " << id << ".\n";
    return 0.0;
  }
}


// Min X
double wxcDCGetMinX(WXID id)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    return dc->MinX();
  }
  else
  {
    ExtensionError << "Error in dc-get-min-x: could not find dc " << id << ".\n";
    return 0.0;
  }
}

// Min Y
double wxcDCGetMinY(WXID id)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    return dc->MinY();
  }
  else
  {
    ExtensionError << "Error in dc-get-min-y: could not find dc " << id << ".\n";
    return 0.0;
  }
}

// Max X
double wxcDCGetMaxX(WXID id)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    return dc->MaxX();
  }
  else
  {
    ExtensionError << "Error in dc-get-max-x: could not find dc " << id << ".\n";
    return 0.0;
  }
}

// Max Y
double wxcDCGetMaxY(WXID id)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    return dc->MaxY();
  }
  else
  {
    ExtensionError << "Error in dc-get-max-y: could not find dc " << id << ".\n";
    return 0.0;
  }
}


// Is the DC ok?
Bool wxcDCOk(WXID id)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    return dc->Ok();
  }
  else
  {
    ExtensionError << "Error in dc-ok: could not find dc " << id << ".\n";
    return 0;
  }
}

// Set pen

Bool wxcDCSetPen(WXID id, WXID penId)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    if (penId == 0)
    {
      dc->SetPen(NULL);
      return TRUE;
    }
    else
    {
      wxPen *pen = (wxPen *)wxGetTypedObject(penId, CLASSINFO(wxPen));
      if (pen)
      {
        dc->SetPen(pen);
        return TRUE;
      }
      else
      {
        ExtensionError << "Error in dc-set-pen: could not find pen " << penId << ".\n";
        return FALSE;
      }
    }
  }
  else
  {
    ExtensionError << "Error in dc-set-pen: could not find dc " << id << ".\n";
    return FALSE;
  }
}


Bool wxcDCSetBrush(WXID id, WXID brushId)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    if (brushId == 0)
    {
      dc->SetBrush(NULL);
      return TRUE;
    }
    else
    {
      wxBrush *brush = (wxBrush *)wxGetTypedObject(brushId, CLASSINFO(wxBrush));
      if (brush)
      {
        dc->SetBrush(brush);
        return TRUE;
      }
      else
      {
        ExtensionError << "Error in dc-set-brush: could not find brush " << brushId << ".\n";
        return FALSE;
      }
    }
  }
  else
  {
    ExtensionError << "Error in dc-set-brush: could not find dc " << id << ".\n";
    return FALSE;
  }
}


Bool wxcDCSetBackground(WXID id, WXID brushId)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    if (brushId == 0)
    {
      dc->SetBackground(NULL);
      return TRUE;
    }
    else
    {
      wxBrush *brush = (wxBrush *)wxGetTypedObject(brushId, CLASSINFO(wxBrush));
      if (brush)
      {
        dc->SetBackground(brush);
        return TRUE;
      }
      else
      {
        ExtensionError << "Error in dc-set-background: could not find brush " << brushId << ".\n";
        return FALSE;
      }
    }
  }
  else
  {
    ExtensionError << "Error in dc-set-background: could not find dc " << id << ".\n";
    return FALSE;
  }
}


Bool wxcDCSetFont(WXID id, WXID fontId)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    if (fontId == 0)
    {
      dc->SetFont(NULL);
      return TRUE;
    }
    else
    {
      wxFont *font = (wxFont *)wxGetTypedObject(fontId, CLASSINFO(wxFont));
      if (font)
      {
        dc->SetFont(font);
        return TRUE;
      }
      else
      {
        ExtensionError << "Error in dc-set-font: could not find font " << fontId << ".\n";
        return FALSE;
      }
    }
  }
  else
  {
    ExtensionError << "Error in dc-set-font: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCSetColourMap(WXID id, long cmapId)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    if (cmapId == FALSE)
    {
      dc->SetColourMap(NULL);
      return TRUE;
    }
    else
    {
      wxColourMap *cmap = (wxColourMap *)wxGetTypedObject(cmapId, CLASSINFO(wxColourMap));
      if (cmap)
      {
        dc->SetColourMap(cmap);
        return TRUE;
      }
      else
      {
        ExtensionError << "Error in dc-set-colourmap: could not find colourmap " << cmapId << ".\n";
        return FALSE;
      }
    }
  }
  else
  {
    ExtensionError << "Error in dc-set-colourmap: could not find dc " << id << ".\n";
    return FALSE;
  }
}


Bool wxcDCSetTextForeground(WXID id, char *colour)
{
  if (!colour)
    return FALSE;

  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    wxColour *col = wxTheColourDatabase->FindColour(colour);
    if (col)
    {
      dc->SetTextForeground(col);
      return TRUE;
    }
    else
    {
      ExtensionError << "Error in dc-set-text-foreground: could not find colour " << colour << ".\n";
      return FALSE;
    }
  }
  else
  {
    ExtensionError << "Error in dc-set-text-foreground: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCSetTextBackground(WXID id, char *colour)
{
  if (!colour)
    return FALSE;

  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    wxColour *col = wxTheColourDatabase->FindColour(colour);
    if (col)
    {
      dc->SetTextBackground(col);
      return TRUE;
    }
    else
    {
      ExtensionError << "Error in dc-set-text-background: could not find colour " << colour << ".\n";
      return FALSE;
    }
  }
  else
  {
    ExtensionError << "Error in dc-set-text-background: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCSetBackgroundMode(WXID id, int mode)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->SetBackgroundMode(mode);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-set-background-mode: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCSetBackgroundMode(WXID id, char *mode)
{
  if (!mode)
    return FALSE;
    
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    int intMode = wxSOLID;
    if (strcmp(mode, "wxSOLID") == 0)
      intMode = wxSOLID;
    else if (strcmp(mode, "wxTRANSPARENT") == 0)
      intMode = wxTRANSPARENT;
    else
    {
      ExtensionError << "Error in dc-set-background-mode: argument should be wxSOLID or wxTRANSPARENT.\n";
      return FALSE;
    }      
    
    dc->SetBackgroundMode(intMode);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-set-background-mode: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCSetUserScale(WXID id, double scaleX, double scaleY)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->SetUserScale((float)scaleX, (float)scaleY);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-set-user-scale: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCSetDeviceOrigin(WXID id, double x, double y)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->SetDeviceOrigin((float)x, (float)y);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-set-device-origin: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCSetLogicalOrigin(WXID id, double x, double y)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->SetLogicalOrigin((float)x, (float)y);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-set-logical-origin: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCSetLogicalFunction(WXID id, int op)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->SetLogicalFunction(op);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-set-logical-function: could not find dc " << id << ".\n";
    return FALSE;
  }
}

// Set logical function: string version
Bool wxcDCSetLogicalFunction(WXID id, char *op)
{
  if (!op)
    return FALSE;

  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    int log_op = 0;
    
    if (strcmp(op, "wxCOPY") == 0)
      log_op = wxCOPY;
    else if (strcmp(op, "wxXOR") == 0)
      log_op = wxXOR;
    else if (strcmp(op, "wxINVERT") == 0)
      log_op = wxINVERT;
    else if (strcmp(op, "wxOR_REVERSE") == 0)
      log_op = wxOR_REVERSE;
    else if (strcmp(op, "wxAND_REVERSE") == 0)
      log_op = wxAND_REVERSE;
    else
    {
      ExtensionError << "Error in dc-set-logical-function: unrecognized function " << op << "\n";
      ExtensionError << "Function must be one of wxCOPY, wxXOR, wxINVERT, wxOR_REVERSE, wxAND_REVERSE.\n";
      return 0;
    }
    dc->SetLogicalFunction(log_op);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-set-logical-function: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCBlit(WXID id, double xdest, double ydest, double width, double height, long sourceId,
  double xsrc, double ysrc, int op)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  wxDC *sourceDC = (wxDC *)wxGetTypedObject(sourceId, CLASSINFO(wxDC));
  if (dc && sourceDC)
  {
    dc->Blit((float)xdest, (float)ydest, (float)width, (float)height, (wxCanvasDC *)sourceDC, (float)xsrc, (float)ysrc, op);
    return 1;
  }
  else
  {
    ExtensionError << "Error in dc-blit: could not find one or both device contexts, " << id << " or " << sourceId << ".\n";
    return 0;
  }
}

// Blit: string version
Bool wxcDCBlit(WXID id, double xdest, double ydest, double width, double height, WXID sourceId,
  double xsrc, double ysrc, char *logicalFuncString)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  wxDC *sourceDC = (wxDC *)wxGetTypedObject(sourceId, CLASSINFO(wxDC));
  if (dc && sourceDC)
  {
    long op = wxExtensionParseBitList(logicalFuncString);
    dc->Blit((float)xdest, (float)ydest, (float)width, (float)height, (wxCanvasDC *)sourceDC, (float)xsrc, (float)ysrc, (int)op);
    return 1;
  }
  else
  {
    ExtensionError << "Error in dc-blit: could not find one or both device contexts, " << id << " or " << sourceId << ".\n";
    return 0;
  }
}

Bool wxcDCDrawLine(WXID id, double x1, double y1, double x2, double y2)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->DrawLine((float)x1, (float)y1, (float)x2, (float)y2);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-draw-line: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCDrawPoint(WXID id, double x1, double y1)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->DrawPoint((float)x1, (float)y1);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-draw-point: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCDrawRectangle(WXID id, double x1, double y1, double x2, double y2)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->DrawRectangle((float)x1, (float)y1, (float)x2, (float)y2);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-draw-rectangle: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCDrawRoundedRectangle(WXID id, double x1, double y1, double x2, double y2,
  double radius)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->DrawRoundedRectangle((float)x1, (float)y1, (float)x2, (float)y2, (float)radius);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-draw-rounded-rectangle: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCDrawEllipse(WXID id, double x, double y, double width, double height)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->DrawEllipse((float)x, (float)y, (float)width, (float)height);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-draw-ellipse: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCDrawText(WXID id, char *text, double x1, double y1)
{
  if (!text)
    return FALSE;

  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->DrawText(text, (float)x1, (float)y1);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-draw-text: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCSetClippingRegion(WXID id, double x1, double y1, double x2, double y2)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->SetClippingRegion((float)x1, (float)y1, (float)x2, (float)y2);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-set-clipping-region: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCDestroyClippingRegion(WXID id)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->DestroyClippingRegion();
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-destroy-clipping-region: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCClear(WXID id)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->Clear();
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-clear: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCStartDoc(WXID id, char *message)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    return dc->StartDoc(message);
  }
  else
  {
    ExtensionError << "Error in dc-start-doc: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCEndDoc(WXID id)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->EndDoc();
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-end-doc: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCStartPage(WXID id)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->StartPage();
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-start-page: could not find dc " << id << ".\n";
    return FALSE;
  }
}

Bool wxcDCEndPage(WXID id)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    dc->EndPage();
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-end-page: could not find dc " << id << ".\n";
    return FALSE;
  }
}

// Delete a device context
Bool wxcDCDelete(WXID id)
{
  wxDC *dc = (wxDC *)wxGetTypedObject(id, CLASSINFO(wxDC));
  if (dc)
  {
    if (dc->GetClassInfo() == CLASSINFO(wxCanvasDC))
    {
      ExtensionError << "dc-delete: cannot delete a canvas device context.\n";
      return FALSE;
    }
    wxDeleteThingInTable(id);
    delete dc;
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in dc-delete: could not find device context " << id << ".\n";
  }
  return FALSE;
}

// Create a PostScript device context
WXID wxcPostScriptDCCreate(char *filename, Bool interactive, WXID parentId)
{
  if (filename && (strlen(filename) == 0))
    filename = NULL;
    
  wxWindow *parent = NULL;
  if (parentId != 0)
  {
    parent = (wxWindow *)wxGetTypedObject(parentId, CLASSINFO(wxWindow));
    if (!parent)
    {
      ExtensionError << "Error in postscript-dc-create: could not find parent window " << parentId << ".\n";
      return 0;
    }
  }

  // If this causes an error, you may be using an earlier version
  // of wxWindows that this was written for: please delete the final
  // argument to this constructor.
  wxPostScriptDC *dc = new wxPostScriptDC(filename, (Bool)interactive, parent);

  wxStoreThingInTable((long)dc, dc, CLASSINFO(wxPostScriptDC));
  return (long)dc;
}

// Create a memory device context
long wxcMemoryDCCreate(void)
{
  wxMemoryDC *dc = new wxMemoryDC;
  wxStoreThingInTable((long)dc, dc, CLASSINFO(wxMemoryDC));
  return (long)dc;
}

// Select a bitmap into a memory device context
Bool wxcMemoryDCSelectObject(WXID id, WXID bitmapId)
{
  wxMemoryDC *dc = (wxMemoryDC *)wxGetTypedObject(id, CLASSINFO(wxMemoryDC));
  if (dc)
  {
    if (bitmapId == 0)
    {
      dc->SelectObject(NULL);
      return TRUE;
    }
    else
    {
      wxBitmap *bitmap = (wxBitmap *)wxGetTypedObject(bitmapId, CLASSINFO(wxBitmap));
      if (bitmap)
      {
        dc->SelectObject(bitmap);
        return TRUE;
      }
      else
        ExtensionError << "Error in memory-dc-select-object: could not find bitmap " << bitmapId << ".\n";
    }
  }
  else
  {
    ExtensionError << "Error in memory-dc-select-object: could not find memory device context " << id << ".\n";
  }
  return FALSE;
}

#ifdef wx_msw
// Create a Windows Printer device context
WXID wxcPrinterDCCreate(char *driver, char *device, char *filename, Bool interactive)
{
  if (filename && (strlen(filename) == 0))
    filename = NULL;
    
  wxPrinterDC *dc = new wxPrinterDC(driver, device, filename, (Bool)interactive);

  wxStoreThingInTable((long)dc, dc, CLASSINFO(wxPrinterDC));
  return (long)dc;
}

// Create a Windows Metafile device context
WXID wxcMetaFileDCCreate(char *filename)
{
  if (filename && (strlen(filename) == 0))
    filename = NULL;
    
  wxMetaFileDC *dc = new wxMetaFileDC(filename);

  wxStoreThingInTable((long)dc, dc, CLASSINFO(wxMetaFileDC));
  return (long)dc;
}

// Close a metafile DC, returning a wxMetaFile object
WXID wxcMetaFileDCClose(WXID id)
{
  wxMetaFileDC *dc = (wxMetaFileDC *)wxGetTypedObject(id, CLASSINFO(wxMetaFileDC));
  if (dc)
  {
    wxMetaFile *metaFile = dc->Close();
    if (!metaFile)
      return 0;
    else
    {
      wxStoreThingInTable((long)metaFile, metaFile, CLASSINFO(wxMetaFile));
      return (long)metaFile;
    }
  }
  else
  {
    ExtensionError << "Error in metafile-dc-close: could not find device context " << id << ".\n";
  }
  return 0;
}

// Delete a metafile
Bool wxcMetaFileDelete(WXID id)
{
  wxMetaFile *mf = (wxMetaFile *)wxGetTypedObject(id, CLASSINFO(wxMetaFile));
  if (mf)
  {
    wxDeleteThingInTable(id);
    delete mf;
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in metafile-delete: could not find metafile " << id << ".\n";
  }
  return FALSE;
}

// Set metafile to clipboard
Bool wxcMetaFileSetClipboard(WXID id, double width, double height)
{
  wxMetaFile *mf = (wxMetaFile *)wxGetTypedObject(id, CLASSINFO(wxMetaFile));
  if (mf)
  {
    return mf->SetClipboard((int)width, (int)height);
  }
  else
  {
    ExtensionError << "Error in metafile-set-clipboard: could not find metafile " << id << ".\n";
  }
  return FALSE;
}

// Make a metafile placeable
Bool wxcMakeMetaFilePlaceable(char *filename, int minX, int minY, int maxX, int maxY, double scale)
{
  return wxMakeMetaFilePlaceable(filename, minX, minY, maxX, maxY, (float)scale);
}

#endif // wx_msw

