/*
 * exhtml.cc
 * wxExtend wxHtml class
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#include "wxextend.h"

#if USE_WXHTML

#include "wxwww.h"
#include "wxstring.h"

#include "exhtml.h"


class wxExtensionHtml: public HTML_CLASS
{
 public:
  wxExtensionHtml (wxFrame * parent,
		    int x = -1, int y = -1,
		    int width = -1, int height = -1, long style = 0, char *name = "html"):
    HTML_CLASS (parent, x, y, width, height)
  {
    wxStoreThingInTable ((long) this, this);
    wxCanvasDC *dc = GetDC ();
    if (dc)
      wxStoreThingInTable ((long) dc, dc);
  }
/*
  wxExtensionHtml (void)
  {
    wxStoreThingInTable ((long) this, this);
  }
*/
   ~wxExtensionHtml ()
  {
    wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxHtml), "OnDelete");
    if (func)
      (*(void (*)(long)) func) ((long) this);

    wxDeleteThingInTable ((long) this);
    wxCanvasDC *dc = GetDC ();
    wxDeleteThingInTable ((long) dc);
  }

  void OnSize (int w, int h);
  Bool OnUserOpenURL(const char *url);
  void OnSetStatusText(const char*, int);

//  void OnPaint (void);
};

void wxExtensionHtml::OnSize (int w, int h)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxHtml), "OnSize");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, int, int)) func) (&handled, (long) this, w, h);

      if (!handled)
        HTML_CLASS::OnSize (w, h);
    }
  else
    HTML_CLASS::OnSize (w, h);
}

Bool wxExtensionHtml::OnUserOpenURL (const char *url)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxHtml), "OnOpenURL");
  if (func)
    {
      Bool handled = FALSE;
      Bool returnValue = (*(Bool (*)(Bool *, long, char *)) func) (&handled, (long) this, (char *)url);

      if (!handled && !returnValue)
        return TRUE;
      else
        return returnValue;
    }
  else
    return TRUE;
}

void wxExtensionHtml::OnSetStatusText (const char *url, int)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxHtml), "OnSetStatusText");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long, char *)) func) (&handled, (long) this, (char *)url);

      // You may need to change wxhtml.h and wxwww.h in utils/wxweb/src
      // to make OnSetStatusText a public function.
      if (!handled)
        HTML_CLASS::OnSetStatusText(url, 0);
    }
  else
    HTML_CLASS::OnSetStatusText(url, 0);
}

/*
void wxExtensionHtml::OnPaint (void)
{
  wxExtensionFunction func = wxFindExtensionCallback (CLASSINFO(wxHtml), "OnPaint");
  if (func)
    {
      Bool handled = FALSE;
      (*(void (*)(Bool *, long)) func) (&handled, (long) this);

      if (!handled)
        HTML_CLASS::OnPaint ();
    }
  else
    HTML_CLASS::OnPaint ();
}
*/

// Create a new html
long wxcHtmlCreate (long parentId, int x, int y, int width, int height, long style, char *name)
{
  wxFrame *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxFrame *) wxGetTypedObject (parentId, CLASSINFO(wxFrame));
      if (!parent)
	{
	  ExtensionError << "Error in html-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }

  wxExtensionHtml *html = new wxExtensionHtml (parent, (int) x, (int) y, (int) width, (int) height, style, name);
  return (long) html;
}

// Grid creation (string version)
long wxcHtmlCreate (long parentId, int x, int y, int width, int height, char *styleString, char *name)
{
  wxFrame *parent = NULL;

  if (parentId != 0)
    {
      parent = (wxFrame *) wxGetTypedObject (parentId, CLASSINFO(wxFrame));
      if (!parent)
	{
	  ExtensionError << "Error in html-create: could not find parent window " << parentId << ".\n";
	  return 0;
	}
    }
  long style = wxExtensionParseBitList (styleString);

  wxExtensionHtml *html = new wxExtensionHtml (parent, (int) x, (int) y, (int) width, (int) height, style, name);
  return (long) html;
}

Bool wxcHtmlOpenURL (WXID id, char *url)
{
  if (!url)
    return FALSE;
    
  HTML_CLASS *html = (HTML_CLASS *) wxGetTypedObject (id, CLASSINFO(wxHtml));
  if (!html)
  {
    ExtensionError << "Error in html-open-url: could not find html " << id << ".\n";
    return FALSE;
  }
  return html->OpenURL(url);
}

Bool wxcHtmlOpenFile (WXID id, char *file)
{
  if (!file)
    return FALSE;
    
  HTML_CLASS *html = (HTML_CLASS *) wxGetTypedObject (id, CLASSINFO(wxHtml));
  if (!html)
  {
    ExtensionError << "Error in html-open-file: could not find html " << id << ".\n";
    return FALSE;
  }
  wxString str("file:///");
  str += file;
  return html->OpenURL(str.GetData());
}

Bool wxcHtmlSaveFile (WXID id, char *file)
{
  if (!file)
    return FALSE;
    
  HTML_CLASS *html = (HTML_CLASS *) wxGetTypedObject (id, CLASSINFO(wxHtml));
  if (!html)
  {
    ExtensionError << "Error in html-save-file: could not find html " << id << ".\n";
    return FALSE;
  }
  return html->SaveFile(file);
}

char *wxcHtmlGetCurrentURL (WXID id)
{
  HTML_CLASS *html = (HTML_CLASS *) wxGetTypedObject (id, CLASSINFO(wxHtml));
  if (!html)
  {
    ExtensionError << "Error in html-get-current-url: could not find html " << id << ".\n";
    return NULL;
  }
  return (char *)(html->GetCurrentURL().data());
}

void wxcHtmlResize (WXID id)
{
  HTML_CLASS *html = (HTML_CLASS *) wxGetTypedObject (id, CLASSINFO(wxHtml));
  if (!html)
  {
    ExtensionError << "Error in html-resize: could not find html " << id << ".\n";
    return;
  }
  html->Reload();
}

Bool wxcHtmlBack (WXID id)
{
  HTML_CLASS *html = (HTML_CLASS *) wxGetTypedObject (id, CLASSINFO(wxHtml));
  if (!html)
  {
    ExtensionError << "Error in html-back: could not find html " << id << ".\n";
    return FALSE;
  }
  return (Bool)html->Back();
}

void wxcHtmlClearCache (WXID id)
{
  HTML_CLASS *html = (HTML_CLASS *) wxGetTypedObject (id, CLASSINFO(wxHtml));
  if (!html)
  {
    ExtensionError << "Error in html-clear-cache: could not find html " << id << ".\n";
    return;
  }
  html->ClearCache();
}

void wxcHtmlCancel (WXID id)
{
  HTML_CLASS *html = (HTML_CLASS *) wxGetTypedObject (id, CLASSINFO(wxHtml));
  if (!html)
  {
    ExtensionError << "Error in html-cancel: could not find html " << id << ".\n";
    return;
  }
  html->Stop();
}

void wxcHtmlOnSize (WXID id, int w, int h)
{
  HTML_CLASS *html = (HTML_CLASS *) wxGetTypedObject (id, CLASSINFO(wxHtml));
  if (!html)
  {
    ExtensionError << "Error in html-on-size: could not find html " << id << ".\n";
    return;
  }
  html->HTML_CLASS::OnSize(w, h);
}

/*
void wxcHtmlOnPaint (WXID id)
{
  HTML_CLASS *html = (HTML_CLASS *) wxGetTypedObject (id, CLASSINFO(wxHtml));
  if (!html)
  {
    ExtensionError << "Error in html-on-size: could not find html " << id << ".\n";
    return;
  }
  html->HTML_CLASS::OnPaint();
}
*/

#endif
