/*
 * wxcitems.cc
 * wxWindows command processing: panel items
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#include "wx_hash.h"
#include "wx_types.h"
#include "wx_ipc.h"

#if !defined(USE_IOSTREAMH) || USE_IOSTREAMH
#include <iostream.h>
#else
#include <iostream>
#endif
#include <ctype.h>
#include <stdlib.h>

#include "wxextend.h"
#include "exitemsp.h"
#include "wx_res.h"

extern wxResourceTable wxExtensionResourceTable;

/************************************************************************
 * GENERIC PANEL ITEM FUNCTIONS
 *
 ************************************************************************/

// Set this button to be the default
Bool wxcPanelItemSetDefault(WXID id)
{
  wxButton *item = (wxButton *)wxGetTypedObject(id, CLASSINFO(wxButton));
  if (item)
  {
    item->SetDefault();
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in panel-item-set-default: could not find item " << id << ".\n";
    return FALSE;
  }
}

// Set label
Bool wxcPanelItemSetLabel(WXID id, char *label)
{
  if (!label)
    return FALSE;

  wxItem *item = (wxItem *)wxGetTypedObject(id, CLASSINFO(wxItem));
  if (item)
  {
    item->SetLabel(label);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in panel-item-set-label: could not find item " << id << ".\n";
    return FALSE;
  }
}

// Get label
char *wxcPanelItemGetLabel(WXID id)
{
  wxItem *item = (wxItem *)wxGetTypedObject(id, CLASSINFO(wxItem));
  if (item)
  {
    char *label = item->GetLabel();
    return label;
  }
  else
  {
    ExtensionError << "Error in panel-item-get-label: could not find item " << id << ".\n";
    return NULL;
  }
}

/************************************************************************
 * PANEL ITEM CALLBACKS
 *
 ************************************************************************/
static wxEvent *wxCurrentEvent = NULL;

void wxExtensionButtonCallback(wxExtensionButton& item, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback(CLASSINFO(wxButton), "OnCommand");
  wxCurrentEvent = &event;
  if (func)
  {
    wxStoreThingInTable((long)&event, &event, CLASSINFO(wxCommandEvent));

    Bool handled = FALSE;
    
    (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item, (long)&event);

    wxDeleteThingInTable((long)&event);
  }
  wxCurrentEvent = NULL;
}

void wxExtensionTextCallback(wxExtensionText& item, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback(CLASSINFO(wxText), "OnCommand");
  wxCurrentEvent = &event;
  if (func)
  {
    wxStoreThingInTable((long)&event, &event, CLASSINFO(wxCommandEvent));

    Bool handled = FALSE;
    
    (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item, (long)&event);
    wxDeleteThingInTable((long)&event);
  }
  wxCurrentEvent = NULL;
}

void wxExtensionMultiTextCallback(wxExtensionMultiText& item, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback(CLASSINFO(wxMultiText), "OnCommand");
  wxCurrentEvent = &event;
  if (func)
  {
    wxStoreThingInTable((long)&event, &event, CLASSINFO(wxCommandEvent));

    Bool handled = FALSE;
    
    (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item, (long)&event);
    wxDeleteThingInTable((long)&event);
  }
  wxCurrentEvent = NULL;
}

void wxExtensionListBoxCallback(wxExtensionListBox& item, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback(CLASSINFO(wxListBox), "OnCommand");
  wxCurrentEvent = &event;
  if (func)
  {
    wxStoreThingInTable((long)&event, &event, CLASSINFO(wxCommandEvent));

    Bool handled = FALSE;
    
    (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item, (long)&event);
    wxDeleteThingInTable((long)&event);
  }
  wxCurrentEvent = NULL;
}

void wxExtensionChoiceCallback(wxExtensionChoice& item, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback(CLASSINFO(wxChoice), "OnCommand");
  wxCurrentEvent = &event;
  if (func)
  {
    wxStoreThingInTable((long)&event, &event, CLASSINFO(wxCommandEvent));

    Bool handled = FALSE;
    
    (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item, (long)&event);
    wxDeleteThingInTable((long)&event);
  }
  wxCurrentEvent = NULL;
}


void wxExtensionRadioBoxCallback(wxExtensionRadioBox& item,
                                 wxCommandEvent& event)
{
    wxExtensionFunction func =
        wxFindExtensionCallback(CLASSINFO(wxRadioBox), "OnCommand");
    wxCurrentEvent = &event;
    if (func)
    {
        wxStoreThingInTable((long)&event, &event, CLASSINFO(wxCommandEvent));

        Bool handled = FALSE;
    
        (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item,
                                                 (long)&event);
        wxDeleteThingInTable((long)&event);
    }
    wxCurrentEvent = NULL;
}

void wxExtensionCheckBoxCallback(wxExtensionCheckBox& item, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback(CLASSINFO(wxCheckBox), "OnCommand");
  wxCurrentEvent = &event;
  if (func)
  {
    wxStoreThingInTable((long)&event, &event, CLASSINFO(wxCommandEvent));

    Bool handled = FALSE;
    
    (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item, (long)&event);
    wxDeleteThingInTable((long)&event);
  }
  wxCurrentEvent = NULL;
}

void wxExtensionSliderCallback(wxExtensionSlider& item, wxCommandEvent& event)
{
  wxExtensionFunction func = wxFindExtensionCallback(CLASSINFO(wxSlider), "OnCommand");
  wxCurrentEvent = &event;
  if (func)
  {
    wxStoreThingInTable((long)&event, &event, CLASSINFO(wxCommandEvent));

    Bool handled = FALSE;
    
    (* (void (*)(Bool *, long, long)) func) (&handled, (long)&item, (long)&event);
    wxDeleteThingInTable((long)&event);
  }
  wxCurrentEvent = NULL;
}

/************************************************************************
 * PANEL ITEM FUNCTIONS
 *
 ************************************************************************/

// Create a button
WXID wxcButtonCreate(WXID parentId, char *label,
  int x, int y, int width, int height, long style, char *name)
{
  if (!label)
    return -1;

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, CLASSINFO(wxPanel));
  if (panel)
  {
    wxExtensionButton *button = new wxExtensionButton(panel, (wxFunction)wxExtensionButtonCallback,
     label, (int)x, (int)y, (int)width, (int)height, style, name);

    return (long)button;
  }
  else
  {
    ExtensionError << "Error in button-create: could not find parent panel " << parentId << ".\n";
    return TRUE;
  }
}

// Create a button (string version)
WXID wxcButtonCreate(WXID parentId, char *label,
  int x, int y, int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcButtonCreate(parentId, label, x, y, width, height, style, name);
}

// Create a button from a bitmap
WXID wxcButtonCreateFromBitmap(WXID parentId, long bitmapId,
  int x, int y, int width, int height, long style, char *name)
{
#if (defined(wx_msw) && !defined(FAFA_LIB))
  ExtensionError << "Error in button-create-from-bitmap:\nBitmap buttons not supported in this version.\n";
  return -1;
#else
  wxBitmap *bitmap = (wxBitmap *)wxGetTypedObject(bitmapId, CLASSINFO(wxBitmap));

  if (!bitmap)
  {
    ExtensionError << "Error in button-create-from-bitmap: could not find bitmap " << bitmapId << ".\n";
    return -1;
  }

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, CLASSINFO(wxPanel));
  if (panel)
  {
    wxExtensionButton *button = new wxExtensionButton(panel, (wxFunction)wxExtensionButtonCallback,
     bitmap, (int)x, (int)y, (int)width, (int)height, style, name);

    return (long)button;
  }
  else
  {
    ExtensionError << "Error in button-create-from-bitmap: could not find parent panel " << parentId << ".\n";
    return -1;
  }
#endif
}

// Create a button from a bitmap (string version)
WXID wxcButtonCreateFromBitmap(WXID parentId, WXID bitmapId,
  int x, int y, int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcButtonCreateFromBitmap(parentId, bitmapId, x, y, width, height,
    style, name);
}


// Change button label (string version)
Bool wxcButtonSetLabel(WXID id, char *label)
{
  wxButton *but = (wxButton *)wxGetTypedObject(id, CLASSINFO(wxButton));
  if (but)
  {
    if (label == NULL) label = "";
    but->SetLabel(label);
    return TRUE;
  }
  ExtensionError << "Error in button-set-label: could not find button " << id << ".\n";
  return FALSE;
}

// Change button label (Bitmap version)
Bool wxcButtonSetLabel(WXID id, WXID bitmapId)
{
  wxButton *but = (wxButton *)wxGetTypedObject(id, CLASSINFO(wxButton));
  if (but)
  {
    wxBitmap *bitmap = (wxBitmap *)wxGetTypedObject(bitmapId, CLASSINFO(wxBitmap));
    if (bitmap) { 
      but->SetLabel(bitmap);
      return TRUE;
    }
    ExtensionError << "Error in button-set-label: could not find bitmap " << bitmapId << ".\n";
  } else
    ExtensionError << "Error in button-set-label: could not find button " << id << ".\n";
  return FALSE;
}


// Create a group box
WXID wxcGroupBoxCreate(WXID parentId, char *label,
  int x, int y, int width, int height, long style, char *name)
{
  if (!label)
    return -1;

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, CLASSINFO(wxPanel));
  if (panel)
  {
    wxExtensionGroupBox *groupBox = new wxExtensionGroupBox(panel, 
     label, (int)x, (int)y, (int)width, (int)height, style, name);

    return (long)groupBox;
  }
  else
  {
    ExtensionError << "Error in group-box-create: could not find parent panel " << parentId << ".\n";
    return TRUE;
  }
}

// Create a group box (string version)
WXID wxcGroupBoxCreate(WXID parentId, char *label,
  int x, int y, int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcGroupBoxCreate(parentId, label, x, y, width, height, style, name);
}

// Create a text widget
WXID wxcTextCreate(WXID parentId, char *label, char *value,
  int x, int y, int width, int height, long style, char *name)
{
  if (!label)
    return -1;
  if (!value)
    return -1;

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, CLASSINFO(wxPanel));
  if (panel)
  {
    wxExtensionText *text = new wxExtensionText(panel, (wxFunction)wxExtensionTextCallback,
                   label, value, x, y, width, height, style, name);

    return (long)text;
  }
  else
  {
    ExtensionError << "Error in text-create: could not find parent panel " << parentId << ".\n";
    return 0;
  }
}

// Create a text widget (string version)
WXID wxcTextCreate(WXID parentId, char *label, char *value,
  int x, int y, int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcTextCreate(parentId, label, value, x, y, width, height, style, name);
}

// Set value
Bool wxcTextSetValue(WXID id, char *val)
{
  if (!val)
    return FALSE;

  wxText *item = (wxText *)wxGetTypedObject(id, CLASSINFO(wxText));
  if (item)
  {
    item->SetValue(val);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in text-set-value: could not find text item " << id << ".\n";
    return FALSE;
  }
}

// Get value
char *wxcTextGetValue(WXID id)
{
  wxText *item = (wxText *)wxGetTypedObject(id, CLASSINFO(wxText));
  if (item)
  {
    char *val = item->GetValue();
    return val;
  }
  else
  {
    ExtensionError << "Error in text-get-value: could not find text item " << id << ".\n";
    return NULL;
  }
}

// Create a multi-line text widget
WXID wxcMultiTextCreate(WXID parentId, char *label, char *value,
  int x, int y, int width, int height, long style, char *name)
{
  if (!label)
    return -1;
  if (!value)
    return -1;
  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, CLASSINFO(wxPanel));
  if (panel)
  {
    wxExtensionMultiText *text = new wxExtensionMultiText(panel, (wxFunction)wxExtensionMultiTextCallback, label, value,
      (int)x, (int)y, (int)width, (int)height, style, name);

    return (long)text;
  }
  else
  {
    ExtensionError << "Error in text-create: could not find parent panel " << parentId << ".\n";
    return 0;
  }
}

// Create a multi-line text widget (string version)
WXID wxcMultiTextCreate(WXID parentId, char *label, char *value,
  int x, int y, int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcMultiTextCreate(parentId, label, value, x, y, width, height, style, name);
}

// Set value
Bool wxcMultiTextSetValue(WXID id, char *val)
{
  if (!val)
    return FALSE;

  wxMultiText *item = (wxMultiText *)wxGetTypedObject(id, CLASSINFO(wxMultiText));
  if (item)
  {
    item->SetValue(val);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in multi-text-set-value: could not find text item " << id << ".\n";
    return FALSE;
  }
}

// Get value
char *wxcMultiTextGetValue(WXID id)
{
  wxMultiText *item = (wxMultiText *)wxGetTypedObject(id, CLASSINFO(wxMultiText));
  if (item)
  {
    char *val = item->GetValue();
    return val;
  }
  else
  {
    ExtensionError << "Error in multi-text-get-value: could not find text item " << id << ".\n";
    return NULL;
  }
}

#ifdef wx_msw
Bool wxcMultiTextWrite (WXID id, char *val)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    win->WriteText(val);
  else
    {
      ExtensionError << "Error in multi-text-write: could not find text window " << id << ".\n";
      return FALSE;
    }
  return TRUE;
}

// Copy selection to clipboard
Bool wxcMultiTextCopy (WXID id)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    {
      win->Copy ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in multi-text-copy: could not find text window " << id << ".\n";
      return FALSE;
    }
}

// Paste selection from clipboard
Bool wxcMultiTextPaste (WXID id)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    {
      win->Paste ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in multi-text-paste: could not find text window " << id << ".\n";
      return FALSE;
    }
}

// Copy selection to clipboard, and delete.
Bool wxcMultiTextCut (WXID id)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    {
      win->Cut ();
      return TRUE;
    }
  else
    {
      ExtensionError << "Error in multi-text-cut: could not find text window " << id << ".\n";
      return FALSE;
    }
}

Bool wxcMultiTextReplace (WXID id, long start, long end, char *text)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    {
      win->Replace (start, end, text);
      return TRUE;
    }
  ExtensionError << "\
Error in multi-text-replace: could not find window " << id << ".\n";
  return FALSE;
}

Bool wxcMultiTextRemove (WXID id, long start, long end)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    {
      win->Remove (start, end);
      return TRUE;
    }
  ExtensionError << "\
Error in multi-text-remove: could not find window " << id << ".\n";
  return FALSE;
}

Bool wxcMultiTextSetSelection (WXID id, long start, long end)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    {
      win->SetSelection (start, end);
      return TRUE;
    }
  ExtensionError << "\
Error in multi-text-set-selection: could not find window " << id << ".\n";
  return FALSE;
}

long wxcMultiTextGetLastPosition (WXID id)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    return win->GetLastPosition ();
  ExtensionError << "\
Error in multi-text-get-last-position: could not find window " << id << ".\n";
  return -1;
}

Bool wxcMultiTextShowPosition (WXID id, long position)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    {
      win->ShowPosition (position);
      return TRUE;
    }
  ExtensionError << "Error in multi-text-show-position: could not find window " << id << ".\n";
  return FALSE;
}


long wxcMultiTextXYToPosition (WXID id, long x, long y)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    return win->XYToPosition (x, y);
  ExtensionError << "\
Error in multi-text-XY-to-position: could not find window " << id << ".\n";
  return -1;
}

Bool wxcMultiTextPositionToXY (WXID id, long position, long *x, long *y)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    {
      win->PositionToXY (position, x, y);
      return TRUE;
    }
  ExtensionError << "\
Error in multi-text-position-to-XY: could not find window " << id << ".\n";
  return FALSE;
}

/*
 * New stuff, 13th November 1995 JACS
 */
 
long wxcMultiTextPositionToLine (WXID id, long position)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    {
      long x, y;
      win->PositionToXY (position, &x, &y);
      return y;
    }
  ExtensionError << "Error in multi-text-position-to-line: could not find window " << id << ".\n";
  return 0;
}

long wxcMultiTextPositionToChar (WXID id, long position)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    {
      long x, y;
      win->PositionToXY (position, &x, &y);
      return x;
    }
  ExtensionError << "Error in multi-text-position-to-char: could not find window " << id << ".\n";
  return 0;
}

Bool wxcMultiTextSetInsertionPoint (WXID id, long position)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    {
      win->SetInsertionPoint (position);
      return TRUE;
    }
  ExtensionError << "Error in multi-text-set-insertion-point: could not find window " << id << ".\n";
  return FALSE;
}

long wxcMultiTextGetInsertionPoint (WXID id)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    {
      return win->GetInsertionPoint ();
      return TRUE;
    }
  ExtensionError << "Error in multi-text-get-insertion-point: could not find window " << id << ".\n";
  return 0;
}

long wxcMultiTextGetLineLength (WXID id, long lineNo)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    {
      return win->GetLineLength (lineNo);
    }
  ExtensionError << "Error in multi-text-get-line-length: could not find window " << id << ".\n";
  return 0;
}

char *wxcMultiTextGetLineText (WXID id, long lineNo)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    {
      static char buf[256];
      win->GetLineText (lineNo, buf);
      return buf;
    }
  ExtensionError << "Error in multi-text-get-line-text: could not find window " << id << ".\n";
  return NULL;
}

long wxcMultiTextGetNumberOfLines (WXID id)
{
  wxMultiText *win = (wxMultiText *) wxGetTypedObject (id, CLASSINFO(wxMultiText));
  if (win)
    {
      return win->GetNumberOfLines ();
    }
  ExtensionError << "Error in multi-text-get-number-of-lines: could not find window " << id << ".\n";
  return 0;
}
#endif

// Create a check box widget
WXID wxcCheckBoxCreate(WXID parentId, char *label,
  int x, int y, int width, int height, long style, char *name)
{
  if (!label)
    return -1;

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, CLASSINFO(wxPanel));
  if (panel)
  {
    wxExtensionCheckBox *box = new wxExtensionCheckBox(panel, (wxFunction)wxExtensionCheckBoxCallback, label,
      (int)(int)x, (int)y, (int)width, (int)height, style, name);

    return (long)box;
  }
  else
  {
    ExtensionError << "Error in check-box-create: could not find parent panel " << parentId << ".\n";
    return 0;
  }
}

// Create a check box widget (string version)
WXID wxcCheckBoxCreate(WXID parentId, char *label,
  int x, int y, int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcCheckBoxCreate(parentId, label, x, y, width, height, style, name);
}

// Set value
Bool wxcCheckBoxSetValue(WXID id, Bool val)
{
  wxCheckBox *item = (wxCheckBox *)wxGetTypedObject(id, CLASSINFO(wxCheckBox));
  if (item)
  {
    item->SetValue(val);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in check-box-set-value: could not find check box " << id << ".\n";
    return FALSE;
  }
}

// Get value
Bool wxcCheckBoxGetValue(WXID id)
{
  wxCheckBox *item = (wxCheckBox *)wxGetTypedObject(id, CLASSINFO(wxCheckBox));
  if (item)
  {
    Bool val = item->GetValue();
    return val;
  }
  else
  {
    ExtensionError << "Error in check-box-get-value: could not find check box " << id << ".\n";
    return FALSE;
  }
}

// Create a slider widget
WXID wxcSliderCreate(WXID parentId, char *label,
  int value, int min_value, int max_value, int width, int x, int y, long style, char *name)
{
  if (!label)
    return -1;

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, CLASSINFO(wxPanel));
  if (panel)
  {
    wxExtensionSlider *slider = new wxExtensionSlider(panel, (wxFunction)wxExtensionSliderCallback, label,
    (int)value, (int)min_value, (int)max_value, (int)width, (int)x, (int)y, style, name);

    return (long)slider;
  }
  else
  {
    ExtensionError << "Error in slider-create: could not find parent panel " << parentId << ".\n";
    return 0;
  }
}

// Create a slider widget (string version)
WXID wxcSliderCreate(WXID parentId, char *label,
  int value, int min_value, int max_value, int width, int x, int y, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcSliderCreate(parentId, label, value, min_value, max_value, width, x, y, style, name);
}

// Set value
Bool wxcSliderSetValue(WXID id, int val)
{
  wxSlider *item = (wxSlider *)wxGetTypedObject(id, CLASSINFO(wxSlider));
  if (item)
  {
    item->SetValue(val);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in slider-set-value: could not find slider " << id << ".\n";
    return FALSE;
  }
}

// Get value
int wxcSliderGetValue(WXID id)
{
  wxSlider *item = (wxSlider *)wxGetTypedObject(id, CLASSINFO(wxSlider));
  if (item)
  {
    int val = item->GetValue();
    return val;
  }
  else
  {
    ExtensionError << "Error in slider-get-value: could not find slider " << id << ".\n";
    return -1;
  }
}

// Create a message widget
WXID wxcMessageCreate(WXID parentId, char *label, int x, int y, long style, char *name)
{
  if (!label)
    return -1;
  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, CLASSINFO(wxPanel));
  if (panel)
  {
    wxExtensionMessage *mess = new wxExtensionMessage(panel, label, (int)x, (int)y, -1, -1, style, name);

    return (long)mess;
  }
  else
  {
    ExtensionError << "Error in message-create: could not find parent panel " << parentId << ".\n";
    return 0;
  }
}

// Create a message widget (string version)
WXID wxcMessageCreate(WXID parentId, char *label, int x, int y, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcMessageCreate(parentId, label, x, y, style, name);
}

// Create a message widget
WXID wxcMessageCreateFromBitmap(WXID parentId, WXID bitmapId, int x, int y, long style, char *name)
{
#if (defined(wx_msw) && (!defined(FAFA_LIB) || !USE_BITMAP_MESSAGE))
  ExtensionError << "Error in message-create-from-bitmap:\nBitmap buttons not supported in this version.\n";
  return -1;
#else
  wxBitmap *bitmap = (wxBitmap *)wxGetTypedObject(bitmapId, CLASSINFO(wxBitmap));

  if (!bitmap)
  {
    ExtensionError << "Error in message-create-from-bitmap: could not find bitmap " << bitmapId << ".\n";
    return -1;
  }

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, CLASSINFO(wxPanel));
  if (panel)
  {
    wxExtensionMessage *mess = new wxExtensionMessage(panel, bitmap, x, y, -1, -1, style, name);

    return (long)mess;
  }
  else
  {
    ExtensionError << "Error in message-create-from-bitmap: could not find parent panel " << parentId << ".\n";
    return 0;
  }
#endif
  return 0;
}

// Create a message widget (string version)
WXID wxcMessageCreateFromBitmap(WXID parentId, WXID bitmapId, int x, int y, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcMessageCreateFromBitmap(parentId, bitmapId, x, y, style, name);
}

// Create a listbox widget
WXID wxcListBoxCreate(WXID parentId, char *label, Bool multiple,
  int x, int y, int width, int height, long style, char *name)
{
  if (!label)
    return -1;

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, CLASSINFO(wxPanel));
  if (panel)
  {
    wxExtensionListBox *item = new wxExtensionListBox(panel, (wxFunction)wxExtensionListBoxCallback, label,
     (Bool)multiple, (int)x, (int)y, (int)width, (int)height, 0, NULL, style, name);

    return (long)item;
  }
  else
  {
    ExtensionError << "Error in list-box-create: could not find parent panel " << parentId << ".\n";
    return 0;
  }
}

// Create a listbox widget (string version)
WXID wxcListBoxCreate(WXID parentId, char *label, Bool multiple,
  int x, int y, int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcListBoxCreate(parentId, label, multiple, x, y, width, height, style, name);
}

// Append a string (and optional client data)
Bool wxcListBoxAppend(WXID id, char *item_string, char *client_data)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, CLASSINFO(wxListBox));
  if (item)
  {
    item->Append(item_string, client_data);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in list-box-append: could not find list box " << id << ".\n";
    return FALSE;
  }
}

// Find a string and return ID
int wxcListBoxFindString(WXID id, char *item_string)
{
  if (!item_string)
    return -1;

  wxListBox *item = (wxListBox *)wxGetTypedObject(id, CLASSINFO(wxListBox));
  if (item)
  {
    return item->FindString(item_string);
  }
  else
  {
    ExtensionError << "Error in list-box-find-string: could not find list box " << id << ".\n";
    return -1;
  }
}

// Clear
Bool wxcListBoxClear(WXID id)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, CLASSINFO(wxListBox));
  if (item)
  {
    item->Clear();
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in list-box-clear: could not find list box " << id << ".\n";
    return FALSE;
  }
}

// Set the current selection
Bool wxcListBoxSetSelection(WXID id, int sel, Bool flag)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, CLASSINFO(wxListBox));
  if (item)
  {
    item->SetSelection(sel, flag);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in list-box-set-selection: could not find list box " << id << ".\n";
    return FALSE;
  }
}

// Deselect an item
Bool wxcListBoxDeselect(WXID id, int sel)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, CLASSINFO(wxListBox));
  if (item)
  {
    item->Deselect(sel);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in list-box-deselect: could not find list box " << id << ".\n";
    return FALSE;
  }
}

// Get the current selection
int wxcListBoxGetSelection(WXID id)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, CLASSINFO(wxListBox));
  if (item)
  {
    return item->GetSelection();
  }
  else
  {
    ExtensionError << "Error in list-box-get-selection: could not find list box " << id << ".\n";
    return -1;
  }
}

// Get string selection
char *wxcListBoxGetStringSelection(WXID id)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, CLASSINFO(wxListBox));
  if (item)
  {
    char *s = item->GetStringSelection();
    return s;
  }
  else
  {
    ExtensionError << "Error in list-box-get-string-selection: could not find list box " << id << ".\n";
    return NULL;
  }
}

// Set string selection
Bool wxcListBoxSetStringSelection(WXID id, char *sel)
{
  if (!sel)
    return FALSE;

  wxListBox *item = (wxListBox *)wxGetTypedObject(id, CLASSINFO(wxListBox));
  if (item)
  {
    return item->SetStringSelection(sel);
  }
  else
  {
    ExtensionError << "Error in list-box-set-string-selection: could not find list box " << id << ".\n";
    return FALSE;
  }
}

// Get number of items
int wxcListBoxNumber(WXID id)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, CLASSINFO(wxListBox));
  if (item)
  {
    return item->Number();
  }
  else
  {
    ExtensionError << "Error in list-box-number: could not find list box " << id << ".\n";
    return -1;
  }
}

// Delete one item
Bool wxcListBoxDelete(WXID id, int n)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, CLASSINFO(wxListBox));
  if (item)
  {
    item->Delete(n);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in list-box-delete: could not find list box " << id << ".\n";
    return FALSE;
  }
}

// Get client data for an item
char *wxcListBoxGetClientData(WXID id, int sel)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, CLASSINFO(wxListBox));
  if (item)
  {
    char *s = item->GetClientData(sel);
    return s;
  }
  else
  {
    ExtensionError << "Error in list-box-get-client-data: could not find list box " << id << ".\n";
    return NULL;
  }
}

// Get string for integer ID
char *wxcListBoxGetString(WXID id, int sel)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, CLASSINFO(wxListBox));
  if (item)
  {
    char *s = item->GetString(sel);
    return s;
  }
  else
  {
    ExtensionError << "Error in list-box-get-string: could not find list box " << id << ".\n";
    return NULL;
  }
}

// Get selections
int *wxListBoxSelections = NULL;
int wxListBoxNSelections = 0;
int wxListBoxCurrentSelection;

int wxcListBoxGetFirstSelection(WXID id)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, CLASSINFO(wxListBox));
  if (item)
  {
    wxListBoxNSelections = item->GetSelections(&wxListBoxSelections);
    wxListBoxCurrentSelection = 1;
    if (wxListBoxNSelections > 0)
      return wxListBoxSelections[0];
    else
      return -1;
  }
  else
  {
    ExtensionError << "Error in list-box-get-first-selection: could not find list box " << id << ".\n";
    return -1;
  }
}

int wxcListBoxGetNextSelection(void)
{
  if (wxListBoxCurrentSelection >= wxListBoxNSelections)
    return -1;

  wxListBoxCurrentSelection ++;

  return wxListBoxSelections[wxListBoxCurrentSelection - 1];
}

long wxcListBoxSetFirstItem(WXID id, long sel)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, CLASSINFO(wxListBox));
  if (item)
  {
    item->SetFirstItem((int)sel);
    return 1;
  }
  else
  {
    ExtensionError << "Error in list-box-set-first-item: could not find list box " << id << ".\n";
    return 0;
  }
}

long wxcListBoxIsSelected(WXID id, long sel)
{
  wxListBox *item = (wxListBox *)wxGetTypedObject(id, CLASSINFO(wxListBox));
  if (item)
  {
    return (long)item->Selected((int)sel);
  }
  else
  {
    ExtensionError << "Error in list-box-is-selected: could not find list box " << id << ".\n";
    return 0;
  }
}

// Generate choice widget creator, used only indirectly by C-only or CLIPS-only
// functions. This is so we can have a generic function for all languages.
// Create a choice widget
WXID wxcChoiceCreate(WXID parentId, char *label, int x, int y,
  int width, int height, int n, char **choices, long style, char *name)
{
  if (!label)
    return -1;
  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, CLASSINFO(wxPanel));
  if (panel)
  {
    wxExtensionChoice *item = new wxExtensionChoice(panel, (wxFunction)wxExtensionChoiceCallback, label,
     (int)x, (int)y, (int)width, (int)height, n, choices, style, name);

    return (long)item;
  }
  else
  {
    ExtensionError << "Error in choice-create: could not find parent panel " << parentId << ".\n";
    return 0;
  }
}

// Create a choice widget (string version)
WXID wxcChoiceCreate(WXID parentId, char *label, int x, int y,
  int width, int height, int n, char **choices, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcChoiceCreate(parentId, label, x, y, width, height, n, choices, style, name);
}

// Append a string
Bool wxcChoiceAppend(WXID id, char *item_string)
{
  if (!item_string)
    return FALSE;

  wxChoice *item = (wxChoice *)wxGetTypedObject(id, CLASSINFO(wxChoice));
  if (item)
  {
    item->Append(item_string);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in choice-append: could not find choice " << id << ".\n";
    return FALSE;
  }
}

// Append a string
int wxcChoiceNumber(WXID id)
{
  wxChoice *item = (wxChoice *)wxGetTypedObject(id, CLASSINFO(wxChoice));
  if (item)
  {
    return item->Number();
  }
  else
  {
    ExtensionError << "Error in choice-number: could not find choice " << id << ".\n";
    return 0;
  }
}

// Find a string and return ID
int wxcChoiceFindString(WXID id, char *item_string)
{
  if (!item_string)
    return -1;

  wxChoice *item = (wxChoice *)wxGetTypedObject(id, CLASSINFO(wxChoice));
  if (item)
  {
    return item->FindString(item_string);
  }
  else
  {
    ExtensionError << "Error in choice-find-string: could not find choice " << id << ".\n";
    return -1;
  }
}

// Clear
Bool wxcChoiceClear(WXID id)
{
  wxChoice *item = (wxChoice *)wxGetTypedObject(id, CLASSINFO(wxChoice));
  if (item)
  {
    item->Clear();
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in choice-clear: could not find choice " << id << ".\n";
    return FALSE;
  }
}

// Set the current selection
Bool wxcChoiceSetSelection(WXID id, int sel)
{
  wxChoice *item = (wxChoice *)wxGetTypedObject(id, CLASSINFO(wxChoice));
  if (item)
  {
    item->SetSelection(sel);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in choice-set-selection: could not find choice " << id << ".\n";
    return FALSE;
  }
}

// Get the current selection
int wxcChoiceGetSelection(WXID id)
{
  wxChoice *item = (wxChoice *)wxGetTypedObject(id, CLASSINFO(wxChoice));
  if (item)
  {
    return item->GetSelection();
  }
  else
  {
    ExtensionError << "Error in choice-get-selection: could not find choice " << id << ".\n";
    return -1;
  }
}

// Get string selection
char *wxcChoiceGetStringSelection(WXID id)
{
  wxChoice *item = (wxChoice *)wxGetTypedObject(id, CLASSINFO(wxChoice));
  if (item)
  {
    char *s = item->GetStringSelection();
    return s;
  }
  else
  {
    ExtensionError << "Error in choice-get-string-selection: could not find choice " << id << ".\n";
    return NULL;
  }
}

// Set string selection
Bool wxcChoiceSetStringSelection(WXID id, char *sel)
{
  if (!sel)
    return FALSE;

  wxChoice *item = (wxChoice *)wxGetTypedObject(id, CLASSINFO(wxChoice));
  if (item)
  {
    return item->SetStringSelection(sel);
  }
  else
  {
    ExtensionError << "Error in choice-set-string-selection: could not find choice " << id << ".\n";
    return FALSE;
  }
}

// Get string for integer ID
char *wxcChoiceGetString(WXID id, int sel)
{
  wxChoice *item = (wxChoice *)wxGetTypedObject(id, CLASSINFO(wxChoice));
  if (item)
  {
    char *s = item->GetString(sel);
    return s;
  }
  else
  {
    ExtensionError << "Error in choice-get-string: could not find choice " << id << ".\n";
    return NULL;
  }
}

#if USE_GAUGE
// Create a gauge
WXID wxcGaugeCreate(WXID parentId, char *label,
  int range, int x, int y, int width, int height, long style, char *name)
{
  if (!label)
    return -1;

  wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, CLASSINFO(wxPanel));
  if (panel)
  {
    wxExtensionGauge *gauge = new wxExtensionGauge(panel, label, (int)range, (int)x, (int)y,
      (int)width, (int)height, style, name);

    return (long)gauge;
  }
  else
  {
    ExtensionError << "Error in gauge-create: could not find parent panel " << parentId << ".\n";
    return 0;
  }
}

// Create a gauge (string version)
WXID wxcGaugeCreate(WXID parentId, char *label,
  int range, int x, int y, int width, int height, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcGaugeCreate(parentId, label, range, x, y, width, height, style, name);
}

// Set value
Bool wxcGaugeSetValue(WXID id, int value)
{
  wxGauge *item = (wxGauge *)wxGetTypedObject(id, CLASSINFO(wxGauge));
  if (item)
  {
    item->SetValue(value);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in gauge-set-value: could not find gauge item " << id << ".\n";
    return FALSE;
  }
}

// Set shadow width
Bool wxcGaugeSetShadowWidth(WXID id, int width)
{
  wxGauge *item = (wxGauge *)wxGetTypedObject(id, CLASSINFO(wxGauge));
  if (item)
  {
    item->SetShadowWidth(width);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in gauge-set-shadow-width: could not find gauge item " << id << ".\n";
    return FALSE;
  }
}

// Set bezel face width
Bool wxcGaugeSetBezelFace(WXID id, int width)
{
  wxGauge *item = (wxGauge *)wxGetTypedObject(id, CLASSINFO(wxGauge));
  if (item)
  {
    item->SetBezelFace(width);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in gauge-set-bezel-face: could not find gauge item " << id << ".\n";
    return FALSE;
  }
}

Bool wxcGaugeSetRange(WXID id, int value)
{
  wxGauge *item = (wxGauge *)wxGetTypedObject(id, CLASSINFO(wxGauge));
  if (item)
  {
    item->SetRange(value);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in gauge-set-range: could not find gauge item " << id << ".\n";
    return FALSE;
  }
}


#endif // USE_GAUGE

// Create a menu
WXID wxcMenuCreate(char *title, wxFunction func)
{
  wxExtensionMenu *menu = new wxExtensionMenu(title, func);

  return (long)menu;
}

// Append a string or submenu
Bool wxcMenuAppend(WXID id, int item_id, char *menu_string, WXID submenu_id, char *help_string, Bool checkable)
{
  if (!menu_string)
    return FALSE;

  wxMenu *menu = (wxMenu *)wxGetTypedObject(id, CLASSINFO(wxMenu));
  if (!menu)
  {
    ExtensionError << "Error in menu-append: could not find menu " << id << ".\n";
    return FALSE;
  }

  if (submenu_id > -1)
  {
    wxMenu *submenu = (wxMenu *)wxGetTypedObject(submenu_id, CLASSINFO(wxMenu));
    if (submenu)
    {
      menu->Append((int)item_id, menu_string, submenu, help_string);
      return TRUE;
    }
    else
    {
      ExtensionError << "Error in menu-append: could not find submenu " << submenu_id << "\n";
      return FALSE;
    }
  }
  else
  {
    menu->Append((int)item_id, menu_string, help_string, checkable);
    return TRUE;
  }
}


// Append a Checkable string 
Bool wxcMenuAppend(WXID id, int item_id, char *menu_string, char *help_string, Bool checkable)
{
  if (!menu_string)
    return FALSE;

  wxMenu *menu = (wxMenu *)wxGetTypedObject(id, CLASSINFO(wxMenu));
  if (!menu)
  {
    ExtensionError << "Error in menu-append: could not find menu " << id << ".\n";
    return FALSE;
  }

  menu->Append((int)item_id, menu_string, help_string, checkable);
  return TRUE;
}

// Append a separator
Bool wxcMenuAppendSeparator(WXID id)
{
  wxMenu *menu = (wxMenu *)wxGetTypedObject(id, CLASSINFO(wxMenu));
  if (menu)
  {
    menu->AppendSeparator();
    return TRUE;
  }
  ExtensionError << "Error in menu-append-separator: could not find menu " << id << ".\n";
  return FALSE;
}

// Menu Break
Bool wxcMenuBreak(WXID id)
{
  wxMenu *menu = (wxMenu *)wxGetTypedObject(id, CLASSINFO(wxMenu));
  if (menu)
  {
    menu->Break();
    return TRUE;
  }
  ExtensionError << "Error in menu-break: could not find menu " << id << ".\n";
  return FALSE;
}

// Enable an item
Bool wxcMenuEnable(WXID id, int item_id, Bool enabled)
{
  wxMenu *menu = (wxMenu *)wxGetTypedObject(id, CLASSINFO(wxMenu));
  if (menu)
  {
    menu->Enable(item_id, enabled);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in menu-enable: could not find menu " << id << ".\n";
    return FALSE;
  }
}

// Check an item
Bool wxcMenuCheck(WXID id, int item_id, Bool checked)
{
  wxMenu *menu = (wxMenu *)wxGetTypedObject(id, CLASSINFO(wxMenu));
  if (menu)
  {
    menu->Check(item_id, checked);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in menu-check: could not find menu " << id << ".\n";
    return FALSE;
  }
}

long wxcMenuSetLabel(WXID id, long item_id, char *label)
{
  wxMenu *menu = (wxMenu *)wxGetTypedObject(id, CLASSINFO(wxMenu));
  if (menu)
  {
    menu->SetLabel((int)item_id, label);
    return 1;
  }
  else
  {
    ExtensionError << "Error in menu-set-label: could not find menu "<< id << ".\n";
    return 0;
  }
}

// Create a menu bar
WXID wxcMenuBarCreate(void)
{
  wxExtensionMenuBar *menu_bar = new wxExtensionMenuBar;

  return (long)menu_bar;
}

WXID wxcMenuBarCreateFromResource(char *resourceName)
{
  wxExtensionMenuBar *menuBar = new wxExtensionMenuBar;

  long retValue = (long)wxResourceCreateMenuBar(resourceName, &wxExtensionResourceTable, menuBar);
  if (retValue == 0)
  {
    delete menuBar;
    return 0;
  }
  else
    return retValue;
}

// Append a menu
Bool wxcMenuBarAppend(WXID id, WXID menu_id, char *menu_string)
{
  if (!menu_string)
    return FALSE;
  wxMenuBar *menu_bar = (wxMenuBar *)wxGetTypedObject(id, CLASSINFO(wxMenuBar));
  if (menu_bar)
  {
    wxMenu *menu = (wxMenu *)wxGetTypedObject(menu_id, CLASSINFO(wxMenu));
    if (menu)
    {
      menu_bar->Append(menu, menu_string);
      return TRUE;
    }
    else
    {
      ExtensionError << "Error in menu-bar-append: could not find menu " << menu_id << ".\n";
      return FALSE;
    }
  }
  else
  {
    ExtensionError << "Error in menu-bar-append: could not find menu bar " << id << ".\n";
    return FALSE;
  }
}

// Enable an item
Bool wxcMenuBarEnable(WXID id, int item_id, Bool enabled)
{
  wxMenuBar *menu_bar = (wxMenuBar *)wxGetTypedObject(id, CLASSINFO(wxMenuBar));
  if (menu_bar)
  {
    menu_bar->Enable(item_id, enabled);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in menu-bar-enable: could not find menu bar " << id << ".\n";
    return FALSE;
  }
}

// Check an item
Bool wxcMenuBarCheck(WXID id, int item_id, Bool checked)
{
  wxMenuBar *menu_bar = (wxMenuBar *)wxGetTypedObject(id, CLASSINFO(wxMenuBar));
  if (menu_bar)
  {
    menu_bar->Check(item_id, checked);
    return TRUE;
  }
  else
  {
    ExtensionError << "Error in menu-bar-check: could not find menu bar " << id << ".\n";
    return FALSE;
  }
}

// Get Check status of an item
Bool wxcMenuBarChecked(WXID id, int item_id)
{
  wxMenuBar *menu_bar = (wxMenuBar *)wxGetTypedObject(id, CLASSINFO(wxMenuBar));
  if (menu_bar)
  {
    return menu_bar->Checked(item_id);
  }
  else
  {
    ExtensionError << "Error in menu-bar-checked: could not find menu bar " << id << ".\n";
    return FALSE;
  }
}


WXID wxcRadioBoxCreate(WXID parentId, char *label, int x, int y,
                       int width, int height, int n, char **choices,
                       int majorDim, long style, char *name)
{
    if (!label)
        return -1;
    wxPanel *panel = (wxPanel *)wxGetTypedObject(parentId, CLASSINFO(wxPanel));
    if (panel)
    {
        wxExtensionRadioBox *item =
            new wxExtensionRadioBox(panel,
                                    (wxFunction)wxExtensionRadioBoxCallback,
                                    label, (int)x, (int)y,
                                    (int)width, (int)height,
                                    n, choices, majorDim, style, name);

        return (long)item;
    }
    else
    {
        ExtensionError << "Error in radio-box-create: could not find parent panel " << parentId << ".\n";
        return 0;
    }
}

// Create a radiobox widget (string style version)
WXID wxcRadioBoxCreate(WXID parentId, char *label, int x, int y,
  int width, int height, int n, char **choices, int majorDim, char *styleString, char *name)
{
  long style = wxExtensionParseBitList(styleString);
  return wxcRadioBoxCreate(parentId, label, x, y, width, height, n, choices, majorDim, style, name);
}


long wxcRadioBoxSetSelection(WXID id, long sel)
{
  wxRadioBox *item = (wxRadioBox *)wxGetTypedObject(id, CLASSINFO(wxRadioBox));
  if (item)
  {
    item->SetSelection((int)sel);
    return 1;
  }
  else
  {
    ExtensionError << "Error in radio-box-set-selection: could not find RadioBox " << id << ".\n";
    return 0;
  }
}


long wxcRadioBoxGetSelection(WXID id)
{
  wxRadioBox *item = (wxRadioBox *)wxGetTypedObject(id, CLASSINFO(wxRadioBox));
  if (item)
  {
    return item->GetSelection();
  }
  else
  {
    ExtensionError << "Error in radio-box-get-selection: could not find radiobox " << id << ".\n";
    return -1;
  }
}
