/*
 * wxcmisc.cc
 * wxWindows miscellaneous command processing
 *
 */

// For compilers that support precompilation, includes "wx.h".
#include "wx_prec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx.h"
#endif

#include "wx_hash.h"
#include "wx_types.h"
#include "wx_ipc.h"
#include "wx_help.h"
#include "wx_timer.h"

#if !defined(USE_IOSTREAMH) || USE_IOSTREAMH
#include <iostream.h>
#else
#include <iostream>
#endif
#include <ctype.h>
#include <stdlib.h>

#ifdef wx_msw
#include <direct.h>
#endif

#if defined(wx_msw) && !defined(__win32__) && !defined(WIN32) && !defined(__win32s__)
#include <dos.h>
#endif

#ifdef wx_msw
#include <mmsystem.h>
#include <shellapi.h>
#endif

#include "wxextend.h"

// Record what frames and dialogs are used so we can clear
// them up if necessary
extern wxList wxExtensionFrameList;

/*
 * Mechanism for getting CLIPS callbacks to return a value
 *
 */

/*
 * Get platform:
 *  "Windows 3.1", "XView" or "Motif"
 *
 */

char *wxcGetPlatform(void)
{
#ifdef wx_motif
  return "Motif";
#endif
#ifdef wx_xview
  return "XView";
#endif
#ifdef wx_msw
  return "Windows 3.1";
#endif
}

/*
 * Get OS version (more precise than wxcGetPlatform)
 */

char *wxcGetOsVersion(void)
{
  int ver = wxGetOsVersion();
  switch (ver)
  {
    case wxCURSES:
      return "wxCURSES";
    case wxXVIEW_X:
      return "wxXVIEW_X";
    case wxMOTIF_X:
      return "wxMOTIF_X";
    case wxCOSE_X:
      return "wxCOSE_X";
    case wxNEXTSTEP:
      return "wxNEXTSTEP";
    case wxMACINTOSH:
      return "wxMACINTOSH";
    case wxGEOS:
      return "wxGEOS";
    case wxOS2_PM:
      return "wxOS2_PM";
    case wxWINDOWS:
      return "wxWINDOWS";
    case wxPENWINDOWS:
      return "wxPENWINDOWS";
    case wxWINDOWS_NT:
      return "wxWINDOWS_NT";
    case wxWIN32S:
      return "wxWIN32S";
#if wxRELEASE_NUMBER > 3
    case wxWIN95:
      return "wxWIN95";
#endif
    case wxWIN386:
      return "wxWIN386";
    default:
      return "Unknown";
  }
}

char *wxcGetObjectType(WXID id)
{
/*
  WXTYPE theType = 0;
  wxObject *obj = wxGetThingFromTable(id, &theType);
  if (!obj)
    return NULL;
  return wxGetTypeName(theType);
*/
  wxObject *obj = wxGetThingFromTable(id);
  if (obj)
    return obj->GetClassInfo()->GetClassName();
  else
    return NULL;
}

/************************************************************************
 * CLEARING-UP FUNCTIONS
 *
 ************************************************************************/

void wxcCleanWindows(void)
{
  wxNode *node = wxExtensionFrameList.First();
  while (node)
  {
    wxWindow *win = (wxWindow *)node->Data();
    win->Show(FALSE);
    delete win;
    node = wxExtensionFrameList.First();
  }
}

/*
 * Timer functionality
 */
 
// Timer class
class wxExtensionTimerClass: public wxTimer
{
 public:
  wxExtensionTimerClass(void)
  {
    wxStoreThingInTable((long)this, this, CLASSINFO(wxTimer));
  }
  ~wxExtensionTimerClass(void)
  {
    wxExtensionFunction func = wxFindExtensionCallback(CLASSINFO(wxTimer), "OnDelete");
    if (func)
      (* (void (*)(long)) func) ((long)this);
    wxDeleteThingInTable((long)this);
  }
  void Notify(void);
};


void wxExtensionTimerClass::Notify(void)
{
  wxExtensionFunction func = wxFindExtensionCallback(CLASSINFO(wxTimer), "Notify");
  if (func)
  {
    Bool handled = FALSE;
    (* (void (*)(Bool *, long)) func) (&handled, (long)this);
  }
}

// Create timer
WXID wxcTimerCreate(void)
{
  return (WXID) new wxExtensionTimerClass();
}

// Delete timer
Bool wxcTimerDelete(WXID id)
{
  wxTimer *timer = (wxTimer *)wxGetTypedObject(id, CLASSINFO(wxTimer));
  if (!timer)
  {
    ExtensionError << "Error in timer-delete: could not find timer " << id << ".\n";
    return FALSE;
  }
  delete timer;
  return TRUE;
}

// Start timer
Bool wxcTimerStart(WXID id, long ms, Bool one_shot)
{
  wxTimer *timer = (wxTimer *)wxGetTypedObject(id, CLASSINFO(wxTimer));
  if (!timer)
  {
    ExtensionError << "Error in timer-start: could not find timer " << id << ".\n";
    return FALSE;
  }
  return timer->Start((int)ms, one_shot);
}

// Stop timer
Bool wxcTimerStop(WXID id)
{
  wxTimer *timer = (wxTimer *)wxGetTypedObject(id, CLASSINFO(wxTimer));
  if (!timer)
  {
    ExtensionError << "Error in timer-stop: could not find timer " << id << ".\n";
    return FALSE;
  }
  timer->Stop();
  return TRUE;
}

/*
 * Stopwatch functions
 */
 
long wxcGetElapsedTime(Bool resetTimer)
{
  return wxGetElapsedTime(resetTimer);
}

void wxcStartTimer(void)
{
  wxStartTimer();
}

char *wxcNow(void)
{
  return wxNow();
}

/*
 * Event accessors
 */

long wxcEventGetEventType(WXID id)
{
    wxEvent *event = (wxEvent*)wxGetTypedObject(id, CLASSINFO(wxEvent));
    if (event) {
        return event->GetEventType();
    }
    else {
        ExtensionError
            << "Error in event-get-event-type: could not find event "
            << id << ".\n";
        return FALSE;
    }
}

long wxcEventGetEventClass(WXID id)
{
    wxEvent *event = (wxEvent*)wxGetTypedObject(id, CLASSINFO(wxEvent));
    if (event) {
        return event->GetEventClass();
    }
    else {
        ExtensionError
            << "Error in event-get-event-class: could not find event "
            << id << ".\n";
        return FALSE;
    }
}


// Key event
Bool wxcKeyEventControlDown(WXID id)
{
    wxKeyEvent *event = (wxKeyEvent*)wxGetTypedObject(id, CLASSINFO(wxKeyEvent));
    if (event) {
        return event->ControlDown();
    }
    else {
        ExtensionError
            << "Error in key-event-control-down: could not find event "
            << id << ".\n";
        return FALSE;
    }
}

Bool wxcKeyEventShiftDown(WXID id)
{
    wxKeyEvent *event = (wxKeyEvent*)wxGetTypedObject(id, CLASSINFO(wxKeyEvent));
    if (event) {
        return event->ShiftDown();
    }
    else {
        ExtensionError
            << "Error in key-event-shift-down: could not find event "
            << id << ".\n";
        return FALSE;
    }
}

Bool wxcKeyEventAltDown(WXID id)
{
    wxKeyEvent *event = (wxKeyEvent*)wxGetTypedObject(id, CLASSINFO(wxKeyEvent));
    if (event) {
//        return event->AltDown();
        return event->altDown;
    }
    else {
        ExtensionError
            << "Error in key-event-alt-down: could not find event "
            << id << ".\n";
        return FALSE;
    }
}

long wxcKeyEventKeyCode(WXID id)
{
    wxKeyEvent *event = (wxKeyEvent*)wxGetTypedObject(id, CLASSINFO(wxKeyEvent));
    if (event) {
        return event->KeyCode();
    }
    else {
        ExtensionError
            << "Error in key-event-key-code: could not find event "
            << id << ".\n";
        return FALSE;
    }
}

char *wxcKeyEventKeyCodeString(WXID id)
{
    wxKeyEvent *event = (wxKeyEvent*)wxGetTypedObject(id, CLASSINFO(wxKeyEvent));
    if (event) {
        return wxExtensionKeyCodeIntToString(event->KeyCode());
    }
    else {
        ExtensionError
            << "Error in key-event-key-code: could not find event "
            << id << ".\n";
        return NULL;
    }
}

Bool wxcKeyEventPosition(WXID id, double *x, double *y)
{
    wxKeyEvent *event = (wxKeyEvent*)wxGetTypedObject(id, CLASSINFO(wxKeyEvent));
    if (event) {
        float xf, yf;
        event->Position(&xf, &yf);
        *x = (double)xf;
        *y = (double)yf;
        return TRUE;
    }
    else {
        ExtensionError
            << "Error in key-event-position: could not find event "
            << id << ".\n";
        return FALSE;
    }
}


// Command event
char* wxcCommandEventGetClientData(WXID id)
{
    wxCommandEvent *event = (wxCommandEvent*)wxGetTypedObject(id, CLASSINFO(wxCommandEvent));
    if (event) {
        return event->GetClientData();
    }
    else {
        ExtensionError
            << "Error in command-event-get-client-data: could not find event "
            << id << ".\n";
        return NULL;
    }
}


long wxcCommandEventGetSelection(WXID id)
{
    wxCommandEvent *event = (wxCommandEvent*)wxGetTypedObject(id, CLASSINFO(wxCommandEvent));
    if (event) {
        return event->GetSelection();
    }
    else {
        ExtensionError
            << "Error in commandevent-get-selection: could not find event "
            << id << ".\n";
        return FALSE;
    }
}

char* wxcCommandEventGetString(WXID id)
{
    wxCommandEvent *event = (wxCommandEvent*)wxGetTypedObject(id, CLASSINFO(wxCommandEvent));
    if (event) {
        return event->GetString();
    }
    else {
        ExtensionError
            << "Error in command-event-get-string: could not find event "
            << id << ".\n";
        return NULL;
    }
}

Bool wxcCommandEventChecked(WXID id)
{
    wxCommandEvent *event = (wxCommandEvent*)wxGetTypedObject(id, CLASSINFO(wxCommandEvent));
    if (event) {
        return event->Checked();
    }
    else {
        ExtensionError
            << "Error in command-event-checked: could not find event "
            << id << ".\n";
        return FALSE;
    }
}

Bool wxcCommandEventIsSelection(WXID id)
{
    wxCommandEvent *event = (wxCommandEvent*)wxGetTypedObject(id, CLASSINFO(wxCommandEvent));
    if (event) {
        return event->IsSelection();
    }
    else {
        ExtensionError
            << "Error in command-event-is-selection: could not find event "
            << id << ".\n";
        return FALSE;
    }
}
  
// Mouse event
Bool wxcMouseEventIsButton(WXID id)
{
  wxMouseEvent *event = (wxMouseEvent *)wxGetTypedObject(id, CLASSINFO(wxMouseEvent));
  if (event)
  {
    return event->IsButton();
  }
  else
  {
    ExtensionError << "Error in mouse-event-is-button: could not find event " << id << ".\n";
    return FALSE;
  }
}

Bool wxcMouseEventButtonDown(WXID id)
{
  wxMouseEvent *event = (wxMouseEvent *)wxGetTypedObject(id, CLASSINFO(wxMouseEvent));
  if (event)
  {
    return event->ButtonDown();
  }
  else
  {
    ExtensionError << "Error in mouse-event-button-down: could not find event " << id << ".\n";
    return FALSE;
  }
}

Bool wxcMouseEventControlDown(WXID id)
{
  wxMouseEvent *event = (wxMouseEvent *)wxGetTypedObject(id, CLASSINFO(wxMouseEvent));
  if (event)
  {
    return event->ControlDown();
  }
  else
  {
    ExtensionError << "Error in mouse-event-control-down: could not find event " << id << ".\n";
    return FALSE;
  }
}

Bool wxcMouseEventShiftDown(WXID id)
{
  wxMouseEvent *event = (wxMouseEvent *)wxGetTypedObject(id, CLASSINFO(wxMouseEvent));
  if (event)
  {
    return event->ShiftDown();
  }
  else
  {
    ExtensionError << "Error in mouse-event-shift-down: could not find event " << id << ".\n";
    return FALSE;
  }
}

Bool wxcMouseEventButton(WXID id, int butt)
{
  wxMouseEvent *event = (wxMouseEvent *)wxGetTypedObject(id, CLASSINFO(wxMouseEvent));
  if (event)
  {
    return event->Button(butt);
  }
  else
  {
    ExtensionError << "Error in mouse-event-button: could not find event " << id << ".\n";
    return FALSE;
  }
}

Bool wxcMouseEventLeftDown(WXID id)
{
  wxMouseEvent *event = (wxMouseEvent *)wxGetTypedObject(id, CLASSINFO(wxMouseEvent));
  if (event)
  {
    return event->LeftDown();
  }
  else
  {
    ExtensionError << "Error in mouse-event-left-down: could not find event " << id << ".\n";
    return FALSE;
  }
}

Bool wxcMouseEventMiddleDown(WXID id)
{
  wxMouseEvent *event = (wxMouseEvent *)wxGetTypedObject(id, CLASSINFO(wxMouseEvent));
  if (event)
  {
    return event->MiddleDown();
  }
  else
  {
    ExtensionError << "Error in mouse-event-middle-down: could not find event " << id << ".\n";
    return FALSE;
  }
}

Bool wxcMouseEventRightDown(WXID id)
{
  wxMouseEvent *event = (wxMouseEvent *)wxGetTypedObject(id, CLASSINFO(wxMouseEvent));
  if (event)
  {
    return event->RightDown();
  }
  else
  {
    ExtensionError << "Error in mouse-event-right-down: could not find event " << id << ".\n";
    return FALSE;
  }
}

Bool wxcMouseEventLeftUp(WXID id)
{
  wxMouseEvent *event = (wxMouseEvent *)wxGetTypedObject(id, CLASSINFO(wxMouseEvent));
  if (event)
  {
    return event->LeftUp();
  }
  else
  {
    ExtensionError << "Error in mouse-event-left-up: could not find event " << id << ".\n";
    return FALSE;
  }
}

Bool wxcMouseEventMiddleUp(WXID id)
{
  wxMouseEvent *event = (wxMouseEvent *)wxGetTypedObject(id, CLASSINFO(wxMouseEvent));
  if (event)
  {
    return event->MiddleUp();
  }
  else
  {
    ExtensionError << "Error in mouse-event-middle-up: could not find event " << id << ".\n";
    return FALSE;
  }
}

Bool wxcMouseEventRightUp(WXID id)
{
  wxMouseEvent *event = (wxMouseEvent *)wxGetTypedObject(id, CLASSINFO(wxMouseEvent));
  if (event)
  {
    return event->RightUp();
  }
  else
  {
    ExtensionError << "Error in mouse-event-right-up: could not find event " << id << ".\n";
    return FALSE;
  }
}

Bool wxcMouseEventDragging(WXID id)
{
  wxMouseEvent *event = (wxMouseEvent *)wxGetTypedObject(id, CLASSINFO(wxMouseEvent));
  if (event)
  {
    return event->Dragging();
  }
  else
  {
    ExtensionError << "Error in mouse-event-dragging: could not find event " << id << ".\n";
    return FALSE;
  }
}

long wxcMouseEventPosition(long id, double *x, double *y)
{
  wxMouseEvent* event = (wxMouseEvent*)wxGetTypedObject(id, CLASSINFO(wxMouseEvent));
  if (event) {
    float xf, yf;
    event->Position(&xf, &yf);
    *x = (double)xf;
    *y = (double)yf;
    return 1;
  }
  else
  {
    ExtensionError << "Error in mouse-event-position: could not find event " << id << ".\n";
    return 0;
  }
}

double wxcMouseEventPositionX(WXID id)
{
  wxMouseEvent *event = (wxMouseEvent *)wxGetTypedObject(id, CLASSINFO(wxMouseEvent));
  if (event)
  {
    float x, y;
    event->Position(&x, &y);
    return (double)x;
  }
  else
  {
    ExtensionError << "Error in mouse-event-position-x: could not find event " << id << ".\n";
    return 0.0;
  }
}

double wxcMouseEventPositionY(WXID id)
{
  wxMouseEvent *event = (wxMouseEvent *)wxGetTypedObject(id, CLASSINFO(wxMouseEvent));
  if (event)
  {
    float x, y;
    event->Position(&x, &y);
    return (double)y;
  }
  else
  {
    ExtensionError << "Error in mouse-event-position-y: could not find event " << id << ".\n";
    return 0.0;
  }
}

/************************************************************************
 * FUNCTIONS TO ACCESS HELP SYSTEM
 *
 ************************************************************************/

WXID wxcHelpCreate(Bool native)
{
  wxHelpInstance *theInstance = new wxHelpInstance(native);
  wxStoreThingInTable((long)theInstance, theInstance, CLASSINFO(wxHelpInstance));
  return (WXID)theInstance;
}

Bool wxcHelpDelete(WXID id)
{
  wxHelpInstance *theInstance = (wxHelpInstance *)wxGetTypedObject(id, CLASSINFO(wxHelpInstance));
  if (!theInstance)
  {
    ExtensionError << "Error in help-delete: could not find help instance " << id << ".\n";
    return FALSE;
  }
  wxDeleteThingInTable(id);
  delete theInstance;
  return TRUE;
}

Bool wxcHelpLoadFile(WXID id, char *file)
{
  wxHelpInstance *theInstance = (wxHelpInstance *)wxGetTypedObject(id, CLASSINFO(wxHelpInstance));
  if (!theInstance)
  {
    ExtensionError << "Error in help-load-file: could not find help instance " << id << ".\n";
    return FALSE;
  }
  return theInstance->LoadFile(file);
}

Bool wxcHelpDisplayContents(WXID id)
{
  wxHelpInstance *theInstance = (wxHelpInstance *)wxGetTypedObject(id, CLASSINFO(wxHelpInstance));
  if (!theInstance)
  {
    ExtensionError << "Error in help-display-contents: could not find help instance " << id << ".\n";
    return FALSE;
  }
  return theInstance->DisplayContents();
}

Bool wxcHelpDisplaySection(WXID id, long sectionId)
{
  wxHelpInstance *theInstance = (wxHelpInstance *)wxGetTypedObject(id, CLASSINFO(wxHelpInstance));
  if (!theInstance)
  {
    ExtensionError << "Error in help-display-section: could not find help instance " << id << ".\n";
    return FALSE;
  }
  return theInstance->DisplaySection((int)sectionId);
}

Bool wxcHelpDisplayBlock(WXID id, long blockId)
{
  wxHelpInstance *theInstance = (wxHelpInstance *)wxGetTypedObject(id, CLASSINFO(wxHelpInstance));
  if (!theInstance)
  {
    ExtensionError << "Error in help-display-block: could not find help instance " << id << ".\n";
    return FALSE;
  }
  return theInstance->DisplayBlock(blockId);
}

Bool wxcHelpKeywordSearch(WXID id, char *keyword)
{
  wxHelpInstance *theInstance = (wxHelpInstance *)wxGetTypedObject(id, CLASSINFO(wxHelpInstance));
  if (!theInstance)
  {
    ExtensionError << "Error in help-keyword-search: could not find help instance " << id << ".\n";
    return FALSE;
  }
  if (!keyword)
    return FALSE;
  return theInstance->KeywordSearch(keyword);
}

Bool wxcHelpQuit(WXID id)
{
  wxHelpInstance *theInstance = (wxHelpInstance *)wxGetTypedObject(id, CLASSINFO(wxHelpInstance));
  if (!theInstance)
  {
    ExtensionError << "Error in help-quit: could not find help instance " << id << ".\n";
    return FALSE;
  }
  return theInstance->Quit();
}


/************************************************************************
 * INPUT/OUTPUT AND USER INTERFACE FUNCTIONS
 *
 ************************************************************************/

/*
 * Pass optional message and optional default string
 *
 */

char *wxcGetTextFromUser(char *message, char *caption, char *default_string,
  Bool centre, WXID parentId)
{
  wxWindow *parent = NULL;
  if (parentId > 0)
  {
    parent = (wxWindow *)wxGetTypedObject(parentId, CLASSINFO(wxWindow));
    if (!parent)
    {
      ExtensionError << "Error in get-text-from-user: could not find parent window " << parentId << ".\n";
      return NULL;
    }
  }

  char *s = wxGetTextFromUser(message, caption, default_string, parent, -1, -1, centre);
  return s;
}

/*
 * Convert string to number
 *
 */

double wxcStringToFloat(char *str)
{
  if (!str)
    return 0.0;
  return atof(str);
}

long wxcStringToLong(char *str)
{
  if (!str)
    return 0;
  return atol(str);
}

/*
 * Convert number to string
 *
 */

char *wxcFloatToString(double num)
{
  static char buf[50];
  sprintf(buf, "%.4f", num);
  return buf;
}

char *wxcLongToString(long num)
{
  static char buf[50];
  sprintf(buf, "%ld", num);
  return buf;
}

/*
 * Convert string to symbol
 *
 */

char *wxcStringToSymbol(char *s)
{
  return s;
}

/*
 * Convert symbol to string
 *
 */

char *wxcSymbolToString(char *s)
{
  return s;
}

/*
 * Change directory
 */
 
Bool wxcChdir(char *s)
{
  return wxSetWorkingDirectory(s);
}

/*
 * Make directory
 */
 
Bool wxcMkdir(char *s)
{
  return wxMkdir(s);
}

/*
 * Remove directory
 */
 
Bool wxcRmdir(char *s)
{
  return wxRmdir(s);
}

/*
 * File exists
 */
 
Bool wxcFileExists(char *s)
{
  return wxFileExists(s);
}

/*
 * Copy file
 */
 
Bool wxcCopyFile(char *f1, char *f2)
{
  return wxCopyFile((char *)f1, (char *)f2);
}

/*
 * Dir exists
 */
 
Bool wxcDirExists(char *s)
{
  return wxDirExists(s);
}

/*
 * Pass message, optional flags = "OK", "OK-CANCEL", "YES-NO" or "YES-NO-CANCEL"
 *
 */

char *wxcMessageBox(char *message, char *flags, Bool centre, WXID parentId, char *title)
{
  int wx_flags = wxOK;
  if (strcmp(flags, "OK") == 0)
    wx_flags = wxOK;
  else if (strcmp(flags, "OK-CANCEL") == 0)
    wx_flags = wxOK | wxCANCEL;
  else if (strcmp(flags, "YES-NO") == 0)
    wx_flags = wxYES_NO;
  else if (strcmp(flags, "YES-NO-CANCEL") == 0)
    wx_flags = wxYES_NO | wxCANCEL;
  else
    wx_flags = (int)wxExtensionParseBitList(flags);

  if (centre)
    wx_flags |= wxCENTRE;

  wxWindow *parent = NULL;
  if (parentId > 0)
  {
    parent = (wxWindow *)wxGetTypedObject(parentId, CLASSINFO(wxWindow));
    if (!parent)
    {
      ExtensionError << "Error in message-box: could not find parent window " << parentId << ".\n";
      return NULL;
    }
  }
  if (!title || (strlen(title) == 0))
    title = "Message";

  int res = wxMessageBox(message, title, wx_flags, parent);
  char *ans = "ERROR";
  switch (res)
  {  
    case wxCANCEL: ans = "CANCEL"; break;
    case wxOK:     ans = "OK"; break;
    case wxYES:    ans = "YES"; break;
    case wxNO:     ans = "NO"; break;
  }

  return ans;
}

/*
 * Pass optional message, default path, default file, default extension, default
 * wildcard.
 *
 */

char *wxcFileSelector(char *message,
  char *def_path, char *def_file, char *def_ext, char *def_wildcard, long flags, WXID parentId)
{
  wxWindow *parent = NULL;
  if (parentId > 0)
  {
    parent = (wxWindow *)wxGetTypedObject(parentId, CLASSINFO(wxWindow));
    if (!parent)
    {
      ExtensionError << "Error in file-selector: could not find parent window " << parentId << ".\n";
      return NULL;
    }
  }

  char *s = wxFileSelector(message, def_path, def_file, def_ext, def_wildcard, (int)flags, parent);
  return s;
}

char *wxcFileSelector(char *message,
  char *def_path, char *def_file, char *def_ext, char *def_wildcard, char *stringFlags, WXID parentId)
{
  long flags = wxExtensionParseBitList(stringFlags);
  return wxcFileSelector(message, def_path, def_file, def_ext, def_wildcard, flags, parentId);
}

/*
 * Yield to windowing system (if appropriate -- i.e. Windows only)
 *
 */

Bool wxcYield(void)
{
  wxYield();
  return TRUE;
}

/*
 * Sleep for nSecs seconds
 *
 */

Bool wxcSleep(long nSecs)
{
  wxSleep((int)nSecs);
  return TRUE;
}

/*
 * Write a resource (Windows WIN.INI or Xdefaults)
 *
 */

Bool wxcWriteResource(char *section, char *entry, char *value, char *filename)
{
  return (Bool)wxWriteResource(section, entry, value, filename);
}

/*
 * Get a resource (Windows WIN.INI or Xdefaults)
 *
 */

char *wxcGetResource(char *section, char *entry, char *filename)
{
  char *value = NULL;
  if (wxGetResource(section, entry, &value, filename))
    return value;
  else
    return NULL;
}

/*
 * Write a debug message
 *
 */

Bool wxcDebugMsg(char *msg)
{
  wxDebugMsg(msg);
  return TRUE;
}

/*
 * Begin a 'busy' section
 *
 */

void wxcBeginBusyCursor(void)
{
  wxBeginBusyCursor();
}

/*
 * End a 'busy' section
 *
 */

void wxcEndBusyCursor(void)
{
  wxEndBusyCursor();
}

// This should be ok for many objects, so in theory we can
// remove several wxcDelete... functions.
Bool wxcObjectDelete(WXID id)
{
  wxObject *obj = (wxObject *)wxGetTypedObject(id, CLASSINFO(wxObject));
  if (!obj)
  {
    ExtensionError << "Error in object-delete: could not find object " << id << ".\n";
    return FALSE;
  }
  delete obj;
  return TRUE;
}

#ifdef wx_msw
// MCI (multimedia command interface) string function. Returns
// NULL if no error, otherwise a text description of the error.
char *wxcMCISendString(char *command)
{
  DWORD err = mciSendString(command, NULL, 0, 0);
  if (err == 0)
    return NULL;

  Bool succ = mciGetErrorString(err, wxBuffer, 1000);
  if (!succ)
    return "Could not get error string";

  return wxBuffer;
}
#endif

void wxcBell(void)
{
  wxBell();
}

// Finding windows
WXID wxcFindWindowByName(char *title, WXID parent)
{
  wxWindow *parentWin = NULL;
  if (parent != 0)
  {
    parentWin = (wxWindow *)wxGetTypedObject(parent, CLASSINFO(wxWindow));
    if (!parentWin)
    {
      ExtensionError << "Error in find-window-by-name: could not find parent window " << parent << ".\n";
      return 0;
    }
  }
  return (WXID) wxFindWindowByName(title, parentWin);
}

WXID wxcFindWindowByLabel(char *label, WXID parent)
{
  wxWindow *parentWin = NULL;
  if (parent != 0)
  {
    parentWin = (wxWindow *)wxGetTypedObject(parent, CLASSINFO(wxWindow));
    if (!parentWin)
    {
      ExtensionError << "Error in find-window-by-label: could not find parent window " << parent << ".\n";
      return 0;
    }
  }
  return (WXID) wxFindWindowByLabel(label, parentWin);
}

/*
 * Some Windows-specific functions for manipulating
 * external application windows
 */

#ifdef wx_msw
WXID wxcHWNDFind(char *title)
{
  return (WXID)::FindWindow(NULL, title);
}

Bool wxcHWNDIconize(WXID hWnd, Bool iconize)
{
  if (hWnd)
  {
    if (iconize)
      ::ShowWindow((HWND)hWnd, SW_SHOWMINIMIZED);
    else
      ::ShowWindow((HWND)hWnd, SW_RESTORE);
    return TRUE;
  }
  else
    return FALSE;
}

Bool wxcHWNDSendMessage(WXID hWnd, long msg, long wParam, long lParam)
{
  if (hWnd)
  {
    ::SendMessage((HWND)hWnd, (UINT)msg, (WPARAM)wParam, (LPARAM)lParam);
    return TRUE;
  }
  else
    return FALSE;
}

Bool wxcHWNDShow(WXID hWnd, Bool show)
{
  if (hWnd)
  {
    if (show)
    {
      ::ShowWindow((HWND)hWnd, SW_SHOW);
      return ::BringWindowToTop((HWND)hWnd);
    }
    else
      return ::ShowWindow((HWND)hWnd, SW_HIDE);
    return TRUE;
  }
  else
    return FALSE;
}

Bool wxcHWNDMove(WXID hWnd, int x, int y, int w, int h, Bool repaint)
{
  if (hWnd)
  {
    return ::MoveWindow((HWND)hWnd, x, y, w, h, repaint);
  }
  else
    return FALSE;
}

Bool wxcHWNDQuit(WXID hWnd)
{
  if (hWnd)
  {
    ::SendMessage((HWND)hWnd, WM_CLOSE, 0, 0);
    return TRUE;
  }
  else
    return FALSE;
}

Bool wxcHWNDRefresh(WXID hWnd, Bool eraseBackground)
{
  if (hWnd)
  {
    ::InvalidateRect((HWND)hWnd, NULL, eraseBackground);
    return TRUE;
  }
  else
    return FALSE;
}

// Shell execute - valid ops are "print", "open", "explore"
Bool wxcShellExecute(char* op, char* file, char* params, char* defaultDir, char* show)
{
	if (!op)
		return FALSE;
	if (!file)
		return FALSE;

	int iShow = SW_SHOW;
	if (show)
	{
		if (strcmp(show, "show") == 0)
			iShow = SW_SHOW;
		else if (strcmp(show, "hide") == 0)
			iShow = SW_HIDE;
		else if (strcmp(show, "maximize") == 0)
			iShow = SW_MAXIMIZE;
		else if (strcmp(show, "minimize") == 0)
			iShow = SW_MINIMIZE;
		else if (strcmp(show, "maximize") == 0)
			iShow = SW_MAXIMIZE;
		else if (strcmp(show, "restore") == 0)
			iShow = SW_RESTORE;
#if WIN95
		else if (strcmp(show, "showdefault") == 0)
			iShow = SW_SHOWDEFAULT;
#endif
		else if (strcmp(show, "showmaximized") == 0)
			iShow = SW_SHOWMAXIMIZED;
		else if (strcmp(show, "showminimized") == 0)
			iShow = SW_SHOWMINIMIZED;
		else if (strcmp(show, "showminnoactive") == 0)
			iShow = SW_SHOWMINNOACTIVE;
		else if (strcmp(show, "showna") == 0)
			iShow = SW_SHOWNA;
		else if (strcmp(show, "shownoactivate") == 0)
			iShow = SW_SHOWNOACTIVATE;
		else if (strcmp(show, "shownormal") == 0)
			iShow = SW_SHOWNORMAL;
	}
	if (params && (strcmp(params, "") == 0))
		params = NULL;

	if (defaultDir && (strcmp(defaultDir, "") == 0))
		defaultDir = NULL;
    HANDLE ret = ::ShellExecute((HWND) wxTheApp->GetTopWindow()->GetHWND(),
		op, file, params, defaultDir, iShow);
    if ((int)ret <= 32)
    {
		return FALSE;
	}
	else
		return TRUE;
}

#endif

// Create app
WXID wxcAppCreate(void)
{
  if (!wxGetTypedObject ((long)wxTheApp, CLASSINFO(wxApp)))
    wxStoreThingInTable ((long) wxTheApp, wxTheApp);
  return (WXID) wxTheApp;
}

void wxcAppSetShowFrameOnInit(WXID id, Bool flag)
{
  wxApp *app = NULL;
  if (id > 0)
    app = (wxApp *)wxGetTypedObject(id, CLASSINFO(wxApp));
  if (!app)
    app = wxTheApp;
  if (!app)
    return;
      
  app->SetShowFrameOnInit(flag);
}

Bool wxcAppGetShowFrameOnInit(WXID id)
{
  wxApp *app = NULL;
  if (id > 0)
    app = (wxApp *)wxGetTypedObject(id, CLASSINFO(wxApp));
  if (!app)
    app = wxTheApp;
  if (!app)
    return FALSE;
      
  return app->GetShowFrameOnInit();
}

/*
 * Resource table functions
 */
 
wxResourceTable wxExtensionResourceTable;
 
Bool wxcLoadResourceFile(char *filename)
{
  return wxExtensionResourceTable.ParseResourceFile(filename);
}

void wxcClearResources(void)
{
  wxExtensionResourceTable.ClearTable();
}
